<?php

namespace BackOfficeOptimizer\Providers;

use BackOfficeOptimizer\Exceptions\BackOfficeException;

class DataProvider
{
    protected $connection;
    protected $novoCodigo;
    protected $servConfService;

    public function __construct($connection, $novoCodigo, $servConfService)
    {
        $this->connection = $connection;
        $this->novoCodigo = $novoCodigo;
        $this->servConfService = $servConfService;
    }

    private function insertCenario($param)
    {
        $date = new \DateTime();
        $paramObj = [
            "CDOPERADOR" => $param["CDOPERADOR"],
            "JSONCENARIO" => $param["JSONCENARIO"],
            "JSONCARDAPIO" => $param["JSONCARDAPIO"],
            "VRPUCENARIO" => $param["VRPUCENARIO"],
            "DTCENARIOCARD" => $date->format('d-m-Y'),
        ];

        if($param["NRCENARIOCARD"]){
            $paramObj['NRCENARIOCARDORIGINAL'] = $param["NRCENARIOCARD"];
        }
        $resultItGeracaoCard = null;
        if(isset($paramObj['NRCENARIOCARDORIGINAL']) && $paramObj['NRCENARIOCARDORIGINAL']){
            $sql = "SELECT NRCENARIOCARD, NRCENARIOCARDORIGINAL FROM CENARIOCARD WHERE NRCENARIOCARD = :NRCENARIOCARD AND NRCENARIOCARDORIGINAL IS NOT NULL";
            $cenData = $this->execOutSql(array("NRCENARIOCARD" => $param["NRCENARIOCARD"],), $sql);
            if(sizeof($cenData) > 0){
                $paramObj['NRCENARIOCARD'] = $cenData[0]['NRCENARIOCARD'];
                $paramObj['NRCENARIOCARDORIGINAL'] = $cenData[0]['NRCENARIOCARDORIGINAL'];
                
                $sql = "UPDATE CENARIOCARD SET DTCENARIOCARD = :DTCENARIOCARD, CDOPERADOR = :CDOPERADOR, JSONCARDAPIO = :JSONCARDAPIO, VRPUCENARIO = :VRPUCENARIO, JSONCENARIO = :JSONCENARIO WHERE NRCENARIOCARD = :NRCENARIOCARD AND NRCENARIOCARDORIGINAL = :NRCENARIOCARDORIGINAL";
                $cenData = $this->execInSql($paramObj, $sql);
            }else{
                $paramObj["CDFILIAL"] = $param["CDFILIAL"];
                $paramObj["CDSERVFILI"] = $param["CDSERVFILI"];
                $paramObj["DTMESCARD"] = $param["DTMESCARD"];
                $paramObj["QTCOMENSAIS"] = $param["QTCOMENSAIS"];
                $paramObj["IDTIPORESTRICAO"] = $param["IDTIPORESTRICAO"];
                $paramObj["CDTIPOCUSTO"] = $param["NRTIPOCUSTO"];
                $paramObj["DTREFECUSTMES"] = $param["DTREFECUSTO"];
                $paramObj["VRCUSTPADR"] = $param["VRCUSTPADR"];
                $paramObj["NRORG"] = $param["NRORG"];

                $paramObj['NRCENARIOCARD'] = $this->novoCodigo->getCodeByTableName('CENARIOCARD', $param["NRORG"]);
                $this->connection->insert('CENARIOCARD', $paramObj);
            }
            $resultItGeracaoCard = $this->execOutSql([
                'P_NRCENARIOCARD' => $paramObj['NRCENARIOCARDORIGINAL']
            ], "SELECT NRGERACAOCARD, NRITGERACAOCARD, NRCENARIOPUBLICAR FROM ITGERACAOCARD WHERE NRCENARIOCARD = :P_NRCENARIOCARD");

        }else{
            $paramObj["CDFILIAL"] = $param["CDFILIAL"];
            $paramObj["CDSERVFILI"] = $param["CDSERVFILI"];
            $paramObj["DTMESCARD"] = $param["DTMESCARD"];
            $paramObj["QTCOMENSAIS"] = $param["QTCOMENSAIS"];
            $paramObj["IDTIPORESTRICAO"] = $param["IDTIPORESTRICAO"];
            $paramObj["CDTIPOCUSTO"] = $param["NRTIPOCUSTO"];
            $paramObj["DTREFECUSTMES"] = $param["DTREFECUSTO"];
            $paramObj["VRCUSTPADR"] = $param["VRCUSTPADR"];
            $paramObj["NRCENARIOVINCULO"] = $param["NRCENARIOVINCULO"];
            $paramObj["NRORG"] = $param["NRORG"];

            $paramObj['NRCENARIOCARD'] = $this->novoCodigo->getCodeByTableName('CENARIOCARD', $param["NRORG"]);
            $this->connection->insert('CENARIOCARD', $paramObj);
        }
        $response = [
            "NRCENARIOCARD" => $paramObj['NRCENARIOCARD']
        ];
        if(!empty($resultItGeracaoCard)){
            list($itgeracaocard) = $resultItGeracaoCard;
            $response['NRGERACAOCARD'] = $itgeracaocard['NRGERACAOCARD'];
            $response['NRITGERACAOCARD'] = $itgeracaocard['NRITGERACAOCARD'];
            $response['NRCENARIOPUBLICAR'] = $itgeracaocard['NRCENARIOPUBLICAR'];
        }
        return $response;
    }

    private function execInSql($paramReq, $from)
    {
        if (is_array($paramReq) && isset($paramReq[0])) {
            for ($i = 0; $i < sizeof($paramReq); $i++) {
                $curParamReq = $paramReq[$i];
                foreach ($curParamReq as $key => $value) {
                    $params[$key] = $value;
                }
                $statement = $this->connection->prepare($from);
                $res = $statement->execute($params);
            }
        } else if (!empty($paramReq)) {
            $params = [];
            foreach ($paramReq as $key => $value) {
                $params[$key] = $value;
            }
            $statement = $this->connection->prepare($from);
            $res = $statement->execute($params);
        }
    }

    private function execOutSql($paramReq, $from)
    {
        $params = array();

        foreach ($paramReq as $key => $value) {
            $params[$key] = $value;
        }

        $statement = $this->connection->prepare($from);
        $statement->execute($params);
        return $statement->fetchAll();
    }

    private function updateParamPrat($paramReq, $CDOPERADOR)
    {
        $this->connection->beginTransaction();
        try{
            foreach ($paramReq as $paramReqIt) {
                if ((isset($paramReqIt) && $paramReqIt["PRATO_PARAMETRIZADO"] == "S") ||
                    $paramReqIt["PRATO_PARAMETRIZADO"] == "S"
                ) {
                    unset($paramReqIt["PRATO_PARAMETRIZADO"]);
                    $sql = "UPDATE PARAMPRATFILIAL SET IDCOMPLEXIDADE = :IDCOMPLEXIDADE, IDPREPREPARO = :IDPREPREPARO, IDCOR = :IDCOR, NRINGREDIENTEPRINC1 = :NRINGREDIENTEPRINC1, NRINGREDIENTEPRINC2 = :NRINGREDIENTEPRINC2, CDAPRERECE = :CDAPRERECE, NRFORMAPREPARO = :NRFORMAPREPARO, NRATRATIVIDADEPRATO = :NRATRATIVIDADEPRATO WHERE CDFILIAL = :CDFILIAL AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND NRORG =:NRORG";
                    $this->execInSql($paramReqIt, $sql);
                } else {
                    unset($paramReqIt["PRATO_PARAMETRIZADO"]);
                    $insertData = $paramReqIt;
                    $insertData["IDCOMPLEXIDADE"] = isset($insertData["IDCOMPLEXIDADE"]) ? $insertData["IDCOMPLEXIDADE"] : "BAIXA";
                    $insertData["IDPREPREPARO"] = isset($insertData["IDPREPREPARO"]) ? $insertData["IDPREPREPARO"] : "N";
                    $insertData["CDOPERADOR"] = $CDOPERADOR;
                    $sql = "INSERT INTO PARAMPRATFILIAL ( IDCOMPLEXIDADE, IDPREPREPARO, IDCOR, NRINGREDIENTEPRINC1, NRINGREDIENTEPRINC2, CDAPRERECE, NRFORMAPREPARO, NRATRATIVIDADEPRATO, CDFILIAL, CDFILAUXPRAT, CDPRATO, NRORG, CDOPERINCLUSAO) VALUES ( :IDCOMPLEXIDADE, :IDPREPREPARO, :IDCOR, :NRINGREDIENTEPRINC1, :NRINGREDIENTEPRINC2, :CDAPRERECE, :NRFORMAPREPARO, :NRATRATIVIDADEPRATO, :CDFILIAL, :CDFILAUXPRAT, :CDPRATO, :NRORG, :CDOPERADOR)";
                    $this->execInSql(array($insertData), $sql);
                }
            }
            $this->connection->commit();
        } catch(\Exception $e){
            $this->connection->rollBack();
            throw $e;
        }
    }
    
    private function updateOrderRule($paramReq){
        $this->connection->beginTransaction();
        try{
            if(isset($paramReq["NROTMREGRAFLEX_ORI"])){
                $sql = "SELECT NRORDEMREGRA FROM OTMREGRAFLEX WHERE NROTMREGRAFLEX = :NROTMREGRAFLEX_ORI";
                $nrOrdemRegraOri = $this->execOutSql(["NROTMREGRAFLEX_ORI" => $paramReq["NROTMREGRAFLEX_ORI"]], $sql)[0]["NRORDEMREGRA"];
                
                $slqUpdOri = "UPDATE OTMREGRAFLEX SET NRORDEMREGRA = (SELECT NRORDEMREGRA FROM OTMREGRAFLEX WHERE NROTMREGRAFLEX = :NROTMREGRAFLEX_DEST) WHERE NROTMREGRAFLEX = :NROTMREGRAFLEX_ORI";
                $this->execInSql(["NROTMREGRAFLEX_DEST" => $paramReq["NROTMREGRAFLEX_DEST"], "NROTMREGRAFLEX_ORI" => $paramReq["NROTMREGRAFLEX_ORI"]], $slqUpdOri);
                
                $sqlUpdDest = "UPDATE OTMREGRAFLEX SET NRORDEMREGRA = :NRORDEMREGRA WHERE NROTMREGRAFLEX = :NROTMREGRAFLEX_DEST";
                $this->execInSql(["NRORDEMREGRA" => $nrOrdemRegraOri, "NROTMREGRAFLEX_DEST" => $paramReq["NROTMREGRAFLEX_DEST"]], $sqlUpdDest);
            }
            $this->connection->commit();
        } catch(\Exception $e){
            $this->connection->rollBack();
            throw $e;
        }
    }

    private function delPlateRegister($params){
        $sql = "DELETE FROM ITGRUPOPRATOSERV WHERE NRITGRUPOPRATOSERV = :NRITGRUPOPRATOSERV";
        $this->execInSql($params, $sql);
    }

    private function updatePlateGroupStandard($paramReq)
    {
        $plateGroupStandardData = $paramReq[0];
        $updatePlateGroupStandardTable = "UPDATE PADGRUPOPRATO SET NMPADGRUPOPRATO = :NMPADGRUPOPRATO WHERE CDPADGRUPOPRATO = :CDPADGRUPOPRATO";
        
        $this->execInSql([
            "NMPADGRUPOPRATO" => $plateGroupStandardData["NMPADGRUPOPRATO"],
            "CDPADGRUPOPRATO" => $plateGroupStandardData["CDPADGRUPOPRATO"]
        ], $updatePlateGroupStandardTable);
        
        $updatePlateGroupTable = "UPDATE GRUPOPRATO SET NMGRUPOPRATO = :NMGRUPOPRATO WHERE CDGRUPOPRATO = :CDGRUPOPRATO AND IDTIPORESTRICAO = :IDTIPORESTRICAO";

        $this->execInSql([
            "NMGRUPOPRATO" => $plateGroupStandardData["NMPADGRUPOPRATO"],
            "CDGRUPOPRATO" => $plateGroupStandardData["CDPADGRUPOPRATO"],
            "IDTIPORESTRICAO" => $plateGroupStandardData["IDTIPORESTRICAO"]
        ], $updatePlateGroupTable);
    }

    private function deleteItCombGrupPrat($paramReq)
    {
        $countSql = "SELECT COUNT(*) AS COUNT FROM ITCOMBGRUPOTPPRAT WHERE NRCOMBGRUPOTPPRAT = :NRCOMBGRUPOTPPRAT";
        $deleteItCombSql = "DELETE FROM ITCOMBGRUPOTPPRAT WHERE NRITCOMBGRUPOTPPRAT = :NRITCOMBGRUPOTPPRAT";
        $deleteCombSql = "DELETE FROM COMBGRUPOTPPRAT WHERE NRCOMBGRUPOTPPRAT = :NRCOMBGRUPOTPPRAT";
        foreach ($paramReq as $combinationData) {
            $childrenCount = $this->execOutSql(["NRCOMBGRUPOTPPRAT" => $combinationData["NRCOMBGRUPOTPPRAT"]], $countSql)[0];
            $childrenCount = $childrenCount["COUNT"];
            $this->execInSql([["NRITCOMBGRUPOTPPRAT" => $combinationData["NRITCOMBGRUPOTPPRAT"]]], $deleteItCombSql);
            if ($childrenCount == 1) {
                $this->execInSql([["NRCOMBGRUPOTPPRAT" => $combinationData["NRCOMBGRUPOTPPRAT"]]], $deleteCombSql);
            }
        }
    }
    
    private function deleteItPlateGroupFrequency($paramReq)
    {
        $countSql = "SELECT COUNT(*) AS COUNT FROM ITFREQGRPRATMES WHERE NRFREQGRPRATMES = :NRFREQGRPRATMES";
        $deleteItFreqSql = "DELETE FROM ITFREQGRPRATMES WHERE NRITFREQGRPRATMES = :NRITFREQGRPRATMES";
        $deleteFreqSql = "DELETE FROM FREQGRPRATMES WHERE NRFREQGRPRATMES = :NRFREQGRPRATMES";
        foreach ($paramReq as $combinationData) {
            $childrenCount = $this->execOutSql(["NRFREQGRPRATMES" => $combinationData["NRFREQGRPRATMES"]], $countSql)[0];
            $childrenCount = $childrenCount["COUNT"];
            $this->execInSql([["NRITFREQGRPRATMES" => $combinationData["NRITFREQGRPRATMES"]]], $deleteItFreqSql);
            if ($childrenCount == 1) {
                $this->execInSql([["NRFREQGRPRATMES" => $combinationData["NRFREQGRPRATMES"]]], $deleteFreqSql);
            }
        }
    }
    private function deleteItPlateGroupProductFrequency($paramReq)
    {
        $countSql = "SELECT COUNT(*) AS COUNT FROM ITFREQPRODGRPRAT WHERE NRFREQPRODGRPRAT = :NRFREQPRODGRPRAT";
        $deleteItFreqSql = "DELETE FROM ITFREQPRODGRPRAT WHERE NRITFREQPRODGRPRAT = :NRITFREQPRODGRPRAT";
        $deleteFreqSql = "DELETE FROM FREQPRODGRPRAT WHERE NRFREQPRODGRPRAT = :NRFREQPRODGRPRAT";
        foreach ($paramReq as $combinationData) {
            $childrenCount = $this->execOutSql(["NRFREQPRODGRPRAT" => $combinationData["NRFREQPRODGRPRAT"]], $countSql)[0];
            $childrenCount = $childrenCount["COUNT"];
            $this->execInSql([["NRITFREQPRODGRPRAT" => $combinationData["NRITFREQPRODGRPRAT"]]], $deleteItFreqSql);
            if ($childrenCount == 1) {
                $this->execInSql([["NRFREQPRODGRPRAT" => $combinationData["NRFREQPRODGRPRAT"]]], $deleteFreqSql);
            }
        }
    }

    private function deleteItPlateFrequency($paramReq)
    {
        $countSql = "SELECT COUNT(*) AS COUNT FROM ITFREQPRATMES WHERE NRFREQPRATMES = :NRFREQPRATMES";
        $deleteItFreqSql = "DELETE FROM ITFREQPRATMES WHERE NRITFREQPRATMES = :NRITFREQPRATMES";
        $deleteFreqSql = "DELETE FROM FREQPRATMES WHERE NRFREQPRATMES = :NRFREQPRATMES";
        foreach ($paramReq as $combinationData) {
            $childrenCount = $this->execOutSql(["NRFREQPRATMES" => $combinationData["NRFREQPRATMES"]], $countSql)[0];
            $childrenCount = $childrenCount["COUNT"];
            $this->execInSql([["NRITFREQPRATMES" => $combinationData["NRITFREQPRATMES"]]], $deleteItFreqSql);
            if ($childrenCount == 1) {
                $this->execInSql([["NRFREQPRATMES" => $combinationData["NRFREQPRATMES"]]], $deleteFreqSql);
            }
        }
    }

    private function deleteProductGroupPlateRestrictionValidity($paramReq){
        $deleteParams = $paramReq[0];
        $force = isset($deleteParams["force"]) && $deleteParams["force"] == "true" ? true : false;
        $hasOtherChildrenSql = "SELECT S.CDSERVFILI || ' - ' ||S.NMSERVICO AS SERVICO FROM RESTPRODGR R INNER JOIN SERVICO S ON S.CDSERVFILI = R.CDSERVFILI AND S.CDFILIAL = :CDFILIAL WHERE R.NRVIGENCIA = :NRVIGENCIA AND R.CDSERVFILI <> :CDSERVFILI ORDER BY S.CDSERVFILI";
        $deleteSql = "DELETE FROM VIGRESTPRODGR WHERE NRVIGENCIA = :NRVIGENCIA";
        if(!$force){
            $findChildrenParams = [
                "NRVIGENCIA" => $deleteParams["NRVIGENCIA"],
                "CDSERVFILI" => $deleteParams["CDSERVFILI"],
                "CDFILIAL" => $deleteParams["CDFILIAL"]
            ];
            $children = $this->execOutSql($findChildrenParams, $hasOtherChildrenSql);
            if(sizeof($children) > 0){
                return $children;
            } else {
                $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
                return [];
            }
        } else {
            $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
            return [];
        }
        
    }
    
    private function deletePlateGroupProductFrequencyValidity($paramReq){
        $deleteParams = $paramReq[0];
        $force = isset($deleteParams["force"]) && $deleteParams["force"] == "true" ? true : false;
        $hasOtherChildrenSql = "SELECT S.CDSERVFILI || ' - ' ||S.NMSERVICO AS SERVICO FROM FREQPRODGRPRAT F INNER JOIN SERVICO S ON S.CDSERVFILI = F.CDSERVFILI AND S.CDFILIAL = F.CDFILIAL WHERE F.NRVIGENCIA = :NRVIGENCIA AND F.CDSERVFILI <> :CDSERVFILI ORDER BY S.CDSERVFILI";
        $deleteSql = "DELETE FROM VIGFREQPRODGRPRAT WHERE NRVIGENCIA = :NRVIGENCIA";
        if(!$force){
            $findChildrenParams = [
                "NRVIGENCIA" => $deleteParams["NRVIGENCIA"],
                "CDSERVFILI" => $deleteParams["CDSERVFILI"]
            ];
            $children = $this->execOutSql($findChildrenParams, $hasOtherChildrenSql);
            if(sizeof($children) > 0){
                return $children;
            } else {
                $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
                return [];
            }
        } else {
            $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
            return [];
        }
        
    }
    
    private function deletePlateGroupValidity($paramReq){
        $deleteParams = $paramReq[0];
        $force = isset($deleteParams["force"]) && $deleteParams["force"] == "true" ? true : false;
        $hasOtherChildrenSql = "SELECT S.CDSERVFILI || ' - ' ||S.NMSERVICO AS SERVICO FROM FREQGRPRATMES F INNER JOIN SERVICO S ON S.CDSERVFILI = F.CDSERVFILI AND S.CDFILIAL = F.CDFILIAL WHERE F.NRVIGENCIA = :NRVIGENCIA AND F.CDSERVFILI <> :CDSERVFILI ORDER BY S.CDSERVFILI";
        $deleteSql = "DELETE FROM VIGFREQGRPRATMES WHERE NRVIGENCIA = :NRVIGENCIA";
        if(!$force){
            $findChildrenParams = [
                "NRVIGENCIA" => $deleteParams["NRVIGENCIA"],
                "CDSERVFILI" => $deleteParams["CDSERVFILI"]
            ];
            $children = $this->execOutSql($findChildrenParams, $hasOtherChildrenSql);
            if(sizeof($children) > 0){
                return $children;
            } else {
                $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
                return [];
            }
        } else {
            $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
            return [];
        }
        
    }

    private function deletePlateValidity($paramReq){
        $deleteParams = $paramReq[0];
        $force = isset($deleteParams["force"]) && $deleteParams["force"] == "true" ? true : false;
        $hasOtherChildrenSql = "SELECT S.CDSERVFILI || ' - ' ||S.NMSERVICO AS SERVICO FROM FREQPRATMES F INNER JOIN SERVICO S ON S.CDSERVFILI = F.CDSERVFILI AND S.CDFILIAL = F.CDFILIAL WHERE F.NRVIGENCIA = :NRVIGENCIA AND F.CDSERVFILI <> :CDSERVFILI ORDER BY S.CDSERVFILI";
        $deleteSql = "DELETE FROM VIGFREQPRATMES WHERE NRVIGENCIA = :NRVIGENCIA";
        if(!$force){
            $findChildrenParams = [
                "NRVIGENCIA" => $deleteParams["NRVIGENCIA"],
                "CDSERVFILI" => $deleteParams["CDSERVFILI"]
            ];
            $children = $this->execOutSql($findChildrenParams, $hasOtherChildrenSql);
            if(sizeof($children) > 0){
                return $children;
            } else {
                $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
                return [];
            }
        } else {
            $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
            return [];
        }
        
    }
    
    private function deleteWeekdayUsageValidity($paramReq){
        $deleteParams = $paramReq[0];
        $force = isset($deleteParams["force"]) && $deleteParams["force"] == "true" ? true : false;
        $hasOtherChildrenSql = "SELECT DISTINCT SERVICO FROM(
            SELECT S.CDSERVFILI || ' - ' ||S.NMSERVICO AS SERVICO
              FROM UTILPRATOSERV U INNER JOIN SERVICO S ON S.CDSERVFILI = U.CDSERVFILI AND S.CDFILIAL = :CDFILIAL
             WHERE U.NRVIGENCIA = :NRVIGENCIA AND U.CDSERVFILI <> :CDSERVFILI)
            ORDER BY SERVICO";
        $deleteChildrenSql = "DELETE FROM UTILPRATOSERV WHERE NRVIGENCIA = :NRVIGENCIA";
        $deleteSql = "DELETE FROM VIGUTILPRATOSERV WHERE NRVIGENCIA = :NRVIGENCIA";
        if(!$force){
            $findChildrenParams = [
                "NRVIGENCIA" => $deleteParams["NRVIGENCIA"],
                "CDSERVFILI" => $deleteParams["CDSERVFILI"],
                "CDFILIAL" => $deleteParams["CDFILIAL"]
            ];
            $children = $this->execOutSql($findChildrenParams, $hasOtherChildrenSql);
            if(sizeof($children) > 0){
                return $children;
            } else {
                $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteChildrenSql);
                $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
                return [];
            }
        } else {
            $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteChildrenSql);
            $this->execInSql(["NRVIGENCIA" => $deleteParams["NRVIGENCIA"]], $deleteSql);
            return [];
        }
        
    }

    private function _insertWithNewCode($params, $sql, $tablename, $keyname, $removeNRORG = true)
    {
        if (is_array($keyname)) {
            foreach ($keyname as $tname => $kname) {
                $id = $this->novoCodigo->getCodeByTableName($tname, $params["NRORG"]);
                $params[$kname] = $id;
            }
        } else {
            $id = $this->novoCodigo->getCodeByTableName($tablename, $params["NRORG"]);
            $params[$keyname] = $id;
        }
        if($removeNRORG){
            unset($params["NRORG"]);
        }
        $this->execInSql($params, $sql);
        return $id;
    }

    private function insertItCombGrupPrat($paramReq)
    {
        $findParentKeySql = "SELECT NRCOMBGRUPOTPPRAT FROM COMBGRUPOTPPRAT WHERE CDGRTIPOPR = :CDGRTIPOPR AND (CDFILAUXPRAT = :CDFILAUXPRAT OR :CDFILAUXPRAT IS NULL) AND (CDPRATO = :CDPRATO OR :CDPRATO IS NULL) AND (NRGRUPOPRATOSERV = :NRGRUPOPRATOSERV OR :NRGRUPOPRATOSERV IS NULL) AND CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI";
        $combinationData = $paramReq[0];
        $parentSearchParams = array(
            "CDGRTIPOPR" => $combinationData["CDGRTIPOPR"],
            "CDFILAUXPRAT" => $combinationData["CDFILAUXPRAT"],
            "CDSERVFILI" => $combinationData["CDSERVFILI"],
            "CDPRATO" => $combinationData["CDPRATO"],
            "CDFILIAL" => $combinationData["CDFILIAL"],
            "NRGRUPOPRATOSERV" => $combinationData["NRGRUPOPRATOSERV"]
        );
        $parentKeySearch = $this->execOutSql($parentSearchParams, $findParentKeySql);
        $NRCOMBGRUPOTPPRAT = null;
        if (sizeof($parentKeySearch) > 0) {
            $NRCOMBGRUPOTPPRAT = $parentKeySearch[0]["NRCOMBGRUPOTPPRAT"];
        } else {
            $insertParams = [
                "CDGRTIPOPR" => $combinationData["CDGRTIPOPR"],
                "CDFILAUXPRAT" => $combinationData["CDFILAUXPRAT"],
                "CDPRATO" => $combinationData["CDPRATO"],
                "NRGRUPOPRATOSERV" => $combinationData["NRGRUPOPRATOSERV"],
                "CDFILIAL" => $combinationData["CDFILIAL"],
                "CDOPERADOR" => $combinationData["CDOPERADOR"],
                "CDSERVFILI" => $combinationData["CDSERVFILI"],
                "NRORG" => $combinationData["NRORG"]
            ];
            $insertParentSql = "INSERT INTO COMBGRUPOTPPRAT (NRCOMBGRUPOTPPRAT, CDGRTIPOPR, CDFILAUXPRAT, CDPRATO, CDFILIAL, CDOPERINCLUSAO, CDSERVFILI, NRGRUPOPRATOSERV) VALUES (:NRCOMBGRUPOTPPRAT, :CDGRTIPOPR, :CDFILAUXPRAT, :CDPRATO, :CDFILIAL, :CDOPERADOR, :CDSERVFILI, :NRGRUPOPRATOSERV)";
            $NRCOMBGRUPOTPPRAT = $this->_insertWithNewCode($insertParams, $insertParentSql, "COMBGRUPOTPPRAT", "NRCOMBGRUPOTPPRAT");
        }
        $insertParams = [
            "NRCOMBGRUPOTPPRAT" => $NRCOMBGRUPOTPPRAT,
            "COMB_CDGRTIPOPR" => $combinationData["COMB_CDGRTIPOPR"],
            "COMB_CDFILAUXPRAT" => $combinationData["COMB_CDFILAUXPRAT"],
            "COMB_CDPRATO" => $combinationData["COMB_CDPRATO"],
            "COMB_NRGRUPOPRATOSERV" => $combinationData["COMB_NRGRUPOPRATOSERV"],
            "COMB_CDTIPOPRAT" => $combinationData["COMB_CDTIPOPRAT"],
            "IDCOMBINACAO" => $combinationData["IDCOMBINACAO"],
            "PERCENTREDUCAO" => $combinationData["PERCENTREDUCAO"],
            "CDOPERADOR" => $combinationData["CDOPERADOR"],
            "NRORG" => $combinationData["NRORG"]
        ];
        $insertChildSql = "INSERT INTO ITCOMBGRUPOTPPRAT (NRITCOMBGRUPOTPPRAT, NRCOMBGRUPOTPPRAT, CDGRTIPOPR, CDFILAUXPRAT, CDPRATO, IDCOMBINACAO, PERCENTREDUCAO, CDOPERINCLUSAO, NRGRUPOPRATOSERV, CDTIPOPRAT) VALUES (:NRITCOMBGRUPOTPPRAT, :NRCOMBGRUPOTPPRAT, :COMB_CDGRTIPOPR, :COMB_CDFILAUXPRAT, :COMB_CDPRATO, :IDCOMBINACAO, :PERCENTREDUCAO, :CDOPERADOR, :COMB_NRGRUPOPRATOSERV, :COMB_CDTIPOPRAT)";
        $newID = $this->_insertWithNewCode($insertParams, $insertChildSql, "ITCOMBGRUPOTPPRAT", "NRITCOMBGRUPOTPPRAT");
        return [$newID, $NRCOMBGRUPOTPPRAT];

    }

    private function insertPlateGroupFrequency($paramReq){
        $plateGroupFrequencyData = $paramReq[0];
        $NRFREQGRPRATMES = null;
        if(!isset($plateGroupFrequencyData["NRFREQGRPRATMES"]) || 
            $plateGroupFrequencyData["NRFREQGRPRATMES"] === null){
                $insertParams = [
                    "CDFILIAL" => $plateGroupFrequencyData["CDFILIAL"],
                    "CDSERVFILI" => $plateGroupFrequencyData["CDSERVFILI"],
                    "DTVIGENCIA" => $plateGroupFrequencyData["DTVIGENCIA"],
                    "NRVIGENCIA" => $plateGroupFrequencyData["NRVIGENCIA"],
                    "NRORG" => $plateGroupFrequencyData["NRORG"],
                    "CDOPERADOR" => $plateGroupFrequencyData["CDOPERADOR"]
                ];
                $insertParentSql = "INSERT INTO FREQGRPRATMES (NRFREQGRPRATMES, CDFILIAL, CDSERVFILI, NRVIGENCIA, DTVIGENCIA, CDOPERINCLUSAO)
                VALUES (:NRFREQGRPRATMES, :CDFILIAL, :CDSERVFILI, :NRVIGENCIA, :DTVIGENCIA, :CDOPERADOR)";
                $NRFREQGRPRATMES = $this->_insertWithNewCode($insertParams, $insertParentSql, "FREQGRPRATMES", "NRFREQGRPRATMES");
        } else {
            $NRFREQGRPRATMES = $plateGroupFrequencyData["NRFREQGRPRATMES"];
        }

        $insertParams = [
            "NRFREQGRPRATMES" => $NRFREQGRPRATMES,
            "NRMINFREQGRPRATMES" => $plateGroupFrequencyData["NRMINFREQGRPRATMES"],
            "NRMAXFREQGRPRATMES" => $plateGroupFrequencyData["NRMAXFREQGRPRATMES"],
            "NRMAXFREQGRPRATSEM" => $plateGroupFrequencyData["NRMAXFREQGRPRATSEM"],
            "NRGRUPOPRATOSERV" => $plateGroupFrequencyData["NRGRUPOPRATOSERV"],
            "NRORG" => $plateGroupFrequencyData["NRORG"],
            "CDOPERADOR" => $plateGroupFrequencyData["CDOPERADOR"]
        ];
        
        $insertChildSql = "INSERT INTO ITFREQGRPRATMES (NRITFREQGRPRATMES, NRFREQGRPRATMES, NRMINFREQGRPRATMES, NRMAXFREQGRPRATMES, NRMAXFREQGRPRATSEM, NRGRUPOPRATOSERV, CDOPERINCLUSAO) VALUES (:NRITFREQGRPRATMES, :NRFREQGRPRATMES, :NRMINFREQGRPRATMES, :NRMAXFREQGRPRATMES, :NRMAXFREQGRPRATSEM, :NRGRUPOPRATOSERV, :CDOPERADOR)";
        $newID = $this->_insertWithNewCode($insertParams, $insertChildSql, "ITFREQGRPRATMES", "NRITFREQGRPRATMES");
        return [$newID];
    }
    
    private function insertPlateGroupProductFrequency($paramReq){
        $plateGroupFrequencyData = $paramReq[0];
        $NRFREQPRODGRPRAT = null;
        if(!isset($plateGroupFrequencyData["NRFREQPRODGRPRAT"]) || 
            $plateGroupFrequencyData["NRFREQPRODGRPRAT"] === null){
                $insertParams = [
                    "CDFILIAL" => $plateGroupFrequencyData["CDFILIAL"],
                    "CDSERVFILI" => $plateGroupFrequencyData["CDSERVFILI"],
                    "DTVIGENCIA" => $plateGroupFrequencyData["DTVIGENCIA"],
                    "NRVIGENCIA" => $plateGroupFrequencyData["NRVIGENCIA"],
                    "NRORG" => $plateGroupFrequencyData["NRORG"],
                    "CDOPERADOR" => $plateGroupFrequencyData["CDOPERADOR"],
                ];
                $insertParentSql = "INSERT INTO FREQPRODGRPRAT (NRFREQPRODGRPRAT, CDFILIAL, CDSERVFILI, NRVIGENCIA, DTVIGENCIA, CDOPERINCLUSAO)
                VALUES (:NRFREQPRODGRPRAT, :CDFILIAL, :CDSERVFILI, :NRVIGENCIA, :DTVIGENCIA, :CDOPERADOR)";
                $NRFREQPRODGRPRAT = $this->_insertWithNewCode($insertParams, $insertParentSql, "FREQPRODGRPRAT", "NRFREQPRODGRPRAT");
        } else {
            $NRFREQPRODGRPRAT = $plateGroupFrequencyData["NRFREQPRODGRPRAT"];
        }

        $insertParams = [
            "NRFREQPRODGRPRAT" => $NRFREQPRODGRPRAT,
            "NRMINPRODGRPRAT" => $plateGroupFrequencyData["NRMINPRODGRPRAT"],
            "NRMAXPRODGRPRAT" => $plateGroupFrequencyData["NRMAXPRODGRPRAT"],
            "NRMAXPRODGRPRATSEM" => $plateGroupFrequencyData["NRMAXPRODGRPRATSEM"],
            "NRGRUPOPRATOSERV" => $plateGroupFrequencyData["NRGRUPOPRATOSERV"],
            "CDPRODUTO" => $plateGroupFrequencyData["CDPRODUTO"],
            "NRORG" => $plateGroupFrequencyData["NRORG"],
            "CDOPERADOR" => $plateGroupFrequencyData["CDOPERADOR"]
        ];
        
        $insertChildSql = "INSERT INTO ITFREQPRODGRPRAT (NRITFREQPRODGRPRAT, NRFREQPRODGRPRAT, NRMINPRODGRPRAT, NRMAXPRODGRPRAT, NRMAXPRODGRPRATSEM, NRGRUPOPRATOSERV, CDPRODUTO, CDOPERINCLUSAO) VALUES (:NRITFREQPRODGRPRAT, :NRFREQPRODGRPRAT, :NRMINPRODGRPRAT, :NRMAXPRODGRPRAT, :NRMAXPRODGRPRATSEM, :NRGRUPOPRATOSERV, :CDPRODUTO, :CDOPERADOR)";
        $newID = $this->_insertWithNewCode($insertParams, $insertChildSql, "ITFREQPRODGRPRAT", "NRITFREQPRODGRPRAT");
        return [$newID];
    }

    private function insertWeekdayUsage($paramReq){
        $weekdayUsageData = $paramReq[0];
        
        $insertSql = "INSERT INTO UTILPRATOSERV (NRUTILPRATOSERV, NRVIGENCIA, CDSERVFILI, NRGRUPOPRATOSERV, IDUTILPRATOSERVSEG, IDUTILPRATOSERVTER, IDUTILPRATOSERVQUA, IDUTILPRATOSERVQUI, IDUTILPRATOSERVSEX, IDUTILPRATOSERVSAB, IDUTILPRATOSERVDOM, CDOPERINCLUSAO, IDUTILPRATOSERVPRIM, IDUTILPRATOSERVSEM2, IDUTILPRATOSERVSEM3, IDUTILPRATOSERVSEM4, IDUTILPRATOSERVULT, IDUTILPRATOSERVFERI) VALUES (:NRUTILPRATOSERV, :NRVIGENCIA, :CDSERVFILI, :NRGRUPOPRATOSERV, :IDUTILPRATOSERVSEG, :IDUTILPRATOSERVTER, :IDUTILPRATOSERVQUA, :IDUTILPRATOSERVQUI, :IDUTILPRATOSERVSEX, :IDUTILPRATOSERVSAB, :IDUTILPRATOSERVDOM, :CDOPERADOR, :IDUTILPRATOSERVPRIM, :IDUTILPRATOSERVSEM2, :IDUTILPRATOSERVSEM3, :IDUTILPRATOSERVSEM4, :IDUTILPRATOSERVULT, :IDUTILPRATOSERVFERI)";
        $newID = $this->_insertWithNewCode($weekdayUsageData, $insertSql, "UTILPRATOSERV", "NRUTILPRATOSERV");
        return [$newID];
    }

    private function insertProductGroupPlateRestriction($paramReq){
        $productGroupPlateRestriction = $paramReq[0];
        
        $insertSql = "INSERT INTO RESTPRODGR (NRRESTPRODGR, CDFILIAL, CDSERVFILI, CDPRODUTO, NRGRUPOPRATOSERV, NRVIGENCIA, CDOPERINCLUSAO) VALUES (:NRRESTPRODGR, :CDFILIAL, :CDSERVFILI, :CDPRODUTO, :NRGRUPOPRATOSERV, :NRVIGENCIA, :CDOPERADOR)";
        $newID = $this->_insertWithNewCode($productGroupPlateRestriction, $insertSql, "RESTPRODGR", "NRRESTPRODGR");
        return [$newID];
    }

    private function insertPlateFrequency($paramReq){
        $plateFrequencyData = $paramReq[0];

        $getPlateGruopFrequencySQL = "SELECT NRMAXFREQGRPRATMES FROM(SELECT VG.NRVIGENCIA, VG.CDFILIAL, VG.DTVIGENCIA, VG.DESCRICAO, VG.DTVIGENCIAFIM, FR.NRFREQGRPRATMES, FR.CDSERVFILI, IT.NRMINFREQGRPRATMES, IT.NRMAXFREQGRPRATMES, IT.NRMAXFREQGRPRATSEM, IT.NRGRUPOPRATOSERV, CASE WHEN (VG.DTVIGENCIAFIM IS NOT NULL AND VG.DTVIGENCIAFIM >= :DTMESCARD AND VG.DTVIGENCIA <= :DTMESCARD) THEN 0 WHEN (VG.DTVIGENCIAFIM IS NOT NULL AND VG.DTVIGENCIAFIM <= :DTMESCARD) THEN 2 WHEN VG.DTVIGENCIA <= :DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQGRPRATMES VG INNER JOIN FREQGRPRATMES FR ON FR.NRVIGENCIA = VG.NRVIGENCIA INNER JOIN ITFREQGRPRATMES IT ON IT.NRFREQGRPRATMES = FR.NRFREQGRPRATMES INNER JOIN ITGRUPOPRATOSERV ITG ON ITG.NRGRUPOPRATOSERV = IT.NRGRUPOPRATOSERV WHERE VG.IDTIPORESTRICAO = :IDTIPORESTRICAO AND VG.CDFILIAL = :CDFILIAL AND FR.CDSERVFILI = :CDSERVFILI AND ITG.CDFILAUXPRAT = :CDFILAUXPRAT AND ITG.CDPRATO = :CDPRATO ORDER BY ORDEM, VG.DTVIGENCIA DESC) WHERE ROWNUM = 1";

       $plateGroupFrequencyData = $this->execOutSql([
            "DTMESCARD" => $plateFrequencyData["DTMESCARD"],
            "IDTIPORESTRICAO" => $plateFrequencyData["IDTIPORESTRICAO"],
            "CDFILIAL" => $plateFrequencyData["CDFILIAL"],
            "CDSERVFILI" => $plateFrequencyData["CDSERVFILI"],
            "CDFILAUXPRAT" => $plateFrequencyData["CDFILAUXPRAT"],
            "CDPRATO" => $plateFrequencyData["CDPRATO"]
        ], $getPlateGruopFrequencySQL);
        $maxFreqPratMes = intval($plateFrequencyData["NRMAXFREQPRATMES"]);
        if(count($plateGroupFrequencyData) < 0 || 
            $maxFreqPratMes <= intval($plateGroupFrequencyData[0]["NRMAXFREQGRPRATMES"])){

            $NRFREQPRATMES = null;
            if(!isset($plateFrequencyData["NRFREQPRATMES"]) || 
                $plateFrequencyData["NRFREQPRATMES"] === null){
                    $insertParams = [
                        "CDFILIAL" => $plateFrequencyData["CDFILIAL"],
                        "CDSERVFILI" => $plateFrequencyData["CDSERVFILI"],
                        "DTVIGENCIA" => $plateFrequencyData["DTVIGENCIA"],
                        "NRVIGENCIA" => $plateFrequencyData["NRVIGENCIA"],
                        "NRORG" => $plateFrequencyData["NRORG"],
                        "CDOPERADOR" => $plateFrequencyData["CDOPERADOR"]
                    ];
                    $insertParentSql = "INSERT INTO FREQPRATMES (NRFREQPRATMES, CDFILIAL, CDSERVFILI, NRVIGENCIA, DTVIGENCIA, CDOPERINCLUSAO)
                    VALUES (:NRFREQPRATMES, :CDFILIAL, :CDSERVFILI, :NRVIGENCIA, :DTVIGENCIA, :CDOPERADOR)";
                    $NRFREQPRATMES = $this->_insertWithNewCode($insertParams, $insertParentSql, "FREQPRATMES", "NRFREQPRATMES");
            } else {
                $NRFREQPRATMES = $plateFrequencyData["NRFREQPRATMES"];
            }

            $insertParams = [
                "NRFREQPRATMES" => $NRFREQPRATMES,
                "NRMINFREQPRATMES" => $plateFrequencyData["NRMINFREQPRATMES"],
                "NRMAXFREQPRATMES" => $plateFrequencyData["NRMAXFREQPRATMES"],
                "NRMAXFREQPRATSEM" => $plateFrequencyData["NRMAXFREQPRATSEM"],
                "CDFILAUXPRAT" => $plateFrequencyData["CDFILAUXPRAT"],
                "CDPRATO" => $plateFrequencyData["CDPRATO"],
                "NRORG" => $plateFrequencyData["NRORG"],
                "CDOPERADOR" => $plateFrequencyData["CDOPERADOR"]
            ];
            
            $insertChildSql = "INSERT INTO ITFREQPRATMES (NRITFREQPRATMES, NRFREQPRATMES, NRMINFREQPRATMES, NRMAXFREQPRATMES, NRMAXFREQPRATSEM, CDFILAUXPRAT, CDPRATO, CDOPERINCLUSAO) VALUES (:NRITFREQPRATMES, :NRFREQPRATMES, :NRMINFREQPRATMES, :NRMAXFREQPRATMES, :NRMAXFREQPRATSEM, :CDFILAUXPRAT, :CDPRATO, :CDOPERADOR)";
            $newID = $this->_insertWithNewCode($insertParams, $insertChildSql, "ITFREQPRATMES", "NRITFREQPRATMES");
            return [$newID];
        } else {
            $plateGroupMaxFrequency = intval($plateGroupFrequencyData[0]["NRMAXFREQGRPRATMES"]);
            
            $mensagem = $this->formatMsgTradu("FREQ_MAX_GRUPO",$plateGroupMaxFrequency);
            throw new BackOfficeException($mensagem);
        }
    }
    
    private function insertPlateValidity($paramReq){
        $validityData = $paramReq[0];
        $countValiditySql = "SELECT COUNT(NRVIGENCIA) AS COUNT FROM VIGFREQPRATMES WHERE IDTIPORESTRICAO = :IDTIPORESTRICAO AND CDFILIAL = :CDFILIAL AND DTVIGENCIA = :DTVIGENCIA";
        $validityCount = $this->execOutSql([
            "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
            "CDFILIAL" => $validityData["CDFILIAL"], 
            "DTVIGENCIA" => $validityData["DTVIGENCIA"]], 
        $countValiditySql)[0]["COUNT"];
        if($validityCount > 0){
            $mensagem = $this->formatMsgTradu("there_is_already_term_for_the_specified_period");
            throw new BackOfficeException($mensagem);
        } else {
            $insertValiditySql = "DECLARE V_NOVOCODIGO VARCHAR2(20); BEGIN NOVO_CODIGO('VIGFREQPRATMES', V_NOVOCODIGO); INSERT INTO VIGFREQPRATMES (NRVIGENCIA, CDFILIAL, DTVIGENCIA, DESCRICAO, DTVIGENCIAFIM, IDTIPORESTRICAO, CDDESCOBSFILIAL, CDOPERINCLUSAO) VALUES (V_NOVOCODIGO, :CDFILIAL, :DTVIGENCIA, :DESCRICAO, :DTVIGENCIAFIM, :IDTIPORESTRICAO, :CDDESCOBSFILIAL, :CDOPERADOR); END;";
            $params = [
                "CDFILIAL" => $validityData["CDFILIAL"],
                "DTVIGENCIA" => $validityData["DTVIGENCIA"],
                "DESCRICAO" => $validityData["DESCRICAO"],
                "DTVIGENCIAFIM" => $validityData["DTVIGENCIAFIM"],
                "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
                "CDDESCOBSFILIAL" => $validityData["CDDESCOBSFILIAL"],
                "CDOPERADOR" => $validityData["CDOPERADOR"]
            ];
            $this->execInSql($params, $insertValiditySql);
        }
    }

    private function newPlateGroup($params){
        $checkSql = "SELECT MAX(NRGRUPOPRATO) NRGRUPOPRATO FROM GRUPOPRATO WHERE CDGRUPOPRATO = :CDGRUPOPRATO AND CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO";
        
        $checkedResult = $this->execOutSql([
            'CDFILIAL'=>$params['CDFILIAL'],
            'CDGRUPOPRATO'=>$params['CDGRUPOPRATO'],
            'IDTIPORESTRICAO'=>$params['IDTIPORESTRICAO']
        ], $checkSql);

        if(!$checkedResult[0]['NRGRUPOPRATO']){
            $insertPlateGroup = "DECLARE V_NOVOCODIGO VARCHAR2(20); BEGIN NOVO_CODIGO('GRUPOPRATO', V_NOVOCODIGO); INSERT INTO GRUPOPRATO (NRGRUPOPRATO, CDFILIAL, CDGRUPOPRATO, NMGRUPOPRATO, IDTIPORESTRICAO, CDOPERINCLUSAO) VALUES (V_NOVOCODIGO, :CDFILIAL, :CDGRUPOPRATO, :NMGRUPOPRATO, :IDTIPORESTRICAO, :CDOPERADOR); END;";
            $this->execInSql([
                'CDFILIAL'=>$params['CDFILIAL'],
                'CDGRUPOPRATO'=>$params['CDGRUPOPRATO'],
                'NMGRUPOPRATO'=>$params['NMGRUPOPRATO'],
                'IDTIPORESTRICAO'=>$params['IDTIPORESTRICAO'],
                'CDOPERADOR'=>$params['CDOPERADOR']
            ], $insertPlateGroup);

            $checkedResult = $this->execOutSql([
                'CDFILIAL'=>$params['CDFILIAL'],
                'CDGRUPOPRATO'=>$params['CDGRUPOPRATO'],
                'IDTIPORESTRICAO'=>$params['IDTIPORESTRICAO']
            ], $checkSql);
        }

        $nrGrupPrat = $checkedResult[0]['NRGRUPOPRATO'];

        $id = $this->novoCodigo->getCodeByTableName('GRUPOPRATOSERV', $params["NRORG"], 20);
        $insertPlateGroupServ = "INSERT INTO GRUPOPRATOSERV (NRGRUPOPRATOSERV, NRGRUPOPRATO, NRAGRUPGRPRAT, CDSERVFILI, CDOPERINCLUSAO) VALUES (:NOVOCODIGO, :NRGRUPOPRATO, :NRAGRUPGRPRAT, :CDSERVFILI, :CDOPERADOR)";
        $this->execInSql([
            'NOVOCODIGO'=>$id,
            'NRGRUPOPRATO'=>$nrGrupPrat,
            'NRAGRUPGRPRAT'=>$params['NRAGRUPGRPRAT'],
            'CDSERVFILI'=>$params['CDSERVFILI'],
            'CDOPERADOR'=>$params['CDOPERADOR']
        ], $insertPlateGroupServ);

        return $id;
    }

    private function deletePlateGroup($params){
        $delGroupSql = "DELETE FROM GRUPOPRATOSERV WHERE NRGRUPOPRATOSERV = :NRGRUPOPRATOSERV";
        $delPlatesSql = "DELETE FROM ITGRUPOPRATOSERV WHERE NRGRUPOPRATOSERV = :NRGRUPOPRATOSERV";
        
        $this->execInSql([
            'NRGRUPOPRATOSERV'=>$params['NRGRUPOPRATOSERV']
        ], $delGroupSql);
        $this->execInSql([
            'NRGRUPOPRATOSERV'=>$params['NRGRUPOPRATOSERV']
        ], $delPlatesSql);
    }

    private function changePlateGroup($params){
        $updateGroup = "UPDATE GRUPOPRATOSERV SET NRAGRUPGRPRAT = :NRAGRUPGRPRAT WHERE NRGRUPOPRATOSERV = :NRGRUPOPRATOSERV";

        $this->execInSql([
            'NRGRUPOPRATOSERV'=>$params['NRGRUPOPRATOSERV'],
            'NRAGRUPGRPRAT'=>$params['NMAGRUPGRPRAT_key']
        ], $updateGroup);
    }

    private function newPlateRegister($params){
        $id = $this->novoCodigo->getCodeByTableName('ITGRUPOPRATOSERV', $params["NRORG"], 20);
        unset($params['NRORG']);
        $params['NOVOCODIGO'] = $id;
        $insertNewPlate = "INSERT INTO ITGRUPOPRATOSERV (NRITGRUPOPRATOSERV, CDFILAUXPRAT, CDPRATO, NRGRUPOPRATOSERV, CDOPERINCLUSAO) VALUES (:NOVOCODIGO, :CDFILAUXPRAT, :CDPRATO, :NRGRUPOPRATOSERV, :CDOPERADOR)";
        $this->execInSql($params, $insertNewPlate);
        return $id;

    }
    private function insertGroupPlateinPlate($params){

        $query = "INSERT INTO ITGRUPOPRATOSERV (NRITGRUPOPRATOSERV, CDFILAUXPRAT, CDPRATO, NRGRUPOPRATOSERV, CDOPERINCLUSAO) VALUES (:NRITGRUPOPRATOSERV, :CDFILAUXPRAT, :CDPRATO, :NRGRUPOPRATOSERV, :CDOPERADOR)";
        $newID = $this->_insertWithNewCode($params[0], $query, "ITGRUPOPRATOSERV", "NRITGRUPOPRATOSERV");
        return [$newID];
    }

    // only updates the first row in the DataSet
    private function updateGroupPlateinPlates($params){
        $deleteFromITGRUPOPRATOSERV = "DELETE FROM ITGRUPOPRATOSERV WHERE NRITGRUPOPRATOSERV = :NRITGRUPOPRATOSERV";
        $deleteParams = array("NRITGRUPOPRATOSERV" => $params[0]["NRITGRUPOPRATOSERV"]);
        $this->execInSql($deleteParams, $deleteFromITGRUPOPRATOSERV);
        if($params[0]["NRGRUPOPRATOSERV"] != null){
            unset($params["NRITGRUPOPRATOSERV"]);
            return $this->insertGroupPlateinPlate($params);
        }
    }

    private function updatePlateFrequency($p){
        $params = $p[0];
        $getPlateGruopFrequencySQL = "SELECT NRMAXFREQGRPRATMES FROM(SELECT VG.NRVIGENCIA, VG.CDFILIAL, VG.DTVIGENCIA, VG.DESCRICAO, VG.DTVIGENCIAFIM, FR.NRFREQGRPRATMES, FR.CDSERVFILI, IT.NRMINFREQGRPRATMES, IT.NRMAXFREQGRPRATMES, IT.NRMAXFREQGRPRATSEM, IT.NRGRUPOPRATOSERV, CASE WHEN (VG.DTVIGENCIAFIM IS NOT NULL AND VG.DTVIGENCIAFIM >= :DTMESCARD AND VG.DTVIGENCIA <= :DTMESCARD) THEN 0 WHEN (VG.DTVIGENCIAFIM IS NOT NULL AND VG.DTVIGENCIAFIM <= :DTMESCARD) THEN 2 WHEN VG.DTVIGENCIA <= :DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQGRPRATMES VG INNER JOIN FREQGRPRATMES FR ON FR.NRVIGENCIA = VG.NRVIGENCIA INNER JOIN ITFREQGRPRATMES IT ON IT.NRFREQGRPRATMES = FR.NRFREQGRPRATMES INNER JOIN ITGRUPOPRATOSERV ITG ON ITG.NRGRUPOPRATOSERV = IT.NRGRUPOPRATOSERV WHERE VG.IDTIPORESTRICAO = :IDTIPORESTRICAO AND VG.CDFILIAL = :CDFILIAL AND FR.CDSERVFILI = :CDSERVFILI AND ITG.CDFILAUXPRAT = :CDFILAUXPRAT AND ITG.CDPRATO = :CDPRATO ORDER BY ORDEM, VG.DTVIGENCIA DESC) WHERE ROWNUM = 1";
        
        $plateGroupFrequencyData = $this->execOutSql([
             "DTMESCARD" => $params["DTMESCARD"],
             "IDTIPORESTRICAO" => $params["IDTIPORESTRICAO"],
             "CDFILIAL" => $params["CDFILIAL"],
             "CDSERVFILI" => $params["CDSERVFILI"],
             "CDFILAUXPRAT" => $params["CDFILAUXPRAT"],
             "CDPRATO" => $params["CDPRATO"]
         ], $getPlateGruopFrequencySQL);
         
         $maxFreqPratMes = intval($params["NRMAXFREQPRATMES"]);
         
         if(count($plateGroupFrequencyData) < 0 || 
             $maxFreqPratMes <= intval($plateGroupFrequencyData[0]["NRMAXFREQGRPRATMES"])){
                 
            $sql = "UPDATE ITFREQPRATMES SET NRMINFREQPRATMES = :NRMINFREQPRATMES, NRMAXFREQPRATMES = :NRMAXFREQPRATMES, NRMAXFREQPRATSEM = :NRMAXFREQPRATSEM WHERE NRITFREQPRATMES = :NRITFREQPRATMES";

            $updateParams = [
                "NRMINFREQPRATMES" => $params["NRMINFREQPRATMES"],
                "NRMAXFREQPRATMES" => $params["NRMAXFREQPRATMES"],
                "NRMAXFREQPRATSEM" => $params["NRMAXFREQPRATSEM"],
                "NRITFREQPRATMES" => $params["NRITFREQPRATMES"]
            ];
            
            $this->execInSql($updateParams, $sql);
        } else {
            $plateGroupMaxFrequency = intval($plateGroupFrequencyData[0]["NRMAXFREQGRPRATMES"]);
            $mensagem = $this->formatMsgTradu("FREQ_MAX_GRUPO",$plateGroupMaxFrequency);
            throw new BackOfficeException($mensagem);
            
        }
    }

    private function insertPlateGroupValidity($paramReq){
        $validityData = $paramReq[0];
        $countValiditySql = "SELECT COUNT(NRVIGENCIA) AS COUNT FROM VIGFREQGRPRATMES WHERE IDTIPORESTRICAO = :IDTIPORESTRICAO AND CDFILIAL = :CDFILIAL AND DTVIGENCIA = :DTVIGENCIA";
        $validityCount = $this->execOutSql([
            "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
            "CDFILIAL" => $validityData["CDFILIAL"], 
            "DTVIGENCIA" => $validityData["DTVIGENCIA"]], 
        $countValiditySql)[0]["COUNT"];
        if($validityCount > 0){
            $mensagem = $this->formatMsgTradu("there_is_already_term_for_the_specified_period");
            throw new BackOfficeException($mensagem);
        } else {
            $insertValiditySql = "DECLARE V_NOVOCODIGO VARCHAR2(20); BEGIN NOVO_CODIGO('VIGFREQGRPRATMES', V_NOVOCODIGO); INSERT INTO VIGFREQGRPRATMES (NRVIGENCIA, CDFILIAL, DTVIGENCIA, DESCRICAO, DTVIGENCIAFIM, IDTIPORESTRICAO, CDDESCOBSFILIAL, CDOPERINCLUSAO) VALUES (V_NOVOCODIGO, :CDFILIAL, :DTVIGENCIA, :DESCRICAO, :DTVIGENCIAFIM, :IDTIPORESTRICAO, :CDDESCOBSFILIAL, :CDOPERADOR); END;";
            $params = [
                "CDFILIAL" => $validityData["CDFILIAL"],
                "DTVIGENCIA" => $validityData["DTVIGENCIA"],
                "DESCRICAO" => $validityData["DESCRICAO"],
                "DTVIGENCIAFIM" => $validityData["DTVIGENCIAFIM"],
                "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
                "CDDESCOBSFILIAL" => $validityData["CDDESCOBSFILIAL"],
                "CDOPERADOR" => $validityData["CDOPERADOR"]
            ];
            $this->execInSql($params, $insertValiditySql);
        }
    }

    private function insertPlateGroupProductValidity($paramReq){
        $validityData = $paramReq[0];
        $countValiditySql = "SELECT COUNT(NRVIGENCIA) AS COUNT FROM VIGFREQPRODGRPRAT WHERE IDTIPORESTRICAO = :IDTIPORESTRICAO AND CDFILIAL = :CDFILIAL AND DTVIGENCIA = :DTVIGENCIA";
        $validityCount = $this->execOutSql([
            "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
            "CDFILIAL" => $validityData["CDFILIAL"], 
            "DTVIGENCIA" => $validityData["DTVIGENCIA"]], 
        $countValiditySql)[0]["COUNT"];
        if($validityCount > 0){
            $mensagem = $this->formatMsgTradu("there_is_already_term_for_the_specified_period");
            throw new BackOfficeException($mensagem);
        } else {
            $insertValiditySql = "DECLARE V_NOVOCODIGO VARCHAR2(20); BEGIN NOVO_CODIGO('VIGFREQPRODGRPRAT', V_NOVOCODIGO); INSERT INTO VIGFREQPRODGRPRAT (NRVIGENCIA, CDFILIAL, DTVIGENCIA, DESCRICAO, DTVIGENCIAFIM, IDTIPORESTRICAO, CDDESCOBSFILIAL, CDOPERINCLUSAO) VALUES (V_NOVOCODIGO, :CDFILIAL, :DTVIGENCIA, :DESCRICAO, :DTVIGENCIAFIM, :IDTIPORESTRICAO, :CDDESCOBSFILIAL, :CDOPERADOR); END;";
            $params = [
                "CDFILIAL" => $validityData["CDFILIAL"],
                "DTVIGENCIA" => $validityData["DTVIGENCIA"],
                "DESCRICAO" => $validityData["DESCRICAO"],
                "DTVIGENCIAFIM" => $validityData["DTVIGENCIAFIM"],
                "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
                "CDDESCOBSFILIAL" => $validityData["CDDESCOBSFILIAL"],
                "CDOPERADOR" => $validityData["CDOPERADOR"]

            ];
            $this->execInSql($params, $insertValiditySql);
        }
    }

    private function insertProductGroupPlateRestrictionValidity($paramReq){
        $validityData = $paramReq[0];
        $countValiditySql = "SELECT COUNT(NRVIGENCIA) AS COUNT FROM VIGRESTPRODGR WHERE IDTIPORESTRICAO = :IDTIPORESTRICAO AND CDFILIAL = :CDFILIAL AND DTVIGENCIA = :DTVIGENCIA";
        $validityCount = $this->execOutSql([
            "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
            "CDFILIAL" => $validityData["CDFILIAL"],
            "DTVIGENCIA" => $validityData["DTVIGENCIA"]], 
        $countValiditySql)[0]["COUNT"];
        if($validityCount > 0){
            
            $mensagem = $this->formatMsgTradu("there_is_already_term_for_the_specified_period");
            throw new BackOfficeException($mensagem);
        } else {
            $insertValiditySql = "DECLARE V_NOVOCODIGO VARCHAR2(20); BEGIN NOVO_CODIGO('VIGRESTPRODGR', V_NOVOCODIGO); INSERT INTO VIGRESTPRODGR (NRVIGENCIA, CDFILIAL, DTVIGENCIA, DESCRICAO, DTVIGENCIAFIM, IDTIPORESTRICAO, CDDESCOBSFILIAL, CDOPERINCLUSAO) VALUES (V_NOVOCODIGO, :CDFILIAL, :DTVIGENCIA, :DESCRICAO, :DTVIGENCIAFIM, :IDTIPORESTRICAO, :CDDESCOBSFILIAL, :CDOPERADOR); END;";
            $params = [
                "CDFILIAL" => $validityData["CDFILIAL"],
                "DTVIGENCIA" => $validityData["DTVIGENCIA"],
                "DESCRICAO" => $validityData["DESCRICAO"],
                "DTVIGENCIAFIM" => $validityData["DTVIGENCIAFIM"],
                "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
                "CDDESCOBSFILIAL" => $validityData["CDDESCOBSFILIAL"],
                "CDOPERADOR" => $validityData["CDOPERADOR"]
            ];
            $this->execInSql($params, $insertValiditySql);
        }
    }

    private function insertWeekdayUsageValidity($paramReq){
        $validityData = $paramReq[0];
        $countValiditySql = "SELECT COUNT(NRVIGENCIA) AS COUNT FROM VIGUTILPRATOSERV WHERE IDTIPORESTRICAO = :IDTIPORESTRICAO AND CDFILIAL = :CDFILIAL AND DTVIGENCIA = :DTVIGENCIA";
        $validityCount = $this->execOutSql([
            "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
            "CDFILIAL" => $validityData["CDFILIAL"], 
            "DTVIGENCIA" => $validityData["DTVIGENCIA"]], 
        $countValiditySql)[0]["COUNT"];
        if($validityCount > 0){
            $mensagem = $this->formatMsgTradu("there_is_already_term_for_the_specified_period");
            throw new BackOfficeException($mensagem);
        } else {
            $insertValiditySql = "DECLARE V_NOVOCODIGO VARCHAR2(20); BEGIN NOVO_CODIGO('VIGUTILPRATOSERV', V_NOVOCODIGO); INSERT INTO VIGUTILPRATOSERV (NRVIGENCIA, CDFILIAL, DTVIGENCIA, DESCRICAO, DTVIGENCIAFIM, IDTIPORESTRICAO, CDOPERINCLUSAO) VALUES (V_NOVOCODIGO, :CDFILIAL, :DTVIGENCIA, :DESCRICAO, :DTVIGENCIAFIM, :IDTIPORESTRICAO, :CDOPERADOR); END;";
            $params = [
                "CDFILIAL" => $validityData["CDFILIAL"],
                "DTVIGENCIA" => $validityData["DTVIGENCIA"],
                "DESCRICAO" => $validityData["DESCRICAO"],
                "DTVIGENCIAFIM" => $validityData["DTVIGENCIAFIM"],
                "IDTIPORESTRICAO" => $validityData["IDTIPORESTRICAO"],
                "CDOPERADOR" => $validityData["CDOPERADOR"]
            ];
            $this->execInSql($params, $insertValiditySql);
        }
    }

    private function getServConfig($paramReq)
    {
        $sqlColunasTPPrato = "SELECT CF.CDFILIAL, CF.CDSERVFILI, CF.NRSEQCNFCARD, CF.NRORDCNFCARD, CF.CDTIPOPRAT, TP.NMTIPOPRAT || CASE WHEN CF.DSCOMPLTP IS NULL THEN '' ELSE '('||CF.DSCOMPLTP||')' END || CASE WHEN CF.IDOPCCNFCARD = 'S' THEN '(OPÇÃO)' ELSE '' END AS NMTIPOPRAT, CF.IDUTILPERCENTCONS, CF.IDOTMCOLUNAFIXA, CF.IDOTMNAOCONSCOR, CF.IDOTMNAOCONSMOD, CF.IDOTMINTERCPRAT FROM CONFCARD CF INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CF.CDTIPOPRAT WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI ORDER BY NRORDCNFCARD";
        $params = array(
            'CDFILIAL' => $paramReq['CDFILIAL'],
            'CDSERVFILI' => $paramReq['CDSERVFILI'],
        );
        $statement = $this->connection->prepare($sqlColunasTPPrato);
        $statement->execute($params);
        $colunasTPPratoAndAtrati = $statement->fetchAll();

        $sqlTipoPratSeq = "SELECT GP.CDGRUPOPRATO, GP.NMGRUPOPRATO, GS.NRGRUPOPRATOSERV, CF.NRCONFCARDGRUPOPRATO, CF.NRSEQCNFCARD FROM GRUPOPRATO GP INNER JOIN GRUPOPRATOSERV GS ON GP.NRGRUPOPRATO = GS.NRGRUPOPRATO LEFT JOIN CONFCARDGRUPOPRATO CF ON CF.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV AND CF.CDFILIAL = GP.CDFILIAL AND CF.CDSERVFILI = GS.CDSERVFILI WHERE GP.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI AND GP.IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY GP.NMGRUPOPRATO, GP.CDGRUPOPRATO, CF.NRSEQCNFCARD";
        $params = array(
            'CDFILIAL' => $paramReq['CDFILIAL'],
            'CDSERVFILI' => $paramReq['CDSERVFILI'],
            'IDTIPORESTRICAO' => $paramReq['IDTIPORESTRICAO'],
        );
        $statement = $this->connection->prepare($sqlTipoPratSeq);
        $statement->execute($params);
        $tipoPratSeq = $statement->fetchAll();

        return array(
            $colunasTPPratoAndAtrati,
            $tipoPratSeq,
        );
    }

    private function getMonthServConfig($paramReq)
    {
        $sqlColunasTPPrato = "SELECT CF.CDFILIAL, CF.CDSERVFILI, CF.NRSEQCNFCARD, CF.NRORDCNFCARD, CF.CDTIPOPRAT, TP.NMTIPOPRAT || CASE WHEN CF.DSCOMPLTP IS NULL THEN '' ELSE '('||CF.DSCOMPLTP||')' END || CASE WHEN CF.IDOPCCNFCARD = 'S' THEN '(OPÇÃO)' ELSE '' END AS NMTIPOPRAT, CF.IDUTILPERCENTCONS FROM CONFCARD CF INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CF.CDTIPOPRAT WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI ORDER BY NRORDCNFCARD";
        $params = array(
            'CDFILIAL' => $paramReq['CDFILIAL'],
            'CDSERVFILI' => $paramReq['CDSERVFILI'],
        );
        $statement = $this->connection->prepare($sqlColunasTPPrato);
        $statement->execute($params);
        $colunasTPPratoAndAtrati = $statement->fetchAll();

        $sqlMeses = "SELECT CM.NRCONFCARDMESATIVO, C.CDFILIAL, C.CDSERVFILI, C.NRSEQCNFCARD, NVL(CM.IDMESATIVO, 'SSSSSSSSSSSS') AS IDMESATIVO FROM CONFCARD C LEFT JOIN CONFCARDMESATIVO CM ON C.CDFILIAL = CM.CDFILIAL AND C.CDSERVFILI = CM.CDSERVFILI AND C.NRSEQCNFCARD = CM.NRSEQCNFCARD WHERE C.CDFILIAL = :CDFILIAL AND C.CDSERVFILI = :CDSERVFILI";
        $params = array(
            'CDFILIAL' => $paramReq['CDFILIAL'],
            'CDSERVFILI' => $paramReq['CDSERVFILI'],
        );
        $statement = $this->connection->prepare($sqlMeses);
        $statement->execute($params);
        $meses = $statement->fetchAll();

        return array(
            $colunasTPPratoAndAtrati,
            $meses,
        );
    }

    private function updatePortioningSetAutoConsumptionPercentage($paramsArray){
        $params = $paramsArray[0];
        
        $roundSqlParams = $params;
        unset($roundSqlParams['NRCONJUNTOPORCAO']);

        $averageSql = "SELECT ROUND(AVG(VRPECONSCARD),2) AS VRCONJUNTOPORCAO FROM (SELECT SUM(VRPECONSCARD) AS VRPECONSCARD, C.NRCARDAPIO FROM COMPCARD CP INNER JOIN CARDADIA C ON CP.CDFILIAL = C.CDFILIAL AND CP.NRCARDAPIO = C.NRCARDAPIO INNER JOIN TIPOPRAT T ON T.CDTIPOPRAT = CP.CDTIPOPRAT WHERE C.CDFILIAL = :CDFILIAL AND C.CDSERVFILI = :CDSERVFILI AND T.CDGRTIPOPR = :CDGRTIPOPR AND (C.DTMESCARD BETWEEN ADD_MONTHS(TO_DATE(:DTMESCARD),-12) AND TO_DATE(:DTMESCARD)) GROUP BY C.NRCARDAPIO)";
        $average = $this->execOutSql($roundSqlParams, $averageSql);
        
        if(isset($params['NRCONJUNTOPORCAO']) && is_numeric($average[0]['VRCONJUNTOPORCAO'])){
            $updatePortioningSetSql = "UPDATE CONJUNTOPORCAO SET VRCONJUNTOPORCAO = :VRCONJUNTOPORCAO WHERE NRCONJUNTOPORCAO = :NRCONJUNTOPORCAO";
            $updatePercentageParams = [
                'VRCONJUNTOPORCAO' => $average[0]['VRCONJUNTOPORCAO'],
                'NRCONJUNTOPORCAO' => $params['NRCONJUNTOPORCAO'] 
            ];
            $this->execInSql($updatePercentageParams, $updatePortioningSetSql);
        }
        
        return $average;
    }
    
    private function updateGrupProd($params){
        $params = $params[0];
        
        $paramsUpdate = [
            'CDGRUPPROD' => $params['CDGRUPPROD'],
            'NRITGERACAOCARD' => $params['NRITGERACAOCARD']
        ];
        
        $updateGrupProdSql = "UPDATE ITGERACAOCARD SET CDGRUPPROD = :CDGRUPPROD WHERE NRITGERACAOCARD = :NRITGERACAOCARD";
        $this->execInSql($paramsUpdate, $updateGrupProdSql);
        
        if($params['IDPERMITECONGELAR'] == 'S'){
            $paramsFind = [
                'DTMESCARD' => $params['DTMESCARD'],
                'CDFILIAL' => $params['CDFILIAL'],
                'CDGRUPPROD' => $params['CDGRUPPROD']
            ];
        
            $numDaysFreezeSql = "SELECT CASE WHEN MIN(TRUNC(DT.DTDATA)) - TO_DATE(:DTMESCARD) > TO_NUMBER(TO_CHAR(LAST_DAY(:DTMESCARD), 'DD')) THEN TO_NUMBER(TO_CHAR(LAST_DAY(:DTMESCARD), 'DD')) ELSE MIN(TRUNC(DT.DTDATA)) - TO_DATE(:DTMESCARD) END AS QTDIASCONG FROM DATA DT CROSS JOIN (SELECT TO_DATE(:DTMESCARD) + NVL(MAX(TO_NUMBER(L.NRLIMISEGU)),0) AS DATALIMITE FROM PRODUTO P INNER JOIN PRSUBGRPROD S ON P.CDPRODUTO = S.CDPRODUTO AND S.CDGRUPPROD = :CDGRUPPROD INNER JOIN (SELECT P.CDARVPROD, L.NRLIMISEGU FROM LIMISEGPROD L INNER JOIN PRODUTO P ON P.CDPRODUTO = L.CDPRODUTO AND L.CDFILIAL = :CDFILIAL ) L ON P.CDARVPROD LIKE L.CDARVPROD||'%' WHERE (P.CDPRODUTO = P.CDPRODESTO) AND (P.CDPRODUTO = P.CDPRODESTO) AND (NOT EXISTS (SELECT F.CDPRODUTO FROM PRODFILI F WHERE (F.CDFILIAL = :CDFILIAL) AND (F.IDPRODATIV = 'I') AND (F.CDPRODUTO = P.CDPRODUTO))) ) B WHERE NOT EXISTS(SELECT CDFILIAL FROM FERICOMP WHERE TRUNC(DTFERICOMP, 'MM') = TO_DATE(:DTMESCARD) AND CDFILIAL = :CDFILIAL AND DTFERICOMP = DT.DTDATA) AND DT.DTDATA >= B.DATALIMITE + (SELECT COUNT(DTFERICOMP) FROM FERICOMP WHERE (CDFILIAL = :CDFILIAL) AND (DTFERICOMP BETWEEN TO_DATE(:DTMESCARD) AND B.DATALIMITE))";
            $result = $this->execOutSql($paramsFind, $numDaysFreezeSql);
            $numDays = $result[0]['QTDIASCONG'];
            $paramsNumDaysFreeze = [
                'QTDIASCONG' => $numDays,
                'NRITGERACAOCARD' => $params['NRITGERACAOCARD']
            ];
            $updateNumDaysFreezeSql = "UPDATE ITGERACAOCARD SET QTDIASCONG = :QTDIASCONG WHERE NRITGERACAOCARD = :NRITGERACAOCARD";
            $this->execInSql($paramsNumDaysFreeze, $updateNumDaysFreezeSql);
            return array('QTDIASCONG' => $numDays);
        }
    }

    private function updateCusto($params){
        $updateCustoSql = "UPDATE ITGERACAOCARD SET VRCUSTOPADRAO = :VRCUSTOPADRAO, VRCUSTOMAX = :VRCUSTOMAX, VRCUSTOMIN = :VRCUSTOMIN WHERE NRITGERACAOCARD = :NRITGERACAOCARD";
        $params = $params[0];
        
        $paramsUpdate = [
            'VRCUSTOPADRAO' => $params['VRCUSTOPADRAO'],
            'VRCUSTOMAX' => $params['VRCUSTOMAX'],
            'VRCUSTOMIN' => $params['VRCUSTOMIN'],
            'NRITGERACAOCARD' => $params['NRITGERACAOCARD']
        ];
        
        $this->execInSql($paramsUpdate, $updateCustoSql);
        
        return array('CUSTO' => $paramsUpdate);
    }

    private function updateStatusItems($params){
        if(isset($params['ALL_NRITGERACAOCARD']) && is_array($params['ALL_NRITGERACAOCARD'])) {
            $items = array_map(function($NRITGERACAOCARD){
                return (int)$NRITGERACAOCARD;
            }, $params['ALL_NRITGERACAOCARD']);
            $items_str = "'".implode("','", $items)."'";
            
            $matches = [];
            preg_match("/[^\d\,\']/", $items_str, $matches);
            if(empty($matches)){
                $SQL = "UPDATE ITGERACAOCARD SET IDSTATUSGERACAO = :IDSTATUSGERACAO WHERE NRITGERACAOCARD IN ($items_str)";
                $params = [
                    'IDSTATUSGERACAO' => $params['IDSTATUSGERACAO']
                ];
                $this->execInSql($params, $SQL);
            }
        }
    }

    public function find($param){
        ini_set('memory_limit', '2048M');        
        $SQL = array(
            'JSONDATAOFSELECTEDSCENARY' => "SELECT NRCENARIOCARD, JSONCENARIO FROM CENARIOCARD WHERE NRCENARIOCARD = :NRCENARIOCARD",
            'PERMITECONGELAR' => "SELECT A.PERMITE_CONGELAR, A.LASTDAY, CASE WHEN B.QTDIAS > TO_NUMBER(A.LASTDAY) THEN TO_NUMBER(A.LASTDAY) ELSE B.QTDIAS END AS QTDIAS FROM(SELECT CASE WHEN COUNT(*) > 0 THEN 'N' ELSE 'S' END AS PERMITE_CONGELAR, TO_CHAR(LAST_DAY(:DTMESCARD), 'DD') AS LASTDAY FROM(SELECT CN.CDFILIAL, CN.CDSERVFILI, CN.NRSEQCNFMES, CN.CDTIPOPRAT FROM CNFCAMES CN LEFT JOIN CONFCARDMESATIVO CFA ON CFA.CDFILIAL = CN.CDFILIAL AND CFA.CDSERVFILI = CN.CDSERVFILI AND CN.NRSEQCNFMES = CFA.NRSEQCNFCARD WHERE CN.DTREFECNFMES = :DTMESCARD AND CN.CDFILIAL = :CDFILIAL AND CN.CDSERVFILI = :CDSERVFILI AND SUBSTR(NVL(CFA.IDMESATIVO,'SSSSSSSSSSSS'), TO_NUMBER(TO_CHAR(TO_DATE(:DTMESCARD), 'MM')),1) = 'S' AND NOT EXISTS(SELECT CF.CDFILIAL, CF.CDSERVFILI, CF.NRSEQCNFCARD, CF.CDTIPOPRAT FROM CONFCARD CF WHERE CN.CDFILIAL = CF.CDFILIAL AND CN.CDSERVFILI = CF.CDSERVFILI AND CN.NRORDCNFMES = CF.NRORDCNFCARD AND CN.CDTIPOPRAT = CF.CDTIPOPRAT) UNION ALL SELECT CF.CDFILIAL, CF.CDSERVFILI, CF.NRSEQCNFCARD AS NRSEQCNFMES, CF.CDTIPOPRAT FROM CONFCARD CF LEFT JOIN CONFCARDMESATIVO CFA ON CFA.CDFILIAL = CF.CDFILIAL AND CFA.CDSERVFILI = CF.CDSERVFILI AND CFA.NRSEQCNFCARD = CF.NRSEQCNFCARD WHERE CF.CDFILIAL = :CDFILIAL AND CF.CDSERVFILI = :CDSERVFILI AND SUBSTR(NVL(CFA.IDMESATIVO,'SSSSSSSSSSSS'), TO_NUMBER(TO_CHAR(TO_DATE(:DTMESCARD), 'MM')),1) = 'S' AND NOT EXISTS(SELECT CN.CDFILIAL, CN.CDSERVFILI, CN.NRSEQCNFMES, CN.CDTIPOPRAT FROM CNFCAMES CN WHERE CN.DTREFECNFMES = :DTMESCARD AND CN.CDFILIAL = CF.CDFILIAL AND CN.CDSERVFILI = CF.CDSERVFILI AND CN.NRORDCNFMES = CF.NRORDCNFCARD AND CN.CDTIPOPRAT = CF.CDTIPOPRAT) ) ) A CROSS JOIN (SELECT MIN(TRUNC(DT.DTDATA)) - TO_DATE(:DTMESCARD) AS QTDIAS FROM DATA DT CROSS JOIN (SELECT TO_DATE(:DTMESCARD) + NVL(MAX(TO_NUMBER(L.NRLIMISEGU)),0) AS DATALIMITE FROM PRODUTO P INNER JOIN PRSUBGRPROD S ON P.CDPRODUTO = S.CDPRODUTO INNER JOIN PARAMGERAL PG ON PG.CDGRUPPRODOTMCARD = S.CDGRUPPROD INNER JOIN (SELECT P.CDARVPROD, L.NRLIMISEGU FROM LIMISEGPROD L INNER JOIN PRODUTO P ON P.CDPRODUTO = L.CDPRODUTO AND L.CDFILIAL = :CDFILIAL ) L ON P.CDARVPROD LIKE L.CDARVPROD||'%' WHERE (P.CDPRODUTO = P.CDPRODESTO) AND (P.CDPRODUTO = P.CDPRODESTO) AND (NOT EXISTS (SELECT F.CDPRODUTO FROM PRODFILI F WHERE (F.CDFILIAL = :CDFILIAL) AND (F.IDPRODATIV = 'I') AND (F.CDPRODUTO = P.CDPRODUTO))) ) B WHERE NOT EXISTS(SELECT CDFILIAL FROM FERICOMP WHERE TRUNC(DTFERICOMP, 'MM') = TO_DATE(:DTMESCARD) AND CDFILIAL = :CDFILIAL AND DTFERICOMP = DT.DTDATA) AND DT.DTDATA >= B.DATALIMITE + (SELECT COUNT(DTFERICOMP) FROM FERICOMP WHERE (CDFILIAL = :CDFILIAL) AND (DTFERICOMP BETWEEN TO_DATE(:DTMESCARD) AND B.DATALIMITE) ) ) B",
            'CENARIOWOJ' => "SELECT C.NRCENARIOVINCULO, C.NRCENARIOCARD || CASE WHEN C.NRCENARIOCARDORIGINAL IS NULL THEN '' ELSE ' (' || :CENARIO_ORIGINAL || ' - ' || C.NRCENARIOCARDORIGINAL || ')' END AS NRCENARIOCARDORIGINAL, C.NRCENARIOCARDORIGINAL AS ORIGINAL, C.NRCENARIOCARD, OP.NMOPERADOR, C.DTCENARIOCARD, C.CDFILIAL, C.CDSERVFILI, C.DTMESCARD, C.CDOPERADOR, C.VRCUSTPADR, C.VRPUCENARIO, (:N_O || ': ' || C.NRCENARIOCARD || ', ' || :META || ': ' || C.VRCUSTPADR || ', ' || :P_U || ': ' || C.VRPUCENARIO || ', ' || :OPER || ': ' ||  OP.NMOPERADOR ||', ' || :EM || ': ' || TO_CHAR(C.DTCENARIOCARD, 'DD/MM/YYYY')) AS DESCR FROM CENARIOCARD C INNER JOIN OPERADOR OP ON C.CDOPERADOR = OP.CDOPERADOR WHERE C.NRCENARIOCARD = NVL(:NRCENARIOCARD, C.NRCENARIOCARD) AND C.NRORG = :NRORG AND C.CDFILIAL = :CDFILIAL AND C.CDSERVFILI = :CDSERVFILI AND DTMESCARD = :DTMESCARD AND IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY C.NRCENARIOCARD DESC",
            'CENARIO' => "SELECT NRCENARIOCARD, DTCENARIOCARD, CDFILIAL, CDSERVFILI, DTMESCARD, CDOPERADOR, JSONCENARIO, JSONCARDAPIO, VRCUSTPADR, VRPUCENARIO FROM CENARIOCARD WHERE NRCENARIOCARD = NVL(:NRCENARIOCARD,NRCENARIOCARD) AND NRORG= :NRORG ORDER BY DTCENARIOCARD DESC",

            'FILIAL' => "SELECT F.CDFILIAL, (F.CDFILIAL || ' - ' || F.NMFILIAL) NMFILIAL FROM FILIAL F INNER JOIN FILIOPER FO ON FO.CDFILIAL = F.CDFILIAL AND FO.CDOPERADOR = :CDOPERADOR AND F.IDCNTRRESC = 'N'",
            'FILIAL_FILIOPER' => "SELECT DISTINCT F.NRORG, F.CDFILIAL, F.NMFILIAL FROM FILIAL F INNER JOIN FILIOPER FO ON FO.NRORG = F.NRORG AND FO.CDFILIAL = F.CDFILIAL WHERE F.NRORG = :NRORG AND FO.CDOPERADOR = :CDOPERADOR AND F.IDCNTRRESC = 'N' ORDER BY CDFILIAL",
            'SERVICOS' => "SELECT CDFILIAL, CDSERVFILI, (CDSERVFILI || ' - ' || NMSERVICO) NMSERVICO FROM SERVICO WHERE CDFILIAL = :CDFILIAL AND IDSERVATIV = 'S' AND IDOTIMIZADOR = 'S'",
            'SERVICE_FILIOPER' => "SELECT S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO, S.CDFILIAL||S.CDSERVFILI AS CDLISTA FROM SERVICO S INNER JOIN FILIAL F ON F.CDFILIAL = S.CDFILIAL INNER JOIN FILIOPER FO ON FO.NRORG = F.NRORG AND FO.CDFILIAL = F.CDFILIAL WHERE (INSTR(:LISTACDFILIAL, S.CDFILIAL) > 0 OR :LISTACDFILIAL = 'T') AND S.CDFILIAL||S.CDSERVFILI <> :CDFILIAL||:CDSERVFILI AND S.IDSERVATIV = 'S' AND IDTIPOSERV = 'N' AND F.NRORG = :NRORG AND FO.CDOPERADOR = :CDOPERADOR AND F.IDCNTRRESC = 'N' ORDER BY CDFILIAL, CDSERVFILI",
            'AGUPAMENTO_FILIAL' => "SELECT G.CDGRUPFILI, G.NMGRUPFILI, CASE WHEN F.CDFILIAL IS NOT NULL THEN 'S' ELSE 'N' END AS SELECIONADA FROM GRUPFILI G LEFT JOIN FILGRUPFILI F ON F.CDGRUPFILI = G.CDGRUPFILI AND F.CDFILIAL = :CDFILIAL ORDER BY F.CDFILIAL, G.CDGRUPFILI",
            'SERVICOS_AUTOCOMPLETE' => "SELECT CDSERVFILI, NMSERVICO FROM SERVICO WHERE CDFILIAL = :CDFILIAL AND (:USER_QUERY IS NULL OR UPPER(CDSERVFILI) LIKE UPPER(:USER_QUERY)|| '%' OR UPPER(NMSERVICO) LIKE '%'||UPPER(:USER_QUERY)||'%') AND ROWNUM <= :MAX_RESULTS",
            'FILIAL_AUTOCOMPLETE' => "SELECT F.CDFILIAL, F.NMFILIAL FROM FILIAL F INNER JOIN FILIOPER FO ON FO.CDFILIAL = F.CDFILIAL AND FO.CDOPERADOR = :CDOPERADOR LEFT JOIN FILGRUPFILI FG ON F.CDFILIAL = FG.CDFILIAL WHERE((FG.CDGRUPFILI = :CDGRUPFILI) OR (:CDGRUPFILI IS NULL)) AND F.IDCNTRRESC = 'N' AND ( :USER_QUERY IS NULL OR UPPER(F.CDFILIAL) LIKE UPPER(:USER_QUERY) || '%' OR UPPER(F.NMFILIAL) LIKE '%' || UPPER(:USER_QUERY) || '%') AND ROWNUM <= :MAX_RESULTS ORDER BY F.CDFILIAL",
            'SERVICO_AUTOCOMPLETE_DEFAULTCARD_DATA' => "SELECT S.CDSERVFILI, S.NMSERVICO, PP.CDTIPOCUSTOPLA AS CDTIPOCUSTO, T.NMTIPOCUSTO, CP.VRCUSTPADR, NVL(PP.VRPEMINDIFPLAPA,0) AS PERC_CUSTO_MIN, NVL(PP.VRPEMAXDIFPLAPA,0) AS PERC_CUSTO_MAX, DECODE(NVL(PP.VRPEMINDIFPLAPA,0),0,CP.VRCUSTPADR,CP.VRCUSTPADR - CP.VRCUSTPADR * PP.VRPEMINDIFPLAPA / 100) AS VLR_CUSTO_MIN, DECODE(NVL(PP.VRPEMAXDIFPLAPA,0),0,CP.VRCUSTPADR,CP.VRCUSTPADR + CP.VRCUSTPADR * PP.VRPEMAXDIFPLAPA / 100) AS VLR_CUSTO_MAX, TRUNC(NVL(DTBASCONCUSPLAN, SYSDATE)) AS DTREFECUSTO FROM SERVICO S LEFT JOIN PARAMPLANEJA PP ON PP.CDFILIAL = S.CDFILIAL LEFT JOIN TIPOCUSTO T ON T.CDTIPOCUSTO = PP.CDTIPOCUSTOPLA AND T.IDUTTPCUSTO = 'S' LEFT JOIN CUSTPADR CP ON CP.CDFILIAL = S.CDFILIAL AND CP.CDSERVFILI = S.CDSERVFILI AND CP.DTREFCUSPADR = :DTREFCUSPADR LEFT JOIN CALENCARD CL ON CL.CDFILIAL = S.CDFILIAL AND DTMESCALENCARD = CP.DTREFCUSPADR WHERE S.CDFILIAL = :CDFILIAL AND S.IDTIPOSERV = 'N' AND S.IDSERVATIV = 'S' AND( :USER_QUERY IS NULL OR UPPER(S.CDSERVFILI) LIKE UPPER(:USER_QUERY) || '%' OR UPPER(S.NMSERVICO) LIKE '%' || UPPER(:USER_QUERY) || '%') AND ROWNUM <= :MAX_RESULTS ORDER BY S.CDSERVFILI",
            'TIPOCUSTO' => "SELECT CDTIPOCUSTO, NMTIPOCUSTO FROM( SELECT CDTIPOCUSTO, NMTIPOCUSTO, CASE WHEN CDTIPOCUSTOPLA IS NOT NULL THEN 0 ELSE 1 END AS ORDEM FROM TIPOCUSTO TP LEFT JOIN PARAMPLANEJA PJ ON TP.CDTIPOCUSTO = PJ.CDTIPOCUSTOPLA AND CDFILIAL = :CDFILIAL WHERE IDUTTPCUSTO = 'S' ) TP ORDER BY ORDEM, CDTIPOCUSTO",
            
            'GET_GROUP_PRAT' => "SELECT CDPADGRUPOPRATO AS CDGRUPOPRATO, NMPADGRUPOPRATO AS NMGRUPOPRATO FROM PADGRUPOPRATO WHERE IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY CDPADGRUPOPRATO",
            'GET_MULTI_GROUP_PRAT' => "SELECT GS.NRGRUPOPRATOSERV, G.CDGRUPOPRATO, G.NMGRUPOPRATO FROM GRUPOPRATOSERV GS INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO WHERE G.CDFILIAL = :P_CDFILIAL AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND GS.CDSERVFILI = :P_CDSERVFILI ORDER BY NMGRUPOPRATO",

            'TIPOCUSTO_SEARCH' => "SELECT CDTIPOCUSTO, NMTIPOCUSTO FROM( SELECT CDTIPOCUSTO, NMTIPOCUSTO, CASE WHEN CDTIPOCUSTOPLA IS NOT NULL THEN 0 ELSE 1 END AS ORDEM FROM TIPOCUSTO TP LEFT JOIN PARAMPLANEJA PJ ON TP.CDTIPOCUSTO = PJ.CDTIPOCUSTOPLA AND CDFILIAL = :CDFILIAL WHERE IDUTTPCUSTO = 'S' AND (UPPER(NMTIPOCUSTO) LIKE '%'||UPPER(:NMTIPOCUSTO)||'%' OR :NMTIPOCUSTO IS NULL) ) TP ORDER BY ORDEM, CDTIPOCUSTO",
            'DEFAULTOPTDATA' => "SELECT CP.VRCUSTPADR, NVL(PP.VRPEMINDIFPLAPA,0) AS PERC_CUSTO_MIN, NVL(PP.VRPEMAXDIFPLAPA,0) AS PERC_CUSTO_MAX, DECODE(NVL(PP.VRPEMINDIFPLAPA,0),0,CP.VRCUSTPADR,CP.VRCUSTPADR - CP.VRCUSTPADR * PP.VRPEMINDIFPLAPA / 100) AS VLR_CUSTO_MIN, DECODE(NVL(PP.VRPEMAXDIFPLAPA,0),0,CP.VRCUSTPADR,CP.VRCUSTPADR + CP.VRCUSTPADR * PP.VRPEMAXDIFPLAPA / 100) AS VLR_CUSTO_MAX, TRUNC(NVL(DTBASCONCUSPLAN, SYSDATE)) AS DTREFECUSTO FROM CUSTPADR CP LEFT JOIN PARAMPLANEJA PP ON CP.CDFILIAL = PP.CDFILIAL LEFT JOIN CALENCARD CL ON CL.CDFILIAL = CP.CDFILIAL AND DTMESCALENCARD = CP.DTREFCUSPADR WHERE CP.CDFILIAL = :CDFILIAL AND CP.CDSERVFILI = :CDSERVFILI AND CP.DTREFCUSPADR =:DTREFCUSPADR",

            'PARAMPRATO' => "SELECT P.CDFILAUXPRAT, P.CDPRATO, P.NMPRATO, G.NRAGRUPGRPRAT, G.CDAGRUPGRPRAT, G.NMAGRUPGRPRAT, G.NRGRUPOPRATOSERV, G.NMGRUPOPRATO, G.NRITGRUPOPRATOSERV, PF.IDCOMPLEXIDADE, PF.IDPREPREPARO, PF.IDCOR, PF.NRINGREDIENTEPRINC1, I1.NMINGREDIENTEPRINC AS NMINGREDIENTEPRINC1, PF.NRINGREDIENTEPRINC2, I2.NMINGREDIENTEPRINC AS NMINGREDIENTEPRINC2, PF.CDAPRERECE, AP.NMAPRERECE AS TIPOCORTE, PF.NRFORMAPREPARO, FF.NMFORMAPREPARO, PF.NRATRATIVIDADEPRATO, AT.NMATRATIVIDADEPRATO, CASE WHEN PF.CDFILAUXPRAT IS NOT NULL THEN 'S' ELSE 'N' END AS PRATO_PARAMETRIZADO FROM PRATO P LEFT JOIN PARAMPRATFILIAL PF ON PF.CDFILAUXPRAT = P.CDFILAUXPRAT AND PF.CDPRATO = P.CDPRATO AND PF.CDFILIAL = :CDFILIAL LEFT JOIN APRERECE AP ON AP.CDAPRERECE = PF.CDAPRERECE LEFT JOIN FORMAPREPARO FF ON FF.NRFORMAPREPARO = PF.NRFORMAPREPARO LEFT JOIN ATRATIVIDADEPRATO AT ON AT.NRATRATIVIDADEPRATO = PF.NRATRATIVIDADEPRATO LEFT JOIN INGREDIENTEPRINC I1 ON I1.NRINGREDIENTEPRINC = PF.NRINGREDIENTEPRINC1 LEFT JOIN INGREDIENTEPRINC I2 ON I2.NRINGREDIENTEPRINC = PF.NRINGREDIENTEPRINC2 LEFT JOIN( SELECT AG.NRAGRUPGRPRAT, AG.CDAGRUPGRPRAT, AG.NMAGRUPGRPRAT, GS.NRGRUPOPRATOSERV, G.NMGRUPOPRATO, IT.NRITGRUPOPRATOSERV, IT.CDPRATO, IT.CDFILAUXPRAT FROM GRUPOPRATO G INNER JOIN GRUPOPRATOSERV GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO AND G.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI INNER JOIN ITGRUPOPRATOSERV IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND G.IDTIPORESTRICAO = AG.IDTIPORESTRICAO) G ON G.CDFILAUXPRAT = P.CDFILAUXPRAT AND G.CDPRATO = P.CDPRATO WHERE (P.CDFILAUXPRAT = 'P' OR P.CDFILAUXPRAT = :CDFILIAL) AND P.CDNVPRATO = :CDNVPRATO AND P.IDATIVO = 'S' AND ( EXISTS ( SELECT 1 FROM RECESERV RS LEFT JOIN PRATSERV PS ON RS.CDPRATO = PS.CDPRATO AND RS.CDFILAUXPRAT = PS.CDFILAUXPRAT AND RS.CDFILIAL = PS.CDFILIAL AND RS.CDSERVFILI = PS.CDSERVFILI WHERE RS.CDFILIAL = :CDFILIAL AND RS.CDSERVFILI = :CDSERVFILI AND RS.CDPRATO = P.CDPRATO AND RS.CDFILAUXPRAT = P.CDFILAUXPRAT AND NVL(PS.IDATIVO, 'S') = 'S' ) OR EXISTS ( SELECT 1 FROM RECEFILI RF LEFT JOIN PRATFILI PF ON RF.CDPRATO = PF.CDPRATO AND RF.CDFILAUXPRAT = PF.CDFILAUXPRAT AND RF.CDFILIAL = PF.CDFILIAL WHERE RF.CDFILIAL = :CDFILIAL AND RF.CDPRATO = P.CDPRATO AND RF.CDFILAUXPRAT = P.CDFILAUXPRAT AND NVL(PF.IDATIVO, 'S') = 'S' ) OR EXISTS ( SELECT 1 FROM RECEPADR RP LEFT JOIN PRATPADR PP ON RP.CDPRATO = PP.CDPRATO AND RP.CDFILAUXPRAT = PP.CDFILAUXPRAT WHERE RP.CDPRATO = P.CDPRATO AND RP.CDFILAUXPRAT = P.CDFILAUXPRAT AND NVL(PP.IDATIVO, 'S') = 'S' AND :IDUTRPADCA = 'S' ) ) ORDER BY P.CDPRATO ",

            'INGRPRINC' => "SELECT NRINGREDIENTEPRINC, NMINGREDIENTEPRINC FROM INGREDIENTEPRINC",
            'TPCORTE' => "SELECT CDAPRERECE, 'false' AS is_new, NMAPRERECE AS TIPOCORTE FROM APRERECE ORDER BY CDAPRERECE",
            'MDPREPARO' => "SELECT NRFORMAPREPARO, NMFORMAPREPARO FROM FORMAPREPARO",
            'ATRATIVIDADE' => "SELECT NRATRATIVIDADEPRATO, NMATRATIVIDADEPRATO, VRPESOATRATIVIDADE FROM ATRATIVIDADEPRATO WHERE CDFILIAL = :CDFILIAL",

            'COMBRESTR' => "SELECT C.CDFILIAL, C.NRCOMBGRUPOTPPRAT, C.CDGRTIPOPR, G.NMGRTIPOPR, C.CDFILAUXPRAT, C.CDPRATO, P.NMPRATO, IT.NRITCOMBGRUPOTPPRAT, IT.CDGRTIPOPR AS IT_CDGRTIPOPR, GI.NMGRTIPOPR AS IT_NMGRTIPOPR, IT.CDFILAUXPRAT AS IT_CDFILAUXPRAT, IT.CDPRATO AS IT_CDPRATO, PI.NMPRATO AS IT_NMPRATO, IT.IDCOMBINACAO, IT.PERCENTREDUCAO FROM COMBGRUPOTPPRAT C INNER JOIN GRTIPOPR G ON G.CDGRTIPOPR = C.CDGRTIPOPR INNER JOIN PRATO P ON P.CDFILAUXPRAT = C.CDFILAUXPRAT AND C.CDPRATO = P.CDPRATO INNER JOIN ITCOMBGRUPOTPPRAT IT ON IT.NRCOMBGRUPOTPPRAT = C.NRCOMBGRUPOTPPRAT INNER JOIN GRTIPOPR GI ON GI.CDGRTIPOPR = IT.CDGRTIPOPR INNER JOIN PRATO PI ON PI.CDFILAUXPRAT = IT.CDFILAUXPRAT AND IT.CDPRATO = PI.CDPRATO WHERE C.CDFILIAL = :CDFILIAL",
            'GRUPOTIPOPRATO' => "SELECT CDGRTIPOPR, NMGRTIPOPR FROM GRTIPOPR ORDER BY CDGRTIPOPR",
            'GRUPOTIPOPRATO_AUTOCOMPLETE' => "SELECT CDGRTIPOPR, NMGRTIPOPR FROM GRTIPOPR WHERE (:MAX_RESULTS IS NULL OR ROWNUM <=:MAX_RESULTS) AND (:GRUPOPRATO IS NULL OR UPPER(CDGRTIPOPR) LIKE UPPER(:GRUPOPRATO)|| '%' OR UPPER(NMGRTIPOPR) LIKE '%'||UPPER(:GRUPOPRATO)||'%') ORDER BY CDGRTIPOPR",
            'GRUPOPRATO_AUTOCOMPLETE' => "SELECT AG.NMAGRUPGRPRAT, G.NMGRUPOPRATO, GS.NRGRUPOPRATOSERV FROM GRUPOPRATO G INNER JOIN GRUPOPRATOSERV GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO LEFT JOIN AGRUPGRPRAT AG ON GS.NRAGRUPGRPRAT = AG.NRAGRUPGRPRAT WHERE G.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO",
            'COMB_GRUPOTIPOPRATO_AUTOCOMPLETE' => "SELECT CDGRTIPOPR AS COMB_CDGRTIPOPR, NMGRTIPOPR AS COMB_NMGRTIPOPR FROM GRTIPOPR WHERE (:MAX_RESULTS IS NULL OR ROWNUM <=:MAX_RESULTS) AND (:GRUPOPRATO IS NULL OR UPPER(CDGRTIPOPR) LIKE UPPER(:GRUPOPRATO)|| '%' OR UPPER(NMGRTIPOPR) LIKE '%'||UPPER(:GRUPOPRATO)||'%') ORDER BY CDGRTIPOPR",
            'PRATO' => "SELECT CDFILAUXPRAT, CDPRATO, NMPRATO FROM PRATO WHERE (CDFILAUXPRAT = 'P' OR CDFILAUXPRAT =:CDFILIAL) AND CDNVPRATO = (SELECT MAX(CDNVPRATO) FROM PRATO) AND ((:CDPRATO IS NULL OR UPPER(CDPRATO) LIKE UPPER(:CDPRATO)|| '%') AND (:NMPRATO IS NULL OR UPPER(NMPRATO) LIKE '%'||UPPER(:NMPRATO)||'%')) AND (:PRATO IS NULL OR UPPER(CDPRATO) LIKE UPPER(:PRATO)|| '%' OR UPPER(NMPRATO) LIKE '%'||UPPER(:PRATO)||'%') AND (:MAX_RESULTS IS NULL OR ROWNUM <= :MAX_RESULTS) ORDER BY CDPRATO",
            'COMB_PRATO' => "SELECT CDFILAUXPRAT AS COMB_CDFILAUXPRAT, CDPRATO AS COMB_CDPRATO, NMPRATO AS COMB_NMPRATO FROM PRATO WHERE (CDFILAUXPRAT = 'P' OR CDFILAUXPRAT =:CDFILIAL) AND CDNVPRATO = (SELECT MAX(CDNVPRATO) FROM PRATO) AND ((:CDPRATO IS NULL OR UPPER(CDPRATO) LIKE UPPER(:CDPRATO)|| '%') AND (:NMPRATO IS NULL OR UPPER(NMPRATO) LIKE '%'||UPPER(:NMPRATO)||'%')) AND (:PRATO IS NULL OR UPPER(CDPRATO) LIKE UPPER(:PRATO)|| '%' OR UPPER(NMPRATO) LIKE '%'||UPPER(:PRATO)||'%') AND (:MAX_RESULTS IS NULL OR ROWNUM <= :MAX_RESULTS) ORDER BY CDPRATO",

            'VIGENCIAGRUPOPRATO' => "SELECT NRVIGENCIA, DTVIGENCIA, TO_CHAR(DTVIGENCIA,'MM/YYYY') AS VIGENCIA, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS VIGENCIAFIM, DESCRICAO, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS DTVIGENCIAFIM, CASE WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM >= :DTMESCARD AND DTVIGENCIA <= :DTMESCARD) THEN 0 WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM <= :DTMESCARD) THEN 2 WHEN DTVIGENCIA <= :DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQGRPRATMES WHERE CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC",
            'VIGENCIAGRUPOPRATOPRODUTO' => "SELECT NRVIGENCIA, DTVIGENCIA, TO_CHAR(DTVIGENCIA,'MM/YYYY') AS VIGENCIA, DESCRICAO, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS VIGENCIAFIM, CASE WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM >= :DTMESCARD AND DTVIGENCIA <= :DTMESCARD) THEN 0 WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM <= :DTMESCARD) THEN 2 WHEN DTVIGENCIA <= :DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQPRODGRPRAT WHERE CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC",
            'DESCOBSFILIAL' => "SELECT CDDESCOBSFILIAL, DSOBSFILI FROM DESCOBSFILIAL ORDER BY CDDESCOBSFILIAL",
            'FREQUENCIAGRUPO' => "SELECT AG.NRAGRUPGRPRAT, AG.NMAGRUPGRPRAT, G.NRGRUPOPRATO, G.NMGRUPOPRATO, IT.NRGRUPOPRATOSERV, F.NRFREQGRPRATMES, IT.NRITFREQGRPRATMES, IT.NRMINFREQGRPRATMES, IT.NRMAXFREQGRPRATMES, IT.NRMAXFREQGRPRATSEM FROM FREQGRPRATMES F INNER JOIN ITFREQGRPRATMES IT ON IT.NRFREQGRPRATMES = F.NRFREQGRPRATMES INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = IT.NRGRUPOPRATOSERV AND GS.CDSERVFILI = :CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :IDTIPORESTRICAO WHERE F.NRVIGENCIA = :NRVIGENCIA ORDER BY AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO",
            'FREQUENCIAPRATO' => "SELECT IT.CDFILAUXPRAT, IT.CDPRATO, P.NMPRATO, F.NRFREQPRATMES, IT.NRITFREQPRATMES, IT.NRMINFREQPRATMES, IT.NRMAXFREQPRATMES, IT.NRMAXFREQPRATSEM FROM FREQPRATMES F INNER JOIN ITFREQPRATMES IT ON IT.NRFREQPRATMES = F.NRFREQPRATMES INNER JOIN PRATO P ON IT.CDPRATO = P.CDPRATO AND IT.CDFILAUXPRAT = P.CDFILAUXPRAT WHERE F.NRVIGENCIA = :NRVIGENCIA AND F.CDSERVFILI = :CDSERVFILI ORDER BY IT.CDPRATO, P.NMPRATO",
            'FREQUENCIAGRUPOPRATOPRODUTO' => "SELECT AG.NRAGRUPGRPRAT, AG.NMAGRUPGRPRAT, G.NRGRUPOPRATO, G.NMGRUPOPRATO, IT.NRGRUPOPRATOSERV, IT.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, F.NRFREQPRODGRPRAT, IT.NRITFREQPRODGRPRAT, IT.NRMINPRODGRPRAT, IT.NRMAXPRODGRPRAT, IT.NRMAXPRODGRPRATSEM FROM FREQPRODGRPRAT F INNER JOIN ITFREQPRODGRPRAT IT ON IT.NRFREQPRODGRPRAT = F.NRFREQPRODGRPRAT INNER JOIN PRODUTO P ON P.CDPRODUTO = IT.CDPRODUTO INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = IT.NRGRUPOPRATOSERV AND GS.CDSERVFILI = :CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :IDTIPORESTRICAO WHERE F.NRVIGENCIA = :NRVIGENCIA ORDER BY AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO, P.CDARVPROD",
            'PRODUTO_AUTOCOMPLETE' => "SELECT CDPRODUTO, CDARVPROD, NMPRODUTO FROM PRODUTO P WHERE (:PRODUTO IS NULL OR UPPER(CDARVPROD) LIKE UPPER(:PRODUTO)|| '%' OR UPPER(NMPRODUTO) LIKE '%'||UPPER(:PRODUTO)||'%') AND (:MAX_RESULTS IS NULL OR ROWNUM <= :MAX_RESULTS) AND (CDPRODUTO = CDPRODESTO) ORDER BY CDARVPROD, NMPRODUTO",
            'PRODUTOGRUPOPRATORESTRICACAO' => "SELECT AG.NRAGRUPGRPRAT, AG.NMAGRUPGRPRAT, G.NRGRUPOPRATO, G.NMGRUPOPRATO, R.NRGRUPOPRATOSERV, R.NRRESTPRODGR, R.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO FROM RESTPRODGR R INNER JOIN PRODUTO P ON P.CDPRODUTO = R.CDPRODUTO INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = R.NRGRUPOPRATOSERV AND GS.CDSERVFILI = :CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :IDTIPORESTRICAO WHERE R.NRVIGENCIA = :NRVIGENCIA ORDER BY AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO, P.CDARVPROD",
            'RULESPOSPROCESSING' =>"SELECT NROTMREGRAFLEX, IDFLEXIVEL, NRORDEMREGRA, DSREGRAFLEX FROM OTMREGRAFLEX",

            'GRUPOPRATOSERV_AUTOCOMPLETE' => "SELECT AG.NMAGRUPGRPRAT, G.NMGRUPOPRATO, GS.NRGRUPOPRATOSERV FROM GRUPOPRATO G INNER JOIN GRUPOPRATOSERV GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO LEFT JOIN AGRUPGRPRAT AG ON GS.NRAGRUPGRPRAT = AG.NRAGRUPGRPRAT WHERE G.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO AND NOT EXISTS(SELECT ITT.NRGRUPOPRATOSERV FROM ITFREQGRPRATMES ITT INNER JOIN FREQGRPRATMES FG ON ITT.NRFREQGRPRATMES = FG.NRFREQGRPRATMES WHERE FG.NRVIGENCIA = :NRVIGENCIA AND ITT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV) ORDER BY AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO",

            'VIGENCIAPRATO' => "SELECT NRVIGENCIA, TO_CHAR(DTVIGENCIA,'MM/YYYY') AS VIGENCIA, DESCRICAO, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS DTVIGENCIAFIM, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS VIGENCIAFIM, DTVIGENCIA, CASE WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM >= :DTMESCARD AND DTVIGENCIA <= :DTMESCARD) THEN 0 WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM <= :DTMESCARD) THEN 2 WHEN DTVIGENCIA <= :DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGFREQPRATMES WHERE CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC",
            "VIGENCIAPRODUTOGRUPOPRATORESTRICACAO" => "SELECT NRVIGENCIA, TO_CHAR(DTVIGENCIA,'MM/YYYY') AS VIGENCIA, DESCRICAO, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS DTVIGENCIAFIM, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS VIGENCIAFIM, CASE WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM >= :DTMESCARD AND DTVIGENCIA <= :DTMESCARD) THEN 0 WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM <= :DTMESCARD) THEN 2 WHEN DTVIGENCIA <= :DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGRESTPRODGR WHERE CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC",
            'VIGENCIAUTILIZACAOSEMENA' => "SELECT NRVIGENCIA, TO_CHAR(DTVIGENCIA,'MM/YYYY') AS VIGENCIA, DESCRICAO, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS DTVIGENCIAFIM, TO_CHAR(DTVIGENCIAFIM,'MM/YYYY') AS VIGENCIAFIM, CASE WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM >= :DTMESCARD AND DTVIGENCIA <= :DTMESCARD) THEN 0 WHEN (DTVIGENCIAFIM IS NOT NULL AND DTVIGENCIAFIM <= :DTMESCARD) THEN 2 WHEN DTVIGENCIA <= :DTMESCARD THEN 1 ELSE 2 END AS ORDEM FROM VIGUTILPRATOSERV WHERE CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY ORDEM, DTVIGENCIA DESC",
            'PRATO_NAO_PARAMETRIZADO_FREQ' => "SELECT CDFILAUXPRAT, CDPRATO, NMPRATO FROM(SELECT CDFILAUXPRAT, CDPRATO, NMPRATO FROM PRATO WHERE(CDFILAUXPRAT = 'P' OR CDFILAUXPRAT =:CDFILIAL) AND (CDNVPRATO = (SELECT MAX(CDNVPRATO) FROM PRATO)) AND (:PRATO IS NULL OR UPPER(CDPRATO) LIKE UPPER(:PRATO)|| '%' OR UPPER(NMPRATO) LIKE '%'||UPPER(:PRATO)||'%') AND NOT EXISTS (SELECT I.CDPRATO FROM ITFREQPRATMES I WHERE I.NRFREQPRATMES = :NRFREQPRATMES AND I.CDPRATO = PRATO.CDPRATO AND I.CDFILAUXPRAT = PRATO.CDFILAUXPRAT) AND PRATO_RECEITA_ATIVA(CDPRATO,CDFILAUXPRAT,:CDFILIAL, NULL) = 'S' ORDER BY CDPRATO) WHERE :MAX_RESULTS IS NULL OR ROWNUM <= :MAX_RESULTS",
            'GRUPO_NAO_PARAMETRIZADO_UTILIZACAO_SEMANA' => "SELECT AG.NMAGRUPGRPRAT, G.NMGRUPOPRATO, GS.NRGRUPOPRATOSERV FROM GRUPOPRATO G INNER JOIN GRUPOPRATOSERV GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO LEFT JOIN AGRUPGRPRAT AG ON GS.NRAGRUPGRPRAT = AG.NRAGRUPGRPRAT WHERE G.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO AND G.NMGRUPOPRATO LIKE '%'||:GRUPO||'%' AND NOT EXISTS(SELECT UT.NRGRUPOPRATOSERV FROM UTILPRATOSERV UT WHERE UT.NRVIGENCIA = :NRVIGENCIA AND UT.CDSERVFILI = GS.CDSERVFILI AND UT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV) ORDER BY AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO",
            'UTILIZACAOPRATOSEMANA' => "SELECT UT.NRUTILPRATOSERV, AG.NRAGRUPGRPRAT, AG.NMAGRUPGRPRAT, G.NRGRUPOPRATO, G.NMGRUPOPRATO, UT.NRGRUPOPRATOSERV, IDUTILPRATOSERVSEG, IDUTILPRATOSERVTER, IDUTILPRATOSERVQUA, IDUTILPRATOSERVQUI, IDUTILPRATOSERVSEX, IDUTILPRATOSERVSAB, IDUTILPRATOSERVDOM, IDUTILPRATOSERVFERI, IDUTILPRATOSERVPRIM, IDUTILPRATOSERVSEM2, IDUTILPRATOSERVSEM3, IDUTILPRATOSERVSEM4, IDUTILPRATOSERVULT FROM UTILPRATOSERV UT   INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = UT.NRGRUPOPRATOSERV AND GS.CDSERVFILI = :CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :IDTIPORESTRICAO WHERE UT.NRVIGENCIA = :NRVIGENCIA ORDER BY AG.NMAGRUPGRPRAT, G.CDGRUPOPRATO",
            'VINGENCIASOBRIG' => "SELECT NRVIGENCIA, CDFILIAL, DTVIGENCIA, DTVIGENCIAFIM, DESCRICAO FROM VIGUTILPRATOSERV WHERE CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO",
            'OBRIGATORIEDADE' => "SELECT U.NRUTILPRATOSERV, U.NRGRUPOPRATOSERV, G.CDGRUPOPRATO, G.NMGRUPOPRATO, U.IDUTILPRATOSERVSEG, U.IDUTILPRATOSERVTER, U.IDUTILPRATOSERVQUA, U.IDUTILPRATOSERVQUI, U.IDUTILPRATOSERVSEX, U.IDUTILPRATOSERVSAB, U.IDUTILPRATOSERVDOM, U.IDUTILPRATOSERVFERI, U.IDUTILPRATOSERVPRIM, U.IDUTILPRATOSERVULT FROM UTILPRATOSERV U INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = U.NRGRUPOPRATOSERV INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO WHERE U.CDSERVFILI = :CDSERVFILI AND U.NRVIGENCIA = :NRVIGENCIA",
            'COMBGRUPOTPPRAT' => "SELECT C.NRCOMBGRUPOTPPRAT, C.CDGRTIPOPR, G1.NMGRTIPOPR, CASE WHEN C.CDFILAUXPRAT IS NOT NULL THEN 'Prato' ELSE 'Grupo' END AS TIPO, CASE WHEN P1.CDFILAUXPRAT IS NOT NULL THEN CASE WHEN P1.CDFILAUXPRAT = 'P' THEN 'P' ELSE 'L' END || ' - ' || C.CDPRATO ELSE GS1.CDGRUPOPRATO END AS CODIGO, CASE WHEN P1.CDFILAUXPRAT IS NOT NULL THEN P1.NMPRATO ELSE GS1.NMGRUPOPRATO END AS NOME, I.NRITCOMBGRUPOTPPRAT, I.CDGRTIPOPR AS CDGRTIPOPR_COMB, G2.NMGRTIPOPR AS NMGRTIPOPR_COMB, CASE WHEN I.CDFILAUXPRAT IS NOT NULL THEN 'Prato' WHEN I.NRGRUPOPRATOSERV IS NOT NULL THEN 'Grupo' ELSE 'Coluna' END AS TIPO_COMB, CASE WHEN P2.CDFILAUXPRAT IS NOT NULL THEN CASE WHEN P2.CDFILAUXPRAT = 'P' THEN 'P' ELSE 'L' END || ' - ' || I.CDPRATO WHEN I.NRGRUPOPRATOSERV IS NOT NULL THEN GS2.CDGRUPOPRATO ELSE I.CDTIPOPRAT END AS CODIGO_COMB, CASE WHEN P2.CDFILAUXPRAT IS NOT NULL THEN P2.NMPRATO WHEN I.NRGRUPOPRATOSERV IS NOT NULL THEN GS2.NMGRUPOPRATO ELSE TP.NMTIPOPRAT END AS NOME_COMB, I.IDCOMBINACAO, I.PERCENTREDUCAO FROM COMBGRUPOTPPRAT C INNER JOIN ITCOMBGRUPOTPPRAT I ON I.NRCOMBGRUPOTPPRAT = C.NRCOMBGRUPOTPPRAT LEFT JOIN PRATO P1 ON P1.CDFILAUXPRAT = C.CDFILAUXPRAT AND P1.CDPRATO = C.CDPRATO INNER JOIN GRTIPOPR G1 ON G1.CDGRTIPOPR = C.CDGRTIPOPR LEFT JOIN(SELECT GS.NRGRUPOPRATOSERV, G.CDGRUPOPRATO, G.NMGRUPOPRATO FROM GRUPOPRATOSERV GS INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND GS.CDSERVFILI = :CDSERVFILI AND G.CDFILIAL =:CDFILIAL AND G.IDTIPORESTRICAO =:IDTIPORESTRICAO) GS1 ON GS1.NRGRUPOPRATOSERV = C.NRGRUPOPRATOSERV INNER JOIN GRTIPOPR G2 ON G2.CDGRTIPOPR = I.CDGRTIPOPR LEFT JOIN PRATO P2 ON P2.CDFILAUXPRAT = I.CDFILAUXPRAT AND P2.CDPRATO = I.CDPRATO LEFT JOIN (SELECT GS.NRGRUPOPRATOSERV, G.CDGRUPOPRATO, G.NMGRUPOPRATO FROM GRUPOPRATOSERV GS INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND GS.CDSERVFILI = :CDSERVFILI AND G.CDFILIAL =:CDFILIAL AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO ) GS2 ON GS2.NRGRUPOPRATOSERV = I.NRGRUPOPRATOSERV LEFT JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = I.CDTIPOPRAT WHERE C.CDFILIAL = :CDFILIAL AND C.CDSERVFILI = :CDSERVFILI ORDER BY I.IDCOMBINACAO, C.CDGRTIPOPR, TIPO, CODIGO, I.CDGRTIPOPR, TIPO_COMB, CODIGO_COMB ",

            'PADGRUPOPRATO' => "SELECT CDPADGRUPOPRATO, NMPADGRUPOPRATO FROM PADGRUPOPRATO WHERE IDTIPORESTRICAO = :IDTIPORESTRICAO",
            'AGRUPGRPRAT' => "SELECT NRAGRUPGRPRAT, CDAGRUPGRPRAT, NMAGRUPGRPRAT, IDTIPORESTRICAO FROM AGRUPGRPRAT WHERE IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY CDAGRUPGRPRAT",
            'GRUPOPRAT' => "SELECT GP.CDGRUPOPRATO, GP.NMGRUPOPRATO, GS.NRGRUPOPRATOSERV FROM GRUPOPRATO GP INNER JOIN GRUPOPRATOSERV GS ON GP.NRGRUPOPRATO = GS.NRGRUPOPRATO WHERE GP.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI AND GP.IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY GP.CDGRUPOPRATO",
            'PRATO_GRUPO' => "SELECT DISTINCT IT.CDFILAUXPRAT, IT.CDPRATO, NMPRATO FROM ITGRUPOPRATOSERV IT INNER JOIN GRUPOPRATOSERV GS ON GS.NRGRUPOPRATOSERV = IT.NRGRUPOPRATOSERV AND GS.CDSERVFILI = :CDSERVFILI INNER JOIN GRUPOPRATO G ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.CDFILIAL = :CDFILIAL AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO INNER JOIN PRATO P ON IT.CDFILAUXPRAT = P.CDFILAUXPRAT AND P.CDPRATO = IT.CDPRATO INNER JOIN CONFCARDGRUPOPRATO CG ON CG.CDFILIAL = G.CDFILIAL AND CG.CDSERVFILI = GS.CDSERVFILI AND CG.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV INNER JOIN CONFCARD C ON C.CDFILIAL = CG.CDFILIAL AND C.CDSERVFILI = CG.CDSERVFILI AND C.NRSEQCNFCARD = CG.NRSEQCNFCARD INNER JOIN TIPOPRAT T ON T.CDTIPOPRAT = C.CDTIPOPRAT AND T.CDGRTIPOPR = :CDGRTIPOPR WHERE(:PRATO IS NULL OR UPPER(IT.CDPRATO) LIKE UPPER(:PRATO) || '%' OR UPPER(P.NMPRATO) LIKE '%' || UPPER(:PRATO) || '%') AND(:MAX_RESULTS IS NULL OR ROWNUM <=:MAX_RESULTS) AND PRATO_RECEITA_ATIVA(IT.CDPRATO,IT.CDFILAUXPRAT,:CDFILIAL, :CDSERVFILI) = 'S' ORDER BY IT.CDPRATO",
            'GRUPO_PRATO_AUTOCOMPLETE' => "SELECT DISTINCT GP.CDGRUPOPRATO, GP.NMGRUPOPRATO, GS.NRGRUPOPRATOSERV FROM GRUPOPRATO GP INNER JOIN GRUPOPRATOSERV GS ON GP.NRGRUPOPRATO = GS.NRGRUPOPRATO INNER JOIN CONFCARDGRUPOPRATO CG ON CG.CDFILIAL = GP.CDFILIAL AND CG.CDSERVFILI = GS.CDSERVFILI AND CG.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV INNER JOIN CONFCARD C ON C.CDFILIAL = CG.CDFILIAL AND C.CDSERVFILI = CG.CDSERVFILI AND C.NRSEQCNFCARD = CG.NRSEQCNFCARD INNER JOIN TIPOPRAT T ON T.CDTIPOPRAT = C.CDTIPOPRAT AND T.CDGRTIPOPR = :CDGRTIPOPR WHERE GP.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI AND GP.IDTIPORESTRICAO = :IDTIPORESTRICAO AND(:GRUPOPRATO IS NULL OR UPPER(GP.CDGRUPOPRATO) LIKE UPPER(:GRUPOPRATO) || '%' OR UPPER(GP.NMGRUPOPRATO) LIKE '%' || UPPER(:GRUPOPRATO) || '%') AND (:MAX_RESULTS IS NULL OR ROWNUM <= :MAX_RESULTS) ORDER BY GP.CDGRUPOPRATO",
            'TIPOPRATO_AUTOCOMPLETE' => "SELECT CDTIPOPRAT, NMTIPOPRAT FROM TIPOPRAT T WHERE ( CDTIPOPRAT = :TIPOPRAT || '%' OR UPPER(NMTIPOPRAT) LIKE '%'|| UPPER(:TIPOPRAT)|| '%' ) AND  CDGRTIPOPR = :CDGRTIPOPR AND ROWNUM <= :MAX_RESULTS",

            'PRATOSEMGRUPO' => "SELECT CDFILAUXPRAT, RECEITA, CDPRATO, NMPRATO, DTULTATURECE FROM ( SELECT DISTINCT P.CDFILAUXPRAT, ( SELECT DISTINCT 'POR SERVIÇO' AS IDTIPO FROM RECESERV LEFT JOIN PRATSERV ON PRATSERV.CDPRATO = RECESERV.CDPRATO AND PRATSERV.CDFILAUXPRAT = RECESERV.CDFILAUXPRAT AND PRATSERV.CDFILIAL = RECESERV.CDFILIAL AND PRATSERV.CDSERVFILI = RECESERV.CDSERVFILI WHERE ( RECESERV.CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( RECESERV.CDPRATO = P.CDPRATO ) AND ( RECESERV.CDFILIAL = :CDFILIAL ) AND ( RECESERV.CDSERVFILI = :CDSERVFILI ) AND ( NVL(PRATSERV.IDATIVO, 'S') = 'S' ) UNION ALL SELECT DISTINCT 'LOCAL' AS IDTIPO FROM RECEFILI LEFT JOIN PRATFILI ON PRATFILI.CDPRATO = RECEFILI.CDPRATO AND PRATFILI.CDFILAUXPRAT = RECEFILI.CDFILAUXPRAT AND PRATFILI.CDFILIAL = RECEFILI.CDFILIAL WHERE ( RECEFILI.CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( RECEFILI.CDPRATO = P.CDPRATO ) AND ( RECEFILI.CDFILIAL = :CDFILIAL ) AND ( NVL(PRATFILI.IDATIVO, 'S') = 'S' ) AND NOT EXISTS ( SELECT CDPRATO FROM RECESERV WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL = :CDFILIAL ) AND ( CDSERVFILI = :CDSERVFILI ) ) UNION ALL SELECT DISTINCT 'PADRÃO' AS IDTIPO FROM RECEPADR LEFT JOIN PRATPADR ON PRATPADR.CDPRATO = RECEPADR.CDPRATO AND PRATPADR.CDFILAUXPRAT = RECEPADR.CDFILAUXPRAT WHERE ( RECEPADR.CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( RECEPADR.CDPRATO = P.CDPRATO ) AND ( :IDUTRPADCA = 'S' ) AND ( NVL(PRATPADR.IDATIVO, 'S') = 'S' ) AND NOT EXISTS ( SELECT CDPRATO FROM RECEFILI WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL = :CDFILIAL ) ) AND NOT EXISTS ( SELECT CDPRATO FROM RECESERV WHERE ( CDFILAUXPRAT = P.CDFILAUXPRAT ) AND ( CDPRATO = P.CDPRATO ) AND ( CDFILIAL = :CDFILIAL ) AND ( CDSERVFILI = :CDSERVFILI ) ) ) AS RECEITA, P.CDPRATO, P.NMPRATO, TO_CHAR(P.DTULTATURECE, 'DD/MM/YYYY HH24:MI') AS DTULTATURECE FROM PRATO P LEFT JOIN ( SELECT G.NRGRUPOPRATO, IT.CDPRATO, IT.CDFILAUXPRAT FROM GRUPOPRATO G INNER JOIN GRUPOPRATOSERV GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO AND G.CDFILIAL = :CDFILIAL INNER JOIN ITGRUPOPRATOSERV IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV ) G ON G.CDFILAUXPRAT = P.CDFILAUXPRAT AND G.CDPRATO = P.CDPRATO WHERE ( G.NRGRUPOPRATO IS NULL ) AND ( P.CDFILAUXPRAT = 'P' OR P.CDFILAUXPRAT = :CDFILIAL ) AND ( P.CDNVPRATO = :CDNVPRATO ) AND ( P.IDATIVO = 'S' ) ) WHERE ( RECEITA IS NOT NULL ) ORDER BY CDPRATO",
            "PLATEGROUPS" => "SELECT AG.NRAGRUPGRPRAT, AG.CDAGRUPGRPRAT, AG.NMAGRUPGRPRAT, GP.NRGRUPOPRATO, GP.CDGRUPOPRATO, GP.NMGRUPOPRATO, GS.NRGRUPOPRATOSERV FROM GRUPOPRATO GP INNER JOIN GRUPOPRATOSERV GS ON GP.NRGRUPOPRATO = GS.NRGRUPOPRATO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND GP.IDTIPORESTRICAO = AG.IDTIPORESTRICAO WHERE GP.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI AND GP.IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY AG.CDAGRUPGRPRAT, GP.CDGRUPOPRATO",
            "EXPORTPLATEGROUPS" => "SELECT AG.CDAGRUPGRPRAT, AG.NMAGRUPGRPRAT, GP.CDGRUPOPRATO, GP.NMGRUPOPRATO, DECODE(I.CDFILAUXPRAT,'P','Padrão','Local') AS ARVOREPRATO, I.CDPRATO AS CDPRATO, P.NMPRATO AS NMPRATO, DECODE(RECEITA_UTILIZADA(I.CDPRATO,I.CDFILAUXPRAT,:CDFILIAL,:CDSERVFILI),'P','Padrão','F','Local','S','Por Serviço', 'Sem Receita') AS TIPO_RECEITA, DECODE(PRATO_RECEITA_ATIVA(I.CDPRATO,I.CDFILAUXPRAT,:CDFILIAL,:CDSERVFILI, NULL),'S','Sim','Não') AS ATIVO, TO_CHAR(P.DTULTATURECE,'DD/MM/YYYY HH24:MI') AS DTULTATURECE FROM GRUPOPRATO GP INNER JOIN GRUPOPRATOSERV GS ON GP.NRGRUPOPRATO = GS.NRGRUPOPRATO LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND GP.IDTIPORESTRICAO = AG.IDTIPORESTRICAO INNER JOIN ITGRUPOPRATOSERV I ON I.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV JOIN PRATO P ON I.CDPRATO = P.CDPRATO AND I.CDFILAUXPRAT = P.CDFILAUXPRAT AND I.NRORG = P.NRORG WHERE GP.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI AND GP.IDTIPORESTRICAO = :IDTIPORESTRICAO ORDER BY AG.CDAGRUPGRPRAT, GP.CDGRUPOPRATO, I.CDPRATO",
            "PLATES_BY_GROUP" => "SELECT I.NRITGRUPOPRATOSERV, I.CDFILAUXPRAT, I.CDPRATO, P.NMPRATO, DECODE(RECEITA_UTILIZADA(I.CDPRATO,I.CDFILAUXPRAT,:CDFILIAL,:CDSERVFILI),'P','Padrão','F','Local','S','Por Serviço', 'Sem Receita') AS TIPO_RECEITA, PRATO_RECEITA_ATIVA(I.CDPRATO,I.CDFILAUXPRAT,:CDFILIAL,:CDSERVFILI, NULL) AS PRATOATIVO, TO_CHAR(P.DTULTATURECE,'DD/MM/YYYY HH24:MI') AS DTULTATURECE, DECODE(I.CDFILAUXPRAT,'P','P','L') AS ARVOREPRATO, IDUTRPADCA FROM ITGRUPOPRATOSERV I JOIN PRATO P ON I.CDPRATO = P.CDPRATO AND I.CDFILAUXPRAT = P.CDFILAUXPRAT AND I.NRORG = P.NRORG CROSS JOIN PARAMGERAL PG WHERE I.NRGRUPOPRATOSERV =:NRGRUPOPRATOSERV ORDER BY I.CDPRATO",
            "PLATE_RECIPE" => "SELECT P.CDARVPROD, P.NMPRODUTO, P.CDPRODUTO, REC.IDTIPORECEITA, NVL(REC.SGUNIDADE,P.SGUNIDADE) AS SGUNIDADE, ROUND(REC.QTPRODUTO,5)QTPRODUTO FROM(SELECT RS.CDPRODUTO, QTPRODRECSER AS QTPRODUTO, RS.SGUNIDADE, 'S' AS IDTIPORECEITA FROM RECESERV RS WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT UNION ALL SELECT RF.CDPRODUTO, QTPRODRECLOC AS QTPRODUTO, RF.SGUNIDADE, 'L' AS IDTIPORECEITA FROM RECEFILI RF WHERE CDFILIAL = :CDFILIAL AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND NOT EXISTS (SELECT RS.CDPRODUTO FROM RECESERV RS WHERE RS.CDPRATO = RF.CDPRATO AND RS.CDFILAUXPRAT = RF.CDFILAUXPRAT AND RS.CDFILIAL = :CDFILIAL AND RS.CDSERVFILI = :CDSERVFILI) UNION ALL SELECT RP.CDPRODUTO, QTPRODPRAT AS QTPRODUTO, RP.SGUNIDADE, 'P' AS IDTIPORECEITA FROM RECEPADR RP WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND NOT EXISTS (SELECT RS.CDPRODUTO FROM RECESERV RS WHERE RS.CDPRATO = RP.CDPRATO AND RS.CDFILAUXPRAT = RP.CDFILAUXPRAT AND RS.CDFILIAL = :CDFILIAL AND RS.CDSERVFILI = :CDSERVFILI ) AND NOT EXISTS (SELECT RF.CDPRODUTO FROM RECEFILI RF WHERE RF.CDFILIAL = :CDFILIAL AND RF.CDPRATO = :CDPRATO AND RF.CDFILAUXPRAT = :CDFILAUXPRAT ) ) REC INNER JOIN PRODUTO P ON P.CDPRODUTO = REC.CDPRODUTO ORDER BY CDARVPROD",
            "GROUPS_TO_ADD" => "SELECT P.CDPADGRUPOPRATO, P.CDPADGRUPOPRATO AS CDGRUPOPRATO, P.NMPADGRUPOPRATO FROM PADGRUPOPRATO P WHERE P.IDTIPORESTRICAO = :IDTIPORESTRICAO AND NOT EXISTS ( SELECT G.CDGRUPOPRATO FROM GRUPOPRATO       G INNER JOIN GRUPOPRATOSERV   GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI WHERE                                         G.CDGRUPOPRATO = P.CDPADGRUPOPRATO AND G.IDTIPORESTRICAO =  P.IDTIPORESTRICAO ) ORDER BY P.CDPADGRUPOPRATO",
            "GROUPSET_TO_ADD" => "SELECT NRAGRUPGRPRAT, CDAGRUPGRPRAT, NMAGRUPGRPRAT FROM AGRUPGRPRAT WHERE IDTIPORESTRICAO = :IDTIPORESTRICAO",
            "PLATES_TO_ADD" => "SELECT CDFILAUXPRAT, CDFILAUXPRAT AS ARVOREPRATO, CDPRATO, NMPRATO, SGUNIDADE, CASE WHEN TIPORECEITA = 'P' THEN 'PADRÃO' WHEN TIPORECEITA = 'F' THEN 'LOCAL' WHEN TIPORECEITA = 'S' THEN 'POR SERVIÇO' ELSE 'SEM RECEITA' END AS TIPO_RECEITA, TO_CHAR(P.DTULTATURECE, 'DD/MM/YYYY HH24:MI') AS DTULTATURECE FROM ( SELECT DISTINCT PRATO.CDFILAUXPRAT, PRATO.CDPRATO, PRATO.NMPRATO, PRATO.SGUNIDADE, ( SELECT DISTINCT 'S' AS IDTIPO FROM RECESERV LEFT JOIN PRATSERV ON PRATSERV.CDPRATO = RECESERV.CDPRATO AND PRATSERV.CDFILAUXPRAT = RECESERV.CDFILAUXPRAT AND PRATSERV.CDFILIAL = RECESERV.CDFILIAL AND PRATSERV.CDSERVFILI = RECESERV.CDSERVFILI WHERE ( RECESERV.CDFILAUXPRAT = PRATO.CDFILAUXPRAT ) AND ( RECESERV.CDPRATO = PRATO.CDPRATO ) AND ( RECESERV.CDFILIAL = :CDFILIAL ) AND ( RECESERV.CDSERVFILI = :CDSERVFILI ) AND ( NVL(PRATSERV.IDATIVO, 'S') = 'S' ) UNION ALL SELECT DISTINCT 'F' AS IDTIPO FROM RECEFILI LEFT JOIN PRATFILI ON PRATFILI.CDPRATO = RECEFILI.CDPRATO AND PRATFILI.CDFILAUXPRAT = RECEFILI.CDFILAUXPRAT AND PRATFILI.CDFILIAL = RECEFILI.CDFILIAL WHERE ( RECEFILI.CDFILAUXPRAT = PRATO.CDFILAUXPRAT ) AND ( RECEFILI.CDPRATO = PRATO.CDPRATO ) AND ( RECEFILI.CDFILIAL = :CDFILIAL ) AND ( NVL(PRATFILI.IDATIVO, 'S') = 'S' ) AND NOT EXISTS ( SELECT CDPRATO FROM RECESERV WHERE ( CDFILAUXPRAT = PRATO.CDFILAUXPRAT ) AND ( CDPRATO = PRATO.CDPRATO ) AND ( CDFILIAL = :CDFILIAL ) AND ( CDSERVFILI = :CDSERVFILI ) ) UNION ALL SELECT DISTINCT 'P' AS IDTIPO FROM RECEPADR LEFT JOIN PRATPADR ON PRATPADR.CDPRATO = RECEPADR.CDPRATO AND PRATPADR.CDFILAUXPRAT = RECEPADR.CDFILAUXPRAT WHERE ( RECEPADR.CDFILAUXPRAT = PRATO.CDFILAUXPRAT ) AND ( RECEPADR.CDPRATO = PRATO.CDPRATO ) AND ( :IDUTRPADCA = 'S' ) AND ( NVL(PRATPADR.IDATIVO, 'S') = 'S' ) AND NOT EXISTS ( SELECT CDPRATO FROM RECEFILI WHERE ( CDFILAUXPRAT = PRATO.CDFILAUXPRAT ) AND ( CDPRATO = PRATO.CDPRATO ) AND ( CDFILIAL = :CDFILIAL ) ) AND NOT EXISTS ( SELECT CDPRATO FROM RECESERV WHERE ( CDFILAUXPRAT = PRATO.CDFILAUXPRAT ) AND ( CDPRATO = PRATO.CDPRATO ) AND ( CDFILIAL = :CDFILIAL ) AND ( CDSERVFILI = :CDSERVFILI ) ) ) AS TIPORECEITA, DTULTATURECE FROM PRATO LEFT JOIN ( SELECT GS.NRGRUPOPRATOSERV, G.NMGRUPOPRATO, IT.NRITGRUPOPRATOSERV, IT.CDPRATO, IT.CDFILAUXPRAT FROM GRUPOPRATO         G INNER JOIN GRUPOPRATOSERV     GS ON G.NRGRUPOPRATO = GS.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :IDTIPORESTRICAO AND G.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI INNER JOIN ITGRUPOPRATOSERV   IT ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV ) G ON G.CDFILAUXPRAT = PRATO.CDFILAUXPRAT AND G.CDPRATO = PRATO.CDPRATO WHERE ( G.NRITGRUPOPRATOSERV IS NULL ) AND ( PRATO.CDFILAUXPRAT = 'P' OR PRATO.CDFILAUXPRAT = :CDFILIAL ) AND ( PRATO.CDNVPRATO = :CDNVPRATO ) AND ( PRATO.IDATIVO = 'S' ) AND ( :PRATO IS NULL OR UPPER(PRATO.CDPRATO) LIKE UPPER(:PRATO) || '%' OR UPPER(PRATO.NMPRATO) LIKE '%' || UPPER(:PRATO) || '%' ) ORDER BY PRATO.CDPRATO ) P WHERE ( :MAX_RESULTS IS NULL OR ROWNUM <= :MAX_RESULTS ) AND TIPORECEITA IS NOT NULL",
            "SIMILAR_PLATES" => "SELECT DISTINCT PF.IDCOMPLEXIDADE, PF.IDPREPREPARO, PF.IDCOR, PF.NRINGREDIENTEPRINC1, ING1.NMINGREDIENTEPRINC AS NMINGREDIENTEPRINC1, PF.NRINGREDIENTEPRINC2, ING2.NMINGREDIENTEPRINC AS NMINGREDIENTEPRINC2, PF.CDAPRERECE, AP.NMAPRERECE, PF.NRFORMAPREPARO, FP.NMFORMAPREPARO FROM PARAMPRATFILIAL PF INNER JOIN PRATO P ON P.CDPRATO = PF.CDPRATO AND P.CDFILAUXPRAT = PF.CDFILAUXPRAT LEFT JOIN INGREDIENTEPRINC ING1 ON PF.NRINGREDIENTEPRINC1 = ING1.NRINGREDIENTEPRINC LEFT JOIN INGREDIENTEPRINC ING2 ON PF.NRINGREDIENTEPRINC2 = ING2.NRINGREDIENTEPRINC LEFT JOIN ATRATIVIDADEPRATO AT ON AT.NRATRATIVIDADEPRATO = PF.NRATRATIVIDADEPRATO LEFT JOIN FORMAPREPARO FP ON FP.NRFORMAPREPARO = PF.NRFORMAPREPARO LEFT JOIN APRERECE AP ON AP.CDAPRERECE = PF.CDAPRERECE WHERE PF.CDFILIAL <> :CDFILIAL AND PF.CDPRATO = :CDPRATO AND P.NMPRATO = :NMPRATO",
            "CONSUMPTION_HISTORY" => "SELECT H.NRHISTCONSUMOPRATO, H.CDFILAUXPRAT, H.CDPRATO, P.NMPRATO, VRPECONSCARD, MANUTENCAO FROM(SELECT NULL AS NRHISTCONSUMOPRATO, CP.CDFILAUXPRAT, CP.CDPRATO, ROUND( AVG(CP.VRPECONSCARD), 2) AS VRPECONSCARD, 'N' AS MANUTENCAO FROM COMPCARD CP INNER JOIN CARDADIA C ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO INNER JOIN SERVICO S ON S.CDFILIAL = C.CDFILIAL AND S.CDSERVFILI = C.CDSERVFILI WHERE C.CDFILIAL = :CDFILIAL AND C.CDSERVFILI = :CDSERVFILI AND(C.DTMESCARD BETWEEN ADD_MONTHS(TO_DATE(:DTMESCARD),(NVL(NRMESHISTCONS,6)*-1)) AND TO_DATE(:DTMESCARD)) AND NOT EXISTS (SELECT H.CDPRATO FROM HISTCONSUMOPRATO H WHERE H.CDFILIAL =:CDFILIAL AND H.CDSERVFILI =:CDSERVFILI AND H.CDFILAUXPRAT = CP.CDFILAUXPRAT AND H.CDPRATO = CP.CDPRATO) GROUP BY CP.CDFILAUXPRAT, CP.CDPRATO UNION ALL SELECT H.NRHISTCONSUMOPRATO, H.CDFILAUXPRAT, H.CDPRATO, H.VRPECONSCARD, 'S' AS MANUTENCAO FROM HISTCONSUMOPRATO H WHERE H.CDFILIAL = :CDFILIAL AND H.CDSERVFILI = :CDSERVFILI ) H INNER JOIN PRATO P ON P.CDFILAUXPRAT = H.CDFILAUXPRAT AND P.CDPRATO = H.CDPRATO WHERE PRATO_RECEITA_ATIVA(P.CDPRATO,P.CDFILAUXPRAT,:CDFILIAL, :CDSERVFILI) = 'S' ORDER BY H.CDPRATO",
            "PRATO_HIST_CONSUMO" => "SELECT CDFILAUXPRAT, CDPRATO, NMPRATO FROM PRATO WHERE(CDFILAUXPRAT = 'P' OR CDFILAUXPRAT = :CDFILIAL) AND CDNVPRATO =( SELECT MAX(CDNVPRATO) FROM PRATO) AND (:PRATO IS NULL OR UPPER(CDPRATO) LIKE UPPER(:PRATO) || '%' OR UPPER(NMPRATO) LIKE '%' || UPPER(:PRATO) || '%' ) AND ( :MAX_RESULTS IS NULL OR ROWNUM <= :MAX_RESULTS ) AND NOT EXISTS (SELECT CDPRATO, CDFILAUXPRAT FROM HISTCONSUMOPRATO H WHERE H.CDPRATO = PRATO.CDPRATO AND H.CDFILAUXPRAT = PRATO.CDFILAUXPRAT AND H.CDFILIAL = :CDFILIAL AND H.CDSERVFILI = :CDSERVFILI ) AND PRATO_RECEITA_ATIVA(PRATO.CDPRATO,PRATO.CDFILAUXPRAT,:CDFILIAL, :CDSERVFILI) = 'S' ORDER BY CDPRATO",
            "ACCESS_PROFILE" => "SELECT P.NRPERFILACESSO, P.NMPERFILACESSO, P.NRORG FROM PERFILACESSO P WHERE P.NRORG = :NRORG AND NOT EXISTS (SELECT A.NRPERFILACESSO FROM AUTPERFILACESSO A WHERE A.NRPERFILACESSO = P.NRPERFILACESSO AND A.IDAUTPERFACESSO = 'OTIMIZADORCARD') ORDER BY NRPERFILACESSO",
            "ACCESS_PROFILE_PERMISSION" => "SELECT OI.NROTIMIZADORITEM, OI.DSOTIMIZADORITEM, OP.NROTIMIZADORITEMPER, CASE WHEN OP.NROTIMIZADORITEMPER IS NULL THEN 'N' ELSE 'S' END AS ATIVO FROM OTIMIZADORITEM OI LEFT JOIN OTIMIZADORITEMPER OP ON OI.NROTIMIZADORITEM = OP.NROTIMIZADORITEM AND OI.NRORG = OP.NRORG AND OP.NRPERFILACESSO = :NRPERFILACESSO WHERE OI.NRORG = :NRORG ORDER BY OI.NRORDEMITEM",
            "USERS_BY_ACCESS_PROFILE" => "SELECT OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR FROM PERFACESSPARC JOIN OPERADOR ON(PERFACESSPARC.NRPARCNEGOCIO = OPERADOR.NRPARCNEGOCIO AND PERFACESSPARC.NRORG = OPERADOR.NRORG) WHERE PERFACESSPARC.NRPERFILACESSO = :NRPERFILACESSO AND PERFACESSPARC.NRORG = :NRORG ORDER BY OPERADOR.CDOPERADOR",
            "PORTIONING_SET" => "SELECT C.NRCONJUNTOPORCAO, C.CDGRTIPOPR, G.NMGRTIPOPR, C.VRCONJUNTOPORCAO FROM CONJUNTOPORCAO C INNER JOIN GRTIPOPR G ON G.CDGRTIPOPR = C.CDGRTIPOPR WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI ORDER BY C.CDGRTIPOPR",
            "DAYS_EVENT_OUT_OF_RUT" => "SELECT NRCARDIAEVENTO, CDFILIAL, CDSERVFILI, DTEVENTO, QTCLIESTICAR, QTDOAESTICAR, QTFUNESTICAR FROM CARDIAEVENTO WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI",
            "LINK_TYPE_DISHES_BETWEEN_SERVICES" => "SELECT TI.NRTIPOPRATCOMUMSERV, TI.CDFILIAL, TI.CDSERVFILI, C1.CDTIPOPRAT, TP1.NMTIPOPRAT, C1.NRORDCNFCARD, TI.NRSEQCNFCARD, TI.CDFILIALORIG, TI.CDSERVORIG ||' - ' || S.NMSERVICO AS CDNMSERVORIG,TI.CDSERVORIG, S.NMSERVICO, TI.NRSEQCNFCARDORIG, C2.CDTIPOPRAT AS CDTIPOPRATORIG, TP2.NMTIPOPRAT AS NMTIPOPRATORIG, C2.NRORDCNFCARD AS NRORDEMORIG FROM TIPOPRATCOMUMSERV TI INNER JOIN CONFCARD C1 ON C1.CDFILIAL = TI.CDFILIAL AND C1.CDSERVFILI = TI.CDSERVFILI AND C1.NRSEQCNFCARD = TI.NRSEQCNFCARD INNER JOIN CONFCARD C2 ON C2.CDFILIAL = TI.CDFILIALORIG AND C2.CDSERVFILI = TI.CDSERVORIG AND C2.NRSEQCNFCARD = TI.NRSEQCNFCARDORIG INNER JOIN SERVICO S ON S.CDFILIAL = TI.CDFILIALORIG AND S.CDSERVFILI = TI.CDSERVORIG INNER JOIN TIPOPRAT TP1 ON TP1.CDTIPOPRAT = C1.CDTIPOPRAT INNER JOIN TIPOPRAT TP2 ON TP2.CDTIPOPRAT = C2.CDTIPOPRAT WHERE TI.CDFILIAL = :CDFILIAL AND TI.CDSERVFILI = :CDSERVFILI ORDER BY C1.NRORDCNFCARD, C2.NRORDCNFCARD",
            "DESTINE_DISHES" => "SELECT TP.CDTIPOPRAT, TP.NMTIPOPRAT, C.NRSEQCNFCARD, C.NRORDCNFCARD FROM CONFCARD C INNER JOIN TIPOPRAT TP ON C.CDTIPOPRAT = TP.CDTIPOPRAT WHERE (C.CDFILIAL = :CDFILIAL) AND (C.CDSERVFILI = :CDSERVFILI) AND (C.CDTIPOPRAT LIKE NVL(:TIPOPRAT,'%')||'%' OR UPPER(TP.NMTIPOPRAT) LIKE '%'||UPPER(:TIPOPRAT)||'%') AND NOT EXISTS(SELECT TI.NRTIPOPRATCOMUMSERV FROM TIPOPRATCOMUMSERV TI WHERE TI.CDFILIAL = C.CDFILIAL AND TI.CDSERVFILI = C.CDSERVFILI AND TI.NRSEQCNFCARD = C.NRSEQCNFCARD) ORDER BY C.NRORDCNFCARD",
            "ORIGIN_DISHES" => "SELECT TP.CDTIPOPRAT AS CDTIPOPRATORIG, TP.NMTIPOPRAT AS NMTIPOPRATORIG, C.NRSEQCNFCARD AS NRSEQCNFCARDORIG, C.NRORDCNFCARD AS NRORDEMORIG FROM CONFCARD C INNER JOIN TIPOPRAT TP ON C.CDTIPOPRAT = TP.CDTIPOPRAT WHERE (C.CDFILIAL = :CDFILIAL) AND (C.CDSERVFILI = :CDSERVORIG) AND (C.CDTIPOPRAT LIKE NVL(:TIPOPRAT,'%')||'%' OR UPPER(TP.NMTIPOPRAT) LIKE '%'||UPPER(:TIPOPRAT)||'%') ORDER BY C.NRORDCNFCARD",
            "ORIGIN_SERVICE" => "SELECT S.CDSERVFILI, S.NMSERVICO, S.CDSERVFILI || ' - ' || S.NMSERVICO AS CDNMSERVORIG FROM SERVICO S WHERE S.IDOTIMIZADOR = 'S' AND ( S.CDFILIAL = :CDFILIAL ) AND ( S.CDSERVFILI <> :CDSERVFILI ) AND ( S.CDSERVFILI LIKE NVL(:SERVICO, '%') || '%' OR UPPER(S.NMSERVICO) LIKE '%' || UPPER(:SERVICO) || '%' ) AND ( S.CDSERVFILI = ( SELECT MAX(CDSERVORIG) FROM TIPOPRATCOMUMSERV T WHERE T.CDFILIAL = S.CDFILIAL AND T.CDSERVORIG = S.CDSERVFILI ) OR NOT EXISTS ( SELECT NRTIPOPRATCOMUMSERV FROM TIPOPRATCOMUMSERV T WHERE T.CDFILIAL = S.CDFILIAL AND T.CDSERVFILI = :CDSERVFILI ) ) ORDER BY S.CDSERVFILI",
            "GET_LINK_COMMON_DISHES" => "SELECT CN.NRCENARIOCARD, '- Gerado em: '|| TO_CHAR(DTCENARIOCARD, 'DD/MM/YYYY') AS DTCENARIOCARD FROM CENARIOCARD CN WHERE CN.CDFILIAL = :CDFILIAL AND CN.DTMESCARD = :DTMESCARD AND CN.IDTIPORESTRICAO = :IDTIPORESTRICAO AND CN.CDSERVFILI = (SELECT MAX(CDSERVORIG) FROM TIPOPRATCOMUMSERV WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI) ORDER BY NRCENARIOCARD DESC",
            "GRUPOTIPOPRATO_NOT_IN_PORTIONING_SET" => "SELECT CDGRTIPOPR, NMGRTIPOPR FROM GRTIPOPR WHERE (:MAX_RESULTS IS NULL OR ROWNUM <=:MAX_RESULTS) AND (:GRUPOPRATO IS NULL OR UPPER(CDGRTIPOPR) LIKE UPPER(:GRUPOPRATO)|| '%' OR UPPER(NMGRTIPOPR) LIKE '%'||UPPER(:GRUPOPRATO)||'%') AND NOT EXISTS (SELECT C.CDGRTIPOPR FROM CONJUNTOPORCAO C WHERE C.CDFILIAL = :CDFILIAL AND C.CDSERVFILI = :CDSERVFILI AND C.CDGRTIPOPR = GRTIPOPR.CDGRTIPOPR) ORDER BY CDGRTIPOPR",

            "REGRAS_RESTRICAO_REPETICAO" => "SELECT NRREGRARESTREPET, NMREGRARESTREPET FROM REGRARESTREPET WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI ORDER BY NRREGRARESTREPET",
            "TIPOPRATO_POR_REGRAS_RESTRICAO" => "SELECT T.NRRESTRICAOTPPRAT, T.CDTIPOPRAT, TP.NMTIPOPRAT FROM RESTRICAOTPPRAT T INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = T.CDTIPOPRAT WHERE T.NRREGRARESTREPET = :NRREGRARESTREPET ORDER BY T.CDTIPOPRAT",
            "TIPOPRATO_POR_REGRAS_RESTRICAO_AUTOCOMPLETE" => "SELECT TP.CDTIPOPRAT, TP.NMTIPOPRAT FROM TIPOPRAT TP WHERE(CDTIPOPRAT LIKE NVL(:TIPOPRAT,'%')||'%' OR UPPER(NMTIPOPRAT) LIKE '%'||UPPER(:TIPOPRAT)||'%') AND NOT EXISTS(SELECT T.CDTIPOPRAT FROM RESTRICAOTPPRAT T INNER JOIN REGRARESTREPET R ON T.NRREGRARESTREPET = R.NRREGRARESTREPET WHERE R.CDFILIAL = :CDFILIAL AND R.CDSERVFILI = :CDSERVFILI AND T.CDTIPOPRAT = TP.CDTIPOPRAT) ORDER BY TP.CDTIPOPRAT",
            "INGREDIENTE_REGRAS_RESTRICAO" => "SELECT I.NRRESTRICAOINGRED, I.NRINGREDIENTEPRINC, NVL(ING.NMINGREDIENTEPRINC,'TODOS') AS NMINGREDIENTEPRINC, I.IDINGREDIENTE1, I.IDINGREDIENTE2, IDMESMODIA, IDDIASEGUIDO FROM RESTRICAOINGRED I LEFT JOIN INGREDIENTEPRINC ING ON ING.NRINGREDIENTEPRINC = I.NRINGREDIENTEPRINC WHERE I.NRREGRARESTREPET = :NRREGRARESTREPET ORDER BY NVL(ING.NMINGREDIENTEPRINC,'AAAAA')",
            "INGREDIENTE_REGRAS_RESTRICAO_AUTOCOMPLETE" => "SELECT NULL AS NRINGREDIENTEPRINC, 'TODOS' AS NMINGREDIENTEPRINC FROM DUAL UNION ALL SELECT ING.NRINGREDIENTEPRINC, ING.NMINGREDIENTEPRINC FROM INGREDIENTEPRINC ING WHERE UPPER(NMINGREDIENTEPRINC) LIKE '%'||UPPER(:NMINGREDIENTEPRINC)||'%' AND NOT EXISTS(SELECT I.NRINGREDIENTEPRINC FROM RESTRICAOINGRED I WHERE I.NRREGRARESTREPET = :NRREGRARESTREPET AND I.NRINGREDIENTEPRINC = ING.NRINGREDIENTEPRINC) ORDER BY NMINGREDIENTEPRINC",
            "EXPORT_RESTRICTION" => "SELECT R.NRREGRARESTREPET, R.NMREGRARESTREPET, T.CDTIPOPRAT, TP.NMTIPOPRAT, NVL(ING.NMINGREDIENTEPRINC,'TODOS') AS NMINGREDIENTEPRINC, DECODE(I.IDINGREDIENTE1,'S','Não Repetir','Repetir') AS IDINGREDIENTE1, DECODE(I.IDINGREDIENTE2,'S','Não Repetir','Repetir') AS IDINGREDIENTE2, DECODE(IDDIASEGUIDO,'S','Considerar','Não Considerar') AS IDDIASEGUIDO FROM REGRARESTREPET R INNER JOIN RESTRICAOTPPRAT T ON T.NRREGRARESTREPET = R.NRREGRARESTREPET INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = T.CDTIPOPRAT INNER JOIN RESTRICAOINGRED I ON I.NRREGRARESTREPET = R.NRREGRARESTREPET LEFT JOIN INGREDIENTEPRINC ING ON ING.NRINGREDIENTEPRINC = I.NRINGREDIENTEPRINC WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI ORDER BY R.NRREGRARESTREPET, T.CDTIPOPRAT, NVL(ING.NMINGREDIENTEPRINC,'AAAAA')",

            "CONSUMPTION_HISTORY_INTERVAL" => "SELECT NVL(NRMESHISTCONS,6) AS NRMESHISTCONS FROM SERVICO WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI",
            "GET_CDGRUPPROD_PARAMGERAL" => "SELECT CDGRUPPRODOTMCARD FROM PARAMGERAL",
            "GET_CDGRUPPROD" => "SELECT CDGRUPPROD, NMGRUPPROD FROM GRUPPROD ORDER BY CDGRUPPROD",
        
            "GETBATCH" => "SELECT G.NRGERACAOCARD, G.CDOPERADOR, O.NMOPERADOR, TO_CHAR(G.DTINCLUSAO,'DD/MM/YYYY') AS DTINCLUSAO, G.IDSTATUSGERACAO, TO_CHAR(G.DTMESGERACAOCARD, 'DD/MM/YYYY') AS DTMESGERACAOCARD, G.IDTIPORESTRICAO FROM GERACAOCARD G INNER JOIN OPERADOR O ON O.CDOPERADOR = G.CDOPERADOR WHERE G.DTMESGERACAOCARD = :P_DTMESCARD AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO ORDER BY G.NRGERACAOCARD DESC",
            "GETBATCHSCENARIO" => "SELECT I.NRITGERACAOCARD, I.CDFILIAL, I.CDFILIAL || ' | ' || F.NMFILIAL AS NMFILIAL, F.NMFILIAL AS NOME_FILIAL, I.CDSERVFILI, I.CDSERVFILI || ' | ' || S.NMSERVICO AS NMSERVICO, S.NMSERVICO AS NOME_SERVICO, TO_CHAR(I.DTREFERENCIA,'DD/MM/YYYY') AS DTREFERENCIA, I.CDTIPOCUSTO, TC.NMTIPOCUSTO, I.VRCUSTOPADRAO, I.VRCUSTOMAX, I.VRCUSTOMIN, I.CDGRUPPROD, GP.NMGRUPPROD, I.IDPERMITECONGELAR, I.QTDIASCONG, I.NRCENARIOVINCULO, I.IDSTATUSGERACAO, I.NRCENARIOCARD, I.NRCENARIOPUBLICAR, I.NRLOGINTCARDAMES, IDSTATUSALERTA, DTALERTA FROM ITGERACAOCARD I INNER JOIN FILIAL F ON F.CDFILIAL = I.CDFILIAL INNER JOIN SERVICO S ON S.CDFILIAL = I.CDFILIAL AND S.CDSERVFILI = I.CDSERVFILI LEFT JOIN TIPOCUSTO TC ON TC.CDTIPOCUSTO = I.CDTIPOCUSTO LEFT JOIN GRUPPROD GP ON GP.CDGRUPPROD = I.CDGRUPPROD WHERE I.NRGERACAOCARD =:P_NRLOTE ORDER BY I.CDFILIAL, I.CDSERVFILI",
            "GETGENERATIONSUMMARY" => "SELECT IDTIPOLOG, DSLOGGERACAOCARD FROM LOGITGERACAOCARD WHERE NRITGERACAOCARD = :NRITGERACAOCARD ORDER BY IDTIPOLOG, DSLOGGERACAOCARD",
            "GETPUBLICATIONSUMMARY" => "SELECT DSITEM, DSERRO FROM LOGINTCARDWS WHERE NRLOGINTCARDAMES = :NRLOGINTCARDAMES ORDER BY NRLOGINTCARDWS",
            "AGRUPFILI" => "SELECT CDGRUPFILI, NMGRUPFILI FROM GRUPFILI WHERE (:USER_QUERY IS NULL OR UPPER(CDGRUPFILI) LIKE UPPER(:USER_QUERY) || '%' OR UPPER(NMGRUPFILI) LIKE '%' || UPPER(:USER_QUERY) || '%') ORDER BY CDGRUPFILI",
            "FILIAGRUP" => "SELECT F.CDFILIAL, F.NMFILIAL FROM FILIAL F INNER JOIN FILIOPER FO ON FO.CDFILIAL = F.CDFILIAL AND FO.CDOPERADOR = :CDOPERADOR WHERE (EXISTS(SELECT FG.CDGRUPFILI FROM FILGRUPFILI FG WHERE FG.CDFILIAL = F.CDFILIAL AND INSTR(:LISTAGRUPFILI, FG.CDGRUPFILI) <> 0) OR :LISTAGRUPFILI = 'T') AND F.IDCNTRRESC = 'N' AND (:USER_QUERY IS NULL OR UPPER(F.CDFILIAL) LIKE UPPER(:USER_QUERY) || '%' OR UPPER(F.NMFILIAL) LIKE '%' || UPPER(:USER_QUERY) || '%') ORDER BY F.CDFILIAL",
            "SERVFILIAGRUP" => "SELECT S.CDFILIAL, F.NMFILIAL, S.CDSERVFILI, S.NMSERVICO FROM SERVICO S INNER JOIN FILIAL F ON F.CDFILIAL = S.CDFILIAL AND F.IDCNTRRESC = 'N' AND S.IDSERVATIV = 'S' AND S.IDTIPOSERV = 'N' INNER JOIN FILIOPER FO ON FO.CDFILIAL = F.CDFILIAL AND FO.CDOPERADOR =:CDOPERADOR WHERE ( EXISTS ( SELECT FG.CDGRUPFILI FROM FILGRUPFILI FG WHERE FG.CDFILIAL = F.CDFILIAL AND INSTR(:LISTAGRUPFILI,FG.CDGRUPFILI) <> 0 ) OR :LISTAGRUPFILI = 'T' ) AND ( ( INSTR(:LISTAFILIAL,F.CDFILIAL) <> 0 ) OR :LISTAFILIAL = 'T' ) AND (:USER_QUERY IS NULL OR UPPER(CDSERVFILI) LIKE UPPER(:USER_QUERY) || '%' OR UPPER(NMSERVICO) LIKE '%' || UPPER(:USER_QUERY) || '%' ) AND S.IDOTIMIZADOR = 'S' ORDER BY S.CDFILIAL, S.CDSERVFILI",
            "TIPOCUSTO" => "SELECT CDTIPOCUSTO, NMTIPOCUSTO FROM TIPOCUSTO TP WHERE IDUTTPCUSTO = 'S' AND (UPPER(NMTIPOCUSTO) LIKE '%'||UPPER(:NMTIPOCUSTO)||'%' OR :NMTIPOCUSTO IS NULL) ORDER BY CDTIPOCUSTO",
            "GRUPPROD" => "SELECT CDGRUPPROD, NMGRUPPROD FROM GRUPPROD WHERE (:USER_QUERY IS NULL OR UPPER(CDGRUPPROD) LIKE UPPER(:USER_QUERY) || '%' OR UPPER(NMGRUPPROD) LIKE '%' || UPPER(:USER_QUERY) || '%') ORDER BY CDGRUPPROD",
            "CENARIOVINCULO" => "SELECT CN.NRCENARIOCARD AS NRCENARIOVINCULO FROM CENARIOCARD CN WHERE CN.CDFILIAL = :CDFILIAL AND CN.DTMESCARD = :DTMESCARD AND CN.IDTIPORESTRICAO = :IDTIPORESTRICAO AND CN.CDSERVFILI = (SELECT MAX(CDSERVORIG) FROM TIPOPRATCOMUMSERV WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI) ORDER BY NRCENARIOCARD DESC",
            "CENARIOPUBLICAR" => "SELECT C.NRCENARIOCARD AS NRCENARIOPUBLICAR FROM CENARIOCARD C INNER JOIN ITGERACAOCARD I ON C.NRCENARIOCARDORIGINAL = I.NRCENARIOCARD WHERE I.NRITGERACAOCARD = :NRITGERACAOCARD",
            "CABECALHOEXPORTLOG" => "SELECT TO_CHAR(GC.DTMESGERACAOCARD, 'DD/MM/YYYY') AS DTMESGERACAOCARD, GC.IDTIPORESTRICAO, FI.CDFILIAL, FI.NMFILIAL, SE.CDSERVFILI, SE.NMSERVICO, IT.NRCENARIOCARD FROM ITGERACAOCARD IT INNER JOIN GERACAOCARD GC ON GC.NRGERACAOCARD = IT.NRGERACAOCARD INNER JOIN FILIAL FI ON FI.CDFILIAL = IT.CDFILIAL INNER JOIN SERVICO SE ON SE.CDFILIAL = IT.CDFILIAL AND SE.CDSERVFILI = IT.CDSERVFILI WHERE NRITGERACAOCARD = :NRITGERACAOCARD",
            "AGRUPPROD" => "SELECT CDGRUPPROD, NMGRUPPROD FROM (SELECT G.CDGRUPPROD, G.NMGRUPPROD, CASE WHEN P.CDGRUPPRODOTMCARD IS NOT NULL THEN 0 ELSE 1 END AS ORDEM FROM GRUPPROD G LEFT JOIN PARAMGERAL P ON P.CDGRUPPRODOTMCARD = G.CDGRUPPROD) GP ORDER BY ORDEM, CDGRUPPROD",
            "TIPOCUSTOPLA" => "SELECT CDTIPOCUSTOPLA FROM PARAMPLANEJA WHERE CDFILIAL = :CDFILIAL",
            "GRUPPROD_PARAMGERAL" => "SELECT CDGRUPPRODOTMCARD FROM PARAMGERAL",
            "GENERATIONALERT" => "SELECT NRALERTAGERACAOCARD, IDTIPOALERTA, CODALERTA, DSALERTA FROM ALERTAGERACAOCARD WHERE NRITGERACAOCARD = :NRITGERACAOCARD",
            "LINKEDSERVICE" => "SELECT MAX(IT.NRITGERACAOCARD) AS NRITGERACAOCARD, TPC.CDFILIAL, TPC.CDSERVFILI, TPC.CDFILIALORIG, TPC.CDSERVORIG FROM ITGERACAOCARD IT INNER JOIN TIPOPRATCOMUMSERV TPC ON IT.CDFILIAL = TPC.CDFILIAL AND IT.CDSERVFILI = TPC.CDSERVFILI WHERE IT.NRGERACAOCARD = :NRGERACAOCARD GROUP BY TPC.CDFILIAL, TPC.CDSERVFILI, TPC.CDFILIALORIG, TPC.CDSERVORIG"
        );

        if($param->get('FROM') == 'GET_VALIDITY') {
            $SQL['GET_VALIDITY'] = "SELECT NRVIGENCIA, TO_CHAR(DTVIGENCIA,'DD/MM/YYYY') AS DTVIGENCIA, NVL(DESCRICAO, ' ') AS DESCRICAO FROM " . $param->get('PARAMS')["P_TABLENAME"] . " WHERE CDFILIAL = :P_CDFILIAL AND IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND NRVIGENCIA <> :P_NRVIGENCIA ORDER BY DTVIGENCIA";
            $p = $param->get('PARAMS');
            unset($p["P_TABLENAME"]);
            return $this->execOutSql($p, $SQL['GET_VALIDITY']);
        }

        if ($param->get('FROM') == 'SERVCONFIG') {
            return $this->getServConfig($param->get('PARAMS'));
        } else if($param->get('FROM') == 'MONTHSERVCONFIG') {
            return $this->getMonthServConfig($param->get('PARAMS'));
        } else {
            $from = $SQL[$param->get('FROM')];
            return $this->execOutSql($param->get('PARAMS'), $from);
        }
    }

    public function save($param)
    {
        $SQL = array(
            'CENARIO' => "DECLARE V_SEQUENCIAL VARCHAR2(20); BEGIN NOVO_CODIGO('CENARIOCARD', V_SEQUENCIAL,1,1); INSERT INTO CENARIOCARD(NRCENARIOCARD, DTCENARIOCARD, CDFILIAL, CDSERVFILI, DTMESCARD, CDOPERADOR, JSONCENARIO, JSONCARDAPIO, VRCUSTPADR, VRPUCENARIO, NRORG, IDTIPORESTRICAO, QTCOMENSAIS, NRTIPOCUSTO, DTREFECUSTO, NRCENARIOVINCULO) VALUES (V_SEQUENCIAL, SYSDATE, :CDFILIAL, :CDSERVFILI, :DTMESCARD, :CDOPERADOR, :JSONCENARIO, :JSONCARDAPIO, :VRCUSTPADR, :VRPUCENARIO, :NRORG, :IDTPORESTRICAO, :QTCOMENSAIS, :NRTIPOCUSTO, :DTREFECUSTO, :NRCENARIOVINCULO); END;",

            'TPCORTE' => "INSERT INTO APRERECE (CDAPRERECE, NMAPRERECE, CDOPERINCLUSAO) VALUES(:CDAPRERECE, :NMAPRERECE, :CDOPERADOR)",
            'INGRPRINC' => function ($p) {
                $query = "INSERT INTO INGREDIENTEPRINC (NRINGREDIENTEPRINC, NMINGREDIENTEPRINC, CDOPERINCLUSAO) VALUES (:NRINGREDIENTEPRINC, :NMINGREDIENTEPRINC, :CDOPERADOR)";
                $newID = $this->_insertWithNewCode($p[0], $query, "INGREDIENTEPRINC", "NRINGREDIENTEPRINC");
                return [$newID];
            },
            'MDPREPARO' => function ($p) {
                $query = "INSERT INTO FORMAPREPARO (NRFORMAPREPARO, NMFORMAPREPARO, CDOPERINCLUSAO) VALUES(:NRFORMAPREPARO, :NMFORMAPREPARO, :CDOPERADOR)";
                $newID = $this->_insertWithNewCode($p[0], $query, "FORMAPREPARO", "NRFORMAPREPARO");
                return [$newID];
            },
            'ATRATIVIDADE' => function ($p) {
                $query = "INSERT INTO ATRATIVIDADEPRATO (NRATRATIVIDADEPRATO, NMATRATIVIDADEPRATO, VRPESOATRATIVIDADE, CDFILIAL, CDOPERINCLUSAO) VALUES(:NRATRATIVIDADEPRATO, :NMATRATIVIDADEPRATO, :VRPESOATRATIVIDADE, :CDFILIAL, :CDOPERADOR)";
                $newID = $this->_insertWithNewCode($p[0], $query, "ATRATIVIDADEPRATO", "NRATRATIVIDADEPRATO");
                return [$newID];
            },
            'COMBGRUPOTPPRAT' => function ($p) {
                return $this->insertItCombGrupPrat($p);
            },
            'FREQUENCIAGRUPO' => function ($p) {
                return $this->insertPlateGroupFrequency($p);
            },
            'FREQUENCIAPRATO' => function ($p) {
                return $this->insertPlateFrequency($p);
            },
            'FREQUENCIAGRUPOPRATOPRODUTO' => function ($p) {
                return $this->insertPlateGroupProductFrequency($p);
            },
            'UTILIZACAOPRATOSEMANA' => function ($p) {
                return $this->insertWeekdayUsage($p);
            },
            'PRODUTOGRUPOPRATORESTRICACAO' => function ($p) {
                return $this->insertProductGroupPlateRestriction($p);
            },
            'VIGENCIAGRUPOPRATO' => function($p){
                $this->insertPlateGroupValidity($p);
            },
            'VIGENCIAGRUPOPRATOPRODUTO' => function($p){
                $this->insertPlateGroupProductValidity($p);
            },
            'VIGENCIAUTILIZACAOPRATOSEMANA' => function($p){
                $this->insertWeekdayUsageValidity($p);
            },
            'VIGENCIAPRODUTOGRUPOPRATORESTRICACAO' => function($p){
                $this->insertProductGroupPlateRestrictionValidity($p);
            },
            'VIGENCIAPRATO' => function($p){
                $this->insertPlateValidity($p);
            },
            'PADGRUPOPRATO' => "INSERT INTO PADGRUPOPRATO (CDPADGRUPOPRATO, NMPADGRUPOPRATO, IDTIPORESTRICAO, CDOPERINCLUSAO) VALUES 
            (:CDPADGRUPOPRATO, :NMPADGRUPOPRATO, :IDTIPORESTRICAO, :CDOPERADOR)",
            'AGRUPGRPRAT' => function($p){
                $query = "INSERT INTO AGRUPGRPRAT (NRAGRUPGRPRAT, CDAGRUPGRPRAT, NMAGRUPGRPRAT, IDTIPORESTRICAO, CDOPERINCLUSAO) VALUES (:NRAGRUPGRPRAT, :CDAGRUPGRPRAT, :NMAGRUPGRPRAT, :IDTIPORESTRICAO, :CDOPERADOR)";
                $newID = $this->_insertWithNewCode($p[0], $query, "AGRUPGRPRAT", "NRAGRUPGRPRAT");
                return [$newID];
            },
            'PRATOSEMGRUPO' => "DECLARE V_NOVOCODIGO VARCHAR2(20); BEGIN NOVO_CODIGO('ITGRUPOPRATOSERV', V_NOVOCODIGO); INSERT INTO ITGRUPOPRATOSERV (NRITGRUPOPRATOSERV, CDFILAUXPRAT, CDPRATO, NRGRUPOPRATOSERV, CDOPERINCLUSAO) VALUES (V_NOVOCODIGO, :CDFILAUXPRAT, :CDPRATO, :NRGRUPOPRATOSERV, :CDOPERADOR); END;",
            'PLATE_GROUP' => function($params){
                return [$this->newPlateGroup($params)];
            },
            'NEW_PLATE' => function($params){
                return [$this->newPlateRegister($params)];
            },
            'PARAMPRATGRUPPRAT' => function($params){
                return [$this->insertGroupPlateinPlate($params)];
            },
            "CONSUMPTION_HISTORY" => function ($p) {
                $query = " INSERT INTO HISTCONSUMOPRATO (NRHISTCONSUMOPRATO, CDFILIAL, CDSERVFILI, CDFILAUXPRAT, CDPRATO, VRPECONSCARD) VALUES
                (:NRHISTCONSUMOPRATO, :CDFILIAL, :CDSERVFILI, :CDFILAUXPRAT, :CDPRATO, :VRPECONSCARD)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->_insertWithNewCode($p[0], $query, "HISTCONSUMOPRATO", "NRHISTCONSUMOPRATO");
                return [$newID];
            },
            "ACCESS_PROFILE_PERMISSION" => function ($p) {
                $query = "INSERT INTO OTIMIZADORITEMPER(NROTIMIZADORITEMPER, NROTIMIZADORITEM, NRPERFILACESSO, NRORG) VALUES (:NROTIMIZADORITEMPER, :NROTIMIZADORITEM, :NRPERFILACESSO, :NRORG)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->_insertWithNewCode($p[0], $query, "OTIMIZADORITEMPER", "NROTIMIZADORITEMPER", false);
                return [$newID];
            },
            "ACCESS_PROFILE_PERMISSION_BULK" => function($p){
                $query = "DECLARE V_NOVOCODIGO VARCHAR2(20); BEGIN NOVO_CODIGO('OTIMIZADORITEMPER',V_NOVOCODIGO,:QUANTIDADE); INSERT INTO OTIMIZADORITEMPER(NROTIMIZADORITEMPER, NROTIMIZADORITEM, NRPERFILACESSO, NRORG) (SELECT V_NOVOCODIGO + ROWNUM, NROTIMIZADORITEM, :NRPERFILACESSO, :NRORG FROM OTIMIZADORITEM O WHERE INSTR(:NROTIMIZADORITEM,'_'||NROTIMIZADORITEM||'_') <> 0 AND NRORG = :NRORG AND ROWNUM <= :QUANTIDADE); END;";
                unset($p[0]['CDOPERADOR']);
                $this->execInSql($p, $query);
            },
            "PORTIONING_SET" => function ($p) {
                $query = "INSERT INTO CONJUNTOPORCAO (NRCONJUNTOPORCAO, CDFILIAL, CDSERVFILI, CDGRTIPOPR, VRCONJUNTOPORCAO, NRORG) VALUES (:NRCONJUNTOPORCAO, :CDFILIAL, :CDSERVFILI, :CDGRTIPOPR, :VRCONJUNTOPORCAO, :NRORG)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->_insertWithNewCode($p[0], $query, "CONJUNTOPORCAO", "NRCONJUNTOPORCAO", false);
                return [$newID];
            },
            "REGRAS_RESTRICAO_REPETICAO" => function($p){
                $query = "INSERT INTO REGRARESTREPET(NRREGRARESTREPET, NMREGRARESTREPET, CDFILIAL, CDSERVFILI) VALUES (:NRREGRARESTREPET, :NMREGRARESTREPET, :CDFILIAL, :CDSERVFILI)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->_insertWithNewCode($p[0], $query, "REGRARESTREPET", "NRREGRARESTREPET", true);
                return [$newID];
            },
            "TIPO_PRATO_REGRAS_RESTRICAO_REPETICAO" => function($p){
                $query = "INSERT INTO RESTRICAOTPPRAT(NRRESTRICAOTPPRAT, NRREGRARESTREPET, CDTIPOPRAT) VALUES (:NRRESTRICAOTPPRAT, :NRREGRARESTREPET,:CDTIPOPRAT)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->_insertWithNewCode($p[0], $query, "RESTRICAOTPPRAT", "NRRESTRICAOTPPRAT", true);
                return [$newID];
            },
            "INGREDIENTE_REGRAS_RESTRICAO_REPETICAO" => function($p){
                $query = "INSERT INTO RESTRICAOINGRED(NRRESTRICAOINGRED, NRREGRARESTREPET, NRINGREDIENTEPRINC, IDINGREDIENTE1, IDINGREDIENTE2, IDMESMODIA, IDDIASEGUIDO) VALUES (:NRRESTRICAOINGRED,:NRREGRARESTREPET,:NRINGREDIENTEPRINC,:IDINGREDIENTE1,:IDINGREDIENTE2, :IDMESMODIA, :IDDIASEGUIDO)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->_insertWithNewCode($p[0], $query, "RESTRICAOINGRED", "NRRESTRICAOINGRED", true);
                return [$newID];
            },
            "DAYS_EVENT_OUT_OF_RUT" => function ($p) {
                $query = "INSERT INTO CARDIAEVENTO(NRCARDIAEVENTO, CDFILIAL, CDSERVFILI, DTEVENTO, QTCLIESTICAR, QTDOAESTICAR, QTFUNESTICAR) VALUES (:NRCARDIAEVENTO, :CDFILIAL, :CDSERVFILI, :DTEVENTO, :QTCLIESTICAR, :QTDOAESTICAR, :QTFUNESTICAR)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->_insertWithNewCode($p[0], $query, "CARDIAEVENTO", "NRCARDIAEVENTO", true);
                return [$newID];
            },
            "LINK_TYPE_DISHES_BETWEEN_SERVICES" => function ($p) {
                $query = "INSERT INTO TIPOPRATCOMUMSERV(NRTIPOPRATCOMUMSERV, CDFILIAL, CDSERVFILI, NRSEQCNFCARD, CDFILIALORIG, CDSERVORIG, NRSEQCNFCARDORIG, NRORG) VALUES (:NRTIPOPRATCOMUMSERV,:CDFILIAL,:CDSERVFILI, :NRSEQCNFCARD,:CDFILIALORIG,:CDSERVORIG,:NRSEQCNFCARDORIG, :NRORG)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->_insertWithNewCode($p[0], $query, "TIPOPRATCOMUMSERV", "NRTIPOPRATCOMUMSERV", false);
                return [$newID];
            },
            "INSERT_EXPORT_DATA_DAY_OUT_RUT" => function ($p) {
                $query = "INSERT INTO CARDIAEVENTO(NRCARDIAEVENTO, CDFILIAL, CDSERVFILI, DTEVENTO, QTCLIESTICAR, QTFUNESTICAR, QTDOAESTICAR, DTINCLUSAO, CDOPERINCLUSAO) (SELECT RETORNA_NOVO_CODIGO('CARDIAEVENTO',1) AS NRCARDIAEVENTO, SUBSTR(L.CDLISTA, 1,4) AS CDFILIAL, SUBSTR(L.CDLISTA, 5,5) AS CDSERVFILI, DTEVENTO, QTCLIESTICAR, QTFUNESTICAR, QTDOAESTICAR, SYSDATE, :P_CDOPERADOR FROM CARDIAEVENTO C INNER JOIN LISTA LI ON LI.NRLISTA = :P_NRLISTA AND C.NRCARDIAEVENTO = LI.CDLISTA CROSS JOIN LISTA L WHERE L.NRLISTA = :P_FILISERV AND :P_CDFILIAL = C.CDFILIAL AND :P_CDSERVFILI = C.CDSERVFILI AND NOT EXISTS (SELECT NRCARDIAEVENTO FROM CARDIAEVENTO CA WHERE CA.CDFILIAL||CA.CDSERVFILI IN (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :P_FILISERV) AND CA.DTEVENTO = C.DTEVENTO))";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->execInSql($p[0], $query);
                return [$newID];
            },
            "MONTH_SERVCONFIG_PERC_SOURCE" => function ($p) {
                $query = "INSERT INTO CONFCARDMESATIVO (NRCONFCARDMESATIVO, CDFILIAL, CDSERVFILI, NRSEQCNFCARD, IDMESATIVO) VALUES(RETORNA_NOVO_CODIGO('CONFCARDMESATIVO',1), :CDFILIAL, :CDSERVFILI, :NRSEQCNFCARD, :IDMESATIVO)";
                unset($p[0]['CDOPERADOR']);
                $newID = $this->execInSql($p[0], $query);
                return [$newID];
            }
        );

        $paramReq = $param->get('PARAMS');
        if ($param["FROM"] === 'CENARIO') {
            return $this->insertCenario($paramReq);
        } else {
            $from = $SQL[$param->get('FROM')];

            if (is_string($from)) {
                $paramReq = $param->get('PARAMS');
                $this->execInSql($paramReq, $from);
            } else if (is_callable($from)) {
                $paramReq = $param->get('PARAMS');
                return $from($paramReq);
            }
        }
    }

    public function update($param, $CDOPERADOR){
        $SQL = array(
            "QTDERECEITA" => "BEGIN ATUALIZA_QTDE_RECEITA(:CDFILIAL, :CDSERVFILI, :CDFILAUXPRAT, :CDPRATO, :CDPRODUTO, :QTDE, :TIPORECE); END;",
            'ATRATIVIDADE' => "UPDATE ATRATIVIDADEPRATO SET NMATRATIVIDADEPRATO = :NMATRATIVIDADEPRATO, VRPESOATRATIVIDADE = :VRPESOATRATIVIDADE WHERE NRATRATIVIDADEPRATO = :NRATRATIVIDADEPRATO",
            "HISTATRATIVIDADE" => "UPDATE CONFCARD SET IDUTILPERCENTCONS = :IDUTILPERCENTCONS WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND NRSEQCNFCARD = :NRSEQCNFCARD",
            "ASSOSGRUPTIPOPRATO" => "DECLARE V_SEQUENCIAL VARCHAR2(20); BEGIN NOVO_CODIGO('CONFCARDGRUPOPRATO', V_SEQUENCIAL,1,1); INSERT INTO CONFCARDGRUPOPRATO(NRCONFCARDGRUPOPRATO, CDFILIAL, CDSERVFILI, NRSEQCNFCARD, NRGRUPOPRATOSERV) VALUES (V_SEQUENCIAL, :CDFILIAL, :CDSERVFILI, :NRSEQCNFCARD, :NRGRUPOPRATOSERV); END;",
            "REMASSOSGRUPTIPOPRATO" => "DELETE FROM CONFCARDGRUPOPRATO WHERE NRCONFCARDGRUPOPRATO = :NRCONFCARDGRUPOPRATO",
            "MDPREPARO" => "UPDATE FORMAPREPARO SET NMFORMAPREPARO = :NMFORMAPREPARO WHERE NRFORMAPREPARO = :NRFORMAPREPARO",
            "TPCORTE" => "UPDATE APRERECE SET NMAPRERECE = :NMAPRERECE WHERE CDAPRERECE = :CDAPRERECE",
            "INGRPRINC" => "UPDATE INGREDIENTEPRINC SET NMINGREDIENTEPRINC = :NMINGREDIENTEPRINC WHERE NRINGREDIENTEPRINC = :NRINGREDIENTEPRINC",
            "FREQUENCIAGRUPO" => "UPDATE ITFREQGRPRATMES SET NRMINFREQGRPRATMES = :NRMINFREQGRPRATMES, NRMAXFREQGRPRATMES = :NRMAXFREQGRPRATMES, NRMAXFREQGRPRATSEM = :NRMAXFREQGRPRATSEM WHERE NRITFREQGRPRATMES = :NRITFREQGRPRATMES",
            "FREQUENCIAGRUPOPRATOPRODUTO" => "UPDATE ITFREQPRODGRPRAT SET NRMINPRODGRPRAT = :NRMINPRODGRPRAT, NRMAXPRODGRPRAT = :NRMAXPRODGRPRAT, NRMAXPRODGRPRATSEM = :NRMAXPRODGRPRATSEM WHERE NRITFREQPRODGRPRAT = :NRITFREQPRODGRPRAT",
            "UTILIZACAOPRATOSEMANA" => "UPDATE UTILPRATOSERV SET IDUTILPRATOSERVSEG = :IDUTILPRATOSERVSEG, IDUTILPRATOSERVTER = :IDUTILPRATOSERVTER, IDUTILPRATOSERVQUA = :IDUTILPRATOSERVQUA, IDUTILPRATOSERVQUI = :IDUTILPRATOSERVQUI, IDUTILPRATOSERVSEX = :IDUTILPRATOSERVSEX, IDUTILPRATOSERVSAB = :IDUTILPRATOSERVSAB, IDUTILPRATOSERVDOM = :IDUTILPRATOSERVDOM, IDUTILPRATOSERVPRIM = :IDUTILPRATOSERVPRIM, IDUTILPRATOSERVSEM2 = :IDUTILPRATOSERVSEM2, IDUTILPRATOSERVSEM3 = :IDUTILPRATOSERVSEM3, IDUTILPRATOSERVSEM4 = :IDUTILPRATOSERVSEM4, IDUTILPRATOSERVULT = :IDUTILPRATOSERVULT, IDUTILPRATOSERVFERI = :IDUTILPRATOSERVFERI WHERE NRUTILPRATOSERV = :NRUTILPRATOSERV",
            "CONSUMPTION_HISTORY_INTERVAL" => "UPDATE SERVICO SET NRMESHISTCONS = :NRMESHISTCONS WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI",
            "FREQUENCIAPRATO" => function($p){
                $this->updatePlateFrequency($p);
            },
            "PARAMPRAT" => function ($p, $CDOPERADOR) {
                $this->updateParamPrat($p, $CDOPERADOR);
            },
            'SERVCONFIG' => function ($params, $userCode) {
                $this->servConfService->updateServConfig($params, $userCode);
            },
            'SERVCONFIG_PERC_SOURCE' => function ($params) {
                $this->servConfService->updateServConfigPercentualSource($params);
            },
            'SERVCONFIG_FIXED_COLUMN' => function ($params) {
                $this->servConfService->updateServConfigFixedColumn($params);
            },
            'SERVCONFIG_NO_COLOR' => function ($params) {
                $this->servConfService->updateServConfigNoColor($params);
            },
            'SERVCONFIG_NO_PREPARATION_MODE' => function ($params) {
                $this->servConfService->updateServConfigNoPreparationMode($params);
            },
            'SERVCONFIG_INTERLEAVE_DISHES' => function ($params) {
                $this->servConfService->updateServConfigInterleaveDishes($params);
            },
            "RULES_TO_BE_FLEXIBLE" => "UPDATE OTMREGRAFLEX SET IDFLEXIVEL = :IDFLEXIVEL WHERE NROTMREGRAFLEX = :NROTMREGRAFLEX",
            "RULES_UPDATE_ORDER" => function($params){
                $this->updateOrderRule($params);
            },
            "RULES_UPDATE_RESTORE" => "UPDATE OTMREGRAFLEX SET IDFLEXIVEL = :IDFLEXIVEL, NRORDEMREGRA = NROTMREGRAFLEX",
            'MONTH_SERVCONFIG_PERC_SOURCE' => "UPDATE CONFCARDMESATIVO SET IDMESATIVO = :IDMESATIVO WHERE NRCONFCARDMESATIVO = :NRCONFCARDMESATIVO",
            "COMBGRUPOTPPRAT" => "UPDATE ITCOMBGRUPOTPPRAT SET PERCENTREDUCAO = :PERCENTREDUCAO WHERE NRITCOMBGRUPOTPPRAT = :NRITCOMBGRUPOTPPRAT",
            "VIGENCIAGRUPOPRATO" => "UPDATE VIGFREQGRPRATMES SET DESCRICAO = :DESCRICAO WHERE NRVIGENCIA = :NRVIGENCIA",
            "VIGENCIAPRATO" => "UPDATE VIGFREQPRATMES SET DESCRICAO = :DESCRICAO WHERE NRVIGENCIA = :NRVIGENCIA",
            "VIGENCIAGRUPOPRATOPRODUTO" => "UPDATE VIGFREQPRODGRPRAT SET DESCRICAO = :DESCRICAO WHERE NRVIGENCIA = :NRVIGENCIA",
            "VIGENCIAUTILIZACAOPRATOSEMANA" => "UPDATE VIGUTILPRATOSERV SET DESCRICAO = :DESCRICAO WHERE NRVIGENCIA = :NRVIGENCIA",
            "VIGENCIAPRODUTOGRUPOPRATORESTRICACAO" => "UPDATE VIGRESTPRODGR SET DESCRICAO = :DESCRICAO WHERE NRVIGENCIA = :NRVIGENCIA",
            "PADGRUPOPRATO" => function($p){
                $this->updatePlateGroupStandard($p);
            },
            "AGRUPGRPRAT" => "UPDATE AGRUPGRPRAT SET NMAGRUPGRPRAT = :NMAGRUPGRPRAT WHERE NRAGRUPGRPRAT = :NRAGRUPGRPRAT",
            'PLATE_GROUP' => function($params){
                $this->changePlateGroup($params);
            },
            'PARAMPRATGRUPPRAT' => function($params){
                return $this->updateGroupPlateinPlates($params);
            },
            'CONSUMPTION_HISTORY' => "UPDATE HISTCONSUMOPRATO SET VRPECONSCARD = :VRPECONSCARD WHERE NRHISTCONSUMOPRATO = :NRHISTCONSUMOPRATO",
            "PORTIONING_SET" => "UPDATE CONJUNTOPORCAO SET VRCONJUNTOPORCAO = :VRCONJUNTOPORCAO WHERE NRCONJUNTOPORCAO = :NRCONJUNTOPORCAO AND NRORG = :NRORG",

            'PORTIONING_SET_CALCULATE_CONSUMPTION_PERCENTAGE' => function($params){
                return $this->updatePortioningSetAutoConsumptionPercentage($params);
            },

            "INGREDIENTE_REGRAS_RESTRICAO_REPETICAO" => "UPDATE RESTRICAOINGRED SET IDINGREDIENTE1 = :IDINGREDIENTE1, IDINGREDIENTE2 = :IDINGREDIENTE2, IDMESMODIA = :IDMESMODIA, IDDIASEGUIDO = :IDDIASEGUIDO WHERE NRRESTRICAOINGRED = :NRRESTRICAOINGRED",
            "REGRAS_RESTRICAO_REPETICAO" => "UPDATE REGRARESTREPET SET NMREGRARESTREPET = :NMREGRARESTREPET WHERE NRREGRARESTREPET = :NRREGRARESTREPET",
            "DAYS_EVENT_OUT_OF_RUT" => "UPDATE CARDIAEVENTO SET QTCLIESTICAR = :QTCLIESTICAR, QTDOAESTICAR = :QTDOAESTICAR, QTFUNESTICAR = :QTFUNESTICAR WHERE NRCARDIAEVENTO = :NRCARDIAEVENTO",
            "UPDATE_CDGRUPPROD_PARAMGERAL" => "UPDATE PARAMGERAL SET CDGRUPPRODOTMCARD = (SELECT CDGRUPPROD FROM GRUPPROD WHERE CDGRUPPROD = :CDGRUPPROD)",
            
            "UPDATETYPECOST" =>
                "UPDATE ITGERACAOCARD SET CDTIPOCUSTO = :CDTIPOCUSTO WHERE NRITGERACAOCARD = :NRITGERACAOCARD",
            "UPDATEGRUPPROD" =>  function($params){
                $result = $this->updateGrupProd($params);
                if($result) return $result;
            },
            "UPDATELINKEDSCENARIO" => "UPDATE ITGERACAOCARD SET NRCENARIOVINCULO = :NRCENARIOVINCULO WHERE NRITGERACAOCARD = :NRITGERACAOCARD",
            "UPDATEDTREFERENCIA" => "UPDATE ITGERACAOCARD SET DTREFERENCIA = :DTREFERENCIA WHERE NRITGERACAOCARD = :NRITGERACAOCARD",
            "UPDATECUSTO" => function($params){
                $result = $this->updateCusto($params);
                if($result) return $result;
            },
            "UPDATEDIASCONG" => "UPDATE ITGERACAOCARD SET QTDIASCONG = :QTDIASCONG WHERE NRITGERACAOCARD = :NRITGERACAOCARD AND IDPERMITECONGELAR = 'S'",
            "UPDATECENARIOPUBLICAR" => "UPDATE ITGERACAOCARD SET NRCENARIOPUBLICAR = :NRCENARIOPUBLICAR WHERE NRITGERACAOCARD = :NRITGERACAOCARD",
            "UPDATESTATUSITGERACAOCARD" => "UPDATE ITGERACAOCARD SET IDSTATUSGERACAO = :IDSTATUSGERACAO, NRCENARIOPUBLICAR = :NRCENARIOPUBLICAR, NRLOGINTCARDAMES = :NRLOGINTCARDAMES WHERE NRITGERACAOCARD = :NRITGERACAOCARD",
            "UPDATE_STATUS_ITEMS" => function($params){
                $this->updateStatusItems($params);
            }
        );
        
        $from = $SQL[$param->get('FROM')];
        if (is_string($from)) {
            $paramReq = $param->get('PARAMS');
            $this->execInSql($paramReq, $from);
        } else if (is_callable($from)) {
            $paramReq = $param->get('PARAMS');
            return $from($paramReq, $CDOPERADOR);
        }
    }

    public function delete($param)
    {
        $SQL = array(
            'ATRATIVIDADE' => "DELETE FROM ATRATIVIDADEPRATO WHERE NRATRATIVIDADEPRATO = :NRATRATIVIDADEPRATO",
            "MDPREPARO" => "DELETE FROM FORMAPREPARO WHERE NRFORMAPREPARO = :NRFORMAPREPARO",
            "TPCORTE" => "DELETE FROM APRERECE WHERE CDAPRERECE = :CDAPRERECE",
            "INGRPRINC" => "DELETE FROM INGREDIENTEPRINC WHERE NRINGREDIENTEPRINC = :NRINGREDIENTEPRINC",
            "COMBRESTR" => "DELETI RUMBLEBUMBLE",
            "COMBGRUPOTPPRAT" => function ($p) {
                $this->deleteItCombGrupPrat($p);
            },
            "FREQUENCIAGRUPO" => function($p){
                $this->deleteItPlateGroupFrequency($p);
            },
            "FREQUENCIAPRATO" => function($p){
                $this->deleteItPlateFrequency($p);
            },
            "FREQUENCIAGRUPOPRATOPRODUTO" => function($p){
                $this->deleteItPlateGroupProductFrequency($p);
            },
            "UTILIZACAOPRATOSEMANA" => "DELETE FROM UTILPRATOSERV WHERE NRUTILPRATOSERV = :NRUTILPRATOSERV",
            "PRODUTOGRUPOPRATORESTRICACAO" => "DELETE FROM RESTPRODGR WHERE NRRESTPRODGR = :NRRESTPRODGR",
            "VIGENCIAPRODUTOGRUPOPRATORESTRICACAO" => function($p){
                return $this->deleteProductGroupPlateRestrictionValidity($p);
            },
            "VIGENCIAUTILIZACAOPRATOSEMANA" => function($p){
                return $this->deleteWeekdayUsageValidity($p);
            },
            "VIGENCIAGRUPOPRATOPRODUTO" => function($p){
                return $this->deletePlateGroupProductFrequencyValidity($p);
            },
            "VIGENCIAGRUPOPRATO" => function($p){
                return $this->deletePlateGroupValidity($p);
            },
            "VIGENCIAPRATO" => function($p){
                return $this->deletePlateValidity($p);
            },
            "PADGRUPOPRATO" => "DELETE FROM PADGRUPOPRATO WHERE CDPADGRUPOPRATO = :CDPADGRUPOPRATO",
            "AGRUPGRPRAT" => "DELETE FROM AGRUPGRPRAT WHERE NRAGRUPGRPRAT = :NRAGRUPGRPRAT",
            "GROUP_PLATE_PLT" => function($params){
                foreach($params['selectedRow'] as $row){
                    $this->delPlateRegister(["NRITGRUPOPRATOSERV"=>$row]);
                }
            },
            "PLATE_GROUP"=>function($params){
                $this->deletePlateGroup($params);
            },
            "CONSUMPTION_HISTORY" => function($p){
                $deleteConsumptionHistorySql = "DELETE FROM HISTCONSUMOPRATO WHERE NRHISTCONSUMOPRATO = :NRHISTCONSUMOPRATO";
                $params = $p[0];
                $this->execInSql(["NRHISTCONSUMOPRATO" => $params["NRHISTCONSUMOPRATO"]], $deleteConsumptionHistorySql);
                
                $findPreviousDataSql = "SELECT CP.CDFILAUXPRAT, CP.CDPRATO, ROUND(AVG(CP.VRPECONSCARD),2) AS VRPECONSCARD, 'N' AS MANUTENCAO FROM COMPCARD CP INNER JOIN CARDADIA C ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO WHERE C.CDFILIAL = :CDFILIAL AND C.CDSERVFILI = :CDSERVFILI AND :CDFILAUXPRAT = CP.CDFILAUXPRAT AND :CDPRATO = CP.CDPRATO and (C.DTMESCARD BETWEEN ADD_MONTHS(TO_DATE(:DTMESCARD),-6) AND TO_DATE(:DTMESCARD)) GROUP BY CP.CDFILAUXPRAT, CP.CDPRATO";
                unset($params["NRHISTCONSUMOPRATO"]);
                return $this->execOutSql($params, $findPreviousDataSql);
            },
            "ACCESS_PROFILE_PERMISSION" => "DELETE FROM OTIMIZADORITEMPER WHERE NROTIMIZADORITEMPER = :NROTIMIZADORITEMPER AND NRORG = :NRORG",
            "ACCESS_PROFILE_PERMISSION_BULK" => "DELETE FROM OTIMIZADORITEMPER WHERE INSTR(:NROTIMIZADORITENSSTR, NROTIMIZADORITEMPER) > 0 AND NRORG = :NRORG",
            "PORTIONING_SET" => "DELETE FROM CONJUNTOPORCAO WHERE NRCONJUNTOPORCAO = :NRCONJUNTOPORCAO",
            "DAYS_EVENT_OUT_OF_RUT" => "DELETE FROM CARDIAEVENTO WHERE NRCARDIAEVENTO = :NRCARDIAEVENTO",
            "LINK_TYPE_DISHES_BETWEEN_SERVICES" => "DELETE FROM TIPOPRATCOMUMSERV WHERE NRTIPOPRATCOMUMSERV = :NRTIPOPRATCOMUMSERV",
            "TIPO_PRATO_REGRAS_RESTRICAO_REPETICAO" => "DELETE FROM RESTRICAOTPPRAT WHERE NRRESTRICAOTPPRAT = :NRRESTRICAOTPPRAT",
            "INGREDIENTE_REGRAS_RESTRICAO_REPETICAO" => "DELETE FROM RESTRICAOINGRED WHERE NRRESTRICAOINGRED = :NRRESTRICAOINGRED",
            "REGRAS_RESTRICAO_REPETICAO" => "DELETE FROM REGRARESTREPET WHERE NRREGRARESTREPET = :NRREGRARESTREPET",
            "DELETE_DAYS_OUT_RUT_OVERLAP" => "DELETE FROM CARDIAEVENTO WHERE CDFILIAL||CDSERVFILI IN (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :FILISERV)"
        );

        $from = $SQL[$param->get('FROM')];
        if (is_string($from)) {
            $paramReq = $param->get('PARAMS');
            $this->execInSql($paramReq, $from);
        } else if (is_callable($from)) {
            $paramReq = $param->get('PARAMS');
            return $from($paramReq);
        }
    }
    
     /**
     * Prepara retorno de mensagem com ou sem Parâmetro para o Front.
     *
     * @param $msgCode  - Se refere ao código da mensagem (Cadastrado no front em messages.json).
     * @param $params (Não Obrigatório) - Parâmetro do corpo da mensagem, pode ser somente um valor, pode vim como um array de valores, ou não ser informado quando a mensagem não possui parâmetros.
     */
    
    public function formatMsgTradu($msgCode, $params = null){
        
        if(empty($params)){
            
            return serialize($msgCode);
            
        }else if(is_array($params)){
            
            $result = [
                'msg_code' => $msgCode,
                'msg_params' => [],
            ];
            
            foreach($params as $msgParam){
                
                array_push($result['msg_params'],$msgParam);
            }
            
        }else if(!is_array($params)){
            
            $result = [
                'msg_code' => $msgCode,
                'msg_params' => $params,
            ];
            
        }
        
        return serialize($result);
        
    }
    
     /**
     * Realiza o retorno da estrutura original da mensagem executando o unserialize(), se receber uma string sem estar selarizada, retorna a mesma.
     *
     * @param $msg  - Código da mensagem que se encontra no messages.json.
     */
    
    public function preparaEnvioMsgTraducao($msg){
        
        try{
            
            $result = unserialize($msg);
            return $result;
            
        }catch(\Exception $e){
            return $msg;
        }
        
    }

}
