<?php

namespace BackOfficeOptimizer\Services;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\DBAL\Exception\ForeignKeyConstraintViolationException;
use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
use BackOfficeOptimizer\Exceptions\BackOfficeException;
use BackOfficeOptimizer\Libraries\PermissionManager;
use Zeedhi\Framework\DTO\Response\Error as ZhError;

class DataManager
{
    protected $dataProvider;

    public function __construct($dataProvider)
    {
        $this->dataProvider = $dataProvider;
    }

    private function getParamsWithCDOPERADOR($request){
        $row = $request->getRow();
        $params = $row["PARAMS"];
        
        $CDOPERADOR = $request->getParameters()["currentUserCode"];
        
        if(array_key_exists('0', $params)){
            for ($i=0; $i < count($params); $i++) { 
                if(!isset($params[$i]["CDOPERADOR"])){
                    $params[$i]["CDOPERADOR"] = $CDOPERADOR;
                }
            }
        } else if(!isset($params["CDOPERADOR"])) {
            $params["CDOPERADOR"] = $CDOPERADOR;
        }
        return $params;
    }

    public function getData(Request $request, Response $response)
    {
        try{
            $param = $request->getRow();
            $route = $param["FROM"];
            $token = $request->getParameters()['$token'];
            $perm = PermissionManager::userHasPermissionTo("find", $route, $token);
            if(!$perm){
                $msg = $this->dataProvider->formatMsgTradu("user_without_permission_for_this_operation");
                $msgFormatada = $this->dataProvider->preparaEnvioMsgTraducao($msg);
                throw new BackOfficeException($msgFormatada);
            }
            $dataReturn = $this->dataProvider->find($param);
            $response->addDataSet(new DataSet('data', $dataReturn));
        } catch (BackOfficeException $backofficeEx){
            
            $msg = $this->dataProvider->preparaEnvioMsgTraducao($backofficeEx->getMessage());
            $response->setError(new ZhError($msg, 3));
        }
    }

    public function insertData(Request $request, Response $response)
    {
        try{
            $param = $request->getRow();
            $route = $param["FROM"];
            $token = $request->getParameters()['$token'];
            $perm = PermissionManager::userHasPermissionTo("insert", $route, $token);
            if(!$perm){
                $msg = $this->dataProvider->formatMsgTradu("user_without_permission_for_this_operation");
                $msgFormatada = $this->dataProvider->preparaEnvioMsgTraducao($msg);
                throw new BackOfficeException($msgFormatada);
            }
            $rowParams = $this->getParamsWithCDOPERADOR($request);
            $param["PARAMS"] = $rowParams;
            $retorno = $this->dataProvider->save($param);
            if($retorno){
                $response->addDataSet(new DataSet('data', $retorno));
            }
        }catch(UniqueConstraintViolationException $fkException) {
            
            $msg = $this->dataProvider->formatMsgTradu("record_with_this_code_already_exists");
            $msgFormatada = $this->dataProvider->preparaEnvioMsgTraducao($msg);
            $response->setError(new ZhError($msgFormatada, 1));
        } catch (BackOfficeException $backofficeEx){
            
            $msg = $this->dataProvider->preparaEnvioMsgTraducao($backofficeEx->getMessage());
            $response->setError(new ZhError($msg, 3));
       
        } catch(\Exception $e){
            throw $e;
        }
    }

    public function updateData(Request $request, Response $response)
    {
        try{
            $param = $request->getRow();
            $CDOPERADOR = $request->getParameters()["currentUserCode"];
            $route = $param["FROM"];
            $token = $request->getParameters()['$token'];
            $perm = PermissionManager::userHasPermissionTo("update", $route, $token);
            if(!$perm){
                $msg = $this->dataProvider->formatMsgTradu("user_without_permission_for_this_operation");
                $msgFormatada = $this->dataProvider->preparaEnvioMsgTraducao($msg);
                throw new BackOfficeException($msgFormatada);
            }
            $ret = $this->dataProvider->update($param, $CDOPERADOR);
            if($ret){
                $response->addDataSet(new DataSet('data', $ret));
            }
        } catch (BackOfficeException $backofficeEx){
            $msg = $this->dataProvider->preparaEnvioMsgTraducao($backofficeEx->getMessage());
            $response->setError(new ZhError($msg, 3));
        } catch(\Exception $e){
            throw $e;
        }
    }

    public function deleteData(Request $request, Response $response)
    {
        try{
            $param = $request->getRow();
            $route = $param["FROM"];
            $token = $request->getParameters()['$token'];
            $perm = PermissionManager::userHasPermissionTo("delete", $route, $token);
            if(!$perm){
                $msg = $this->dataProvider->formatMsgTradu("user_without_permission_for_this_operation");
                $msgFormatada = $this->dataProvider->preparaEnvioMsgTraducao($msg);
                throw new BackOfficeException($msgFormatada);
            }
            $ret = $this->dataProvider->delete($param);
            if($ret){
                $response->addDataSet(new DataSet('data', $ret));
            }
        }catch(ForeignKeyConstraintViolationException $fkException) {
            
            $msg = $this->dataProvider->formatMsgTradu("there_are_dependent_records");
            $msgFormatada = $this->dataProvider->preparaEnvioMsgTraducao($msg);
            $response->setError(new ZhError($msgFormatada, 2));
            
        } catch (BackOfficeException $backofficeEx){
            $msg = $this->dataProvider->preparaEnvioMsgTraducao($backofficeEx->getMessage());
            $response->setError(new ZhError($msg, 3));
        } catch(\Exception $e){
            throw $e;
        }
    }
    
}